@@ -6,12 +6,16 @@
 #include <wraster.h>
 #include <X11/Xlib.h>
 
-#define WINGS_H_VERSION  980722
+#define WINGS_H_VERSION  980730
+
+
+
+
+
 
 typedef unsigned long WMPixel;
 
 
-
 typedef struct {
     unsigned int width;
     unsigned int height;
@@ -124,12 +128,6 @@ typedef enum {
 	WSUAllParts
 } WMUsableScrollerParts;
 
-/* orientations for scroller and maybe other stuff */
-typedef enum {
-    WOVertical,
-	WOHorizontal
-} WMOrientation;
-
 /* matrix types */
 typedef enum {
     WMMRadioMode,
@@ -139,6 +137,29 @@ typedef enum {
 } WMMatrixTypes;
 
 
+/* text movement types */
+enum {
+    WMIllegalTextMovement,
+	WMReturnTextMovement,
+	WMTabTextMovement,
+	WMBacktabTextMovement
+};
+
+
+/* system images */
+#define WSIReturnArrow			1
+#define WSIHighlightedReturnArrow	2
+#define WSIScrollerDimple		3
+#define WSIArrowLeft			4
+#define WSIHighlightedArrowLeft	        5
+#define WSIArrowRight			6
+#define WSIHighlightedArrowRight	7
+#define WSIArrowUp			8
+#define WSIHighlightedArrowUp		9
+#define WSIArrowDown			10
+#define WSIHighlightedArrowDown		11
+
+
 enum {
     WLDSSelected = (1 << 16),
 	WLDSDisabled = (1 << 17),
@@ -156,21 +177,30 @@ enum {
 };
 
 
+
+
+
+
+
+
 typedef int W_Class;
 
-
-#define WC_Window 	0
-#define WC_Frame 	1
-#define WC_Label 	2
-#define WC_Button	3
-#define WC_TextField	4
-#define WC_Scroller	5
-#define WC_ScrollView 	6
-#define WC_List		7
-#define WC_Browser 	8
-#define WC_PopUpButton	9
-#define WC_ColorWell	10
-
+enum {
+    WC_Window = 0,
+    WC_Frame = 1,
+    WC_Label = 2,
+    WC_Button = 3,
+    WC_TextField = 4,
+    WC_Scroller	= 5,
+    WC_ScrollView = 6,
+    WC_List = 7,
+    WC_Browser = 8,
+    WC_PopUpButton = 9,
+    WC_ColorWell = 10,
+    WC_Slider = 11,
+    WC_Matrix = 12,		       /* not ready */
+    WC_SplitView = 13
+};
 
 /* All widgets must start with the following structure
  * in that order. Used for typecasting to get some generic data */
@@ -185,12 +215,6 @@ typedef struct W_WidgetType {
 #define WMWidgetView(widget)   	(((W_WidgetType*)(widget))->view)
 
 
-typedef struct W_PanelType {
-    struct WMWindow *win;
-} W_PanelType;
-
-#define W_WINDOW(panel)		(((W_PanelType*)(panel))->win)
-
 /* widgets */
 
 typedef void WMWidget;
@@ -214,18 +238,22 @@ typedef struct W_List WMList;
 typedef struct W_Browser WMBrowser;
 typedef struct W_PopUpButton WMPopUpButton;
 typedef struct W_ColorWell WMColorWell;
-
+typedef struct W_Slider WMSlider;
+typedef struct W_Matrix WMMatrix;      /* not ready */
+typedef struct W_SplitView WMSplitView;
 
 /* not widgets */
 typedef struct W_FilePanel WMFilePanel;
 typedef WMFilePanel WMOpenPanel;
 typedef WMFilePanel WMSavePanel;
 
+typedef struct W_FontPanel WMFontPanel;
+
 
 /* item for WMList */
 typedef struct WMListItem {
     char *text;
-    
+        
     struct WMListItem *nextPtr;
     
     unsigned int uflags:16;	       /* flags for the user */
@@ -277,37 +305,54 @@ typedef void WMEventHook(XEvent *event);
 typedef void WMAction(WMWidget *self, void *clientData);
 
 
+typedef void WMCallback(void *data);
+
+
+/* delegate method like stuff */
 typedef void WMFreeDataProc(void *data);
 
 typedef void WMListDrawProc(WMList *lPtr, Drawable d, char *text, int state,
 			    WMRect *rect);
 
-typedef void WMBrowserFillColumn(WMBrowser *bPtr, int column);
+/*
+typedef void WMSplitViewResizeSubviewsProc(WMSplitView *sPtr,
+					   unsigned int oldWidth,
+					   unsigned int oldHeight);
+*/
 
-typedef void WMCallback(void *data);
+typedef void WMSplitViewConstrainProc(WMSplitView *sPtr, int dividerIndex,
+				      int *minCoordinate, int *maxCoordinate);
+
+typedef WMWidget *WMMatrixCreateCellProc(WMMatrix *mPtr);
+
+
+typedef void WMBrowserFillColumnProc(WMBrowser *bPtr, int column);
 
 
 /* ....................................................................... */
 
+void WMInitializeApplication(char *applicationName, int *argc, char **argv);
 
+void WMSetApplicationDataPath(char *path);
+
+/* don't free the returned string */
+char *WMGetApplicationName();
+
+/* Try to locate resource file. ext may be NULL */
+char *WMPathForResourceOfType(char *resource, char *ext);
 
 WMScreen *WMCreateScreenWithRContext(Display *display, int screen, 
-				     RContext *context, char *applicationName,
-				     int *argc, char **argv);
+				     RContext *context);
 
-WMScreen *WMCreateScreen(Display *display, int screen, char *applicationName,
-			 int *argc, char **argv);
+WMScreen *WMCreateScreen(Display *display, int screen);
 
-WMScreen *WMCreateSimpleApplicationScreen(Display *display, 
-					  char *applicationName,
-					  int *argc, char **argv);
+WMScreen *WMCreateSimpleApplicationScreen(Display *display);
 
 void WMScreenMainLoop(WMScreen *scr);
 
 
 RContext *WMScreenRContext(WMScreen *scr);
 
-
 void WMSetApplicationIconImage(WMScreen *app, WMPixmap *icon);
 
 WMPixmap *WMGetApplicationIconImage(WMScreen *app);
@@ -318,6 +363,8 @@ WMEventHook *WMHookEventHandler(WMEventHook *handler);
 
 int WMHandleEvent(XEvent *event);
 
+Bool WMScreenPending(WMScreen *scr);
+
 void WMCreateEventHandler(WMView *view, unsigned long mask,
 			  WMEventProc *eventProc, void *clientData);
 
@@ -341,14 +388,19 @@ WMHandlerID WMAddIdleHandler(WMCallback *callback, void *cdata);
 
 void WMDeleteIdleHandler(WMHandlerID handlerID);
 
+
 /* ....................................................................... */
 
-WMFont *WMCreateFont(WMScreen *scrPtr, char *font_name);
+WMFont *WMCreateFont(WMScreen *scrPtr, char *fontName);
+
+WMFont *WMCreateFontInDefaultEncoding(WMScreen *scrPtr, char *fontName);
 
 WMFont *WMRetainFont(WMFont *font);
 
 void WMReleaseFont(WMFont *font);
 
+unsigned int WMFontHeight(WMFont *font);
+
 /*
 WMFont *WMUserFontOfSize(WMScreen *scrPtr, int size);
 
@@ -359,6 +411,7 @@ WMFont *WMSystemFontOfSize(WMScreen *scrPtr, int size);
 
 WMFont *WMBoldSystemFontOfSize(WMScreen *scrPtr, int size);
 
+XFontSet WMGetFontFontSet(WMFont *font);
 
 /* ....................................................................... */
 
@@ -373,12 +426,19 @@ WMPixmap *WMCreatePixmapFromXPixmaps(WMScreen *scrPtr, Pixmap pixmap,
 WMPixmap *WMCreatePixmapFromRImage(WMScreen *scrPtr, RImage *image, 
 				   int threshold);
 
+WMPixmap *WMCreatePixmapFromXPMData(WMScreen *scrPtr, char **data);
+
 WMSize WMGetPixmapSize(WMPixmap *pixmap);
 
 WMPixmap *WMCreatePixmapFromFile(WMScreen *scrPtr, char *fileName);
 
 WMPixmap *WMCreateBlendedPixmapFromFile(WMScreen *scrPtr, char *fileName,
 					RColor *color);
+
+Pixmap WMGetPixmapXID(WMPixmap *pixmap);
+
+WMPixmap *WMGetSystemPixmap(WMScreen *scr, int image);
+
 /* ....................................................................... */
 
 
@@ -390,14 +450,16 @@ WMColor *WMBlackColor(WMScreen *scr);
 
 WMColor *WMWhiteColor(WMScreen *scr);
 
+void WMSetColorInGC(WMColor *color, GC gc);
+
 GC WMColorGC(WMColor *color);
 
 WMPixel WMColorPixel(WMColor *color);
 
-void WMPaintColorRectangle(WMScreen *scr, WMColor *color, Drawable d, int x, 
-			   int y, unsigned int width, unsigned int height);
+void WMPaintColorSwatch(WMColor *color, Drawable d, int x, int y, 
+			unsigned int width, unsigned int height);
 
-void WMReleaseColor(WMScreen *scr, WMColor *color);
+void WMReleaseColor(WMColor *color);
 
 WMColor *WMRetainColor(WMColor *color);
 
@@ -405,11 +467,35 @@ WMColor *WMCreateRGBColor(WMScreen *scr, unsigned short red,
 			  unsigned short green, unsigned short blue,
 			  Bool exact);
 
+WMColor *WMCreateNamedColor(WMScreen *scr, char *name, Bool exact);
+
+unsigned short WMRedComponentOfColor(WMColor *color);
+
+unsigned short WMGreenComponentOfColor(WMColor *color);
+
+unsigned short WMBlueComponentOfColor(WMColor *color);
+
+char *WMGetColorRGBDescription(WMColor *color);
+
+/* ....................................................................... */
+
+
+void WMDrawString(WMScreen *scr, Drawable d, GC gc, WMFont *font, int x, 
+		  int y, char *text, int length);
+
+void WMDrawImageString(WMScreen *scr, Drawable d, GC gc, WMFont *font, int x, 
+		       int y, char *text, int length);
+
+int WMWidthOfString(WMFont *font, char *text, int length);
+
+
 
 /* ....................................................................... */
 
 WMScreen *WMWidgetScreen(WMWidget *w);
 
+Display *WMScreenDisplay(WMScreen *scr);
+
 void WMUnmapWidget(WMWidget *w);
 
 void WMMapWidget(WMWidget *w);
@@ -430,8 +516,6 @@ void WMHangData(WMWidget *widget, void *data);
 
 void *WMGetHangedData(WMWidget *widget);
 
-void WMAddDestroyCallback(WMWidget *w, WMCallback *callback, void *clientData);
-
 unsigned int WMWidgetWidth(WMWidget *w);
 
 unsigned int WMWidgetHeight(WMWidget *w);
@@ -439,22 +523,42 @@ unsigned int WMWidgetHeight(WMWidget *w);
 Window WMWidgetXID(WMWidget *w);
 
 
+void WMRedisplayWidget(WMWidget *w);
+
+void WMSetViewNotifySizeChanges(WMView *view, Bool flag);
+
+WMSize WMGetViewSize(WMView *view);
+
+WMPoint WMGetViewPosition(WMView *view);
+
+WMWidget *WMWidgetOfView(WMView *view);
+
+/* notifications */
+extern char *WMViewSizeDidChangeNotification;
+
+extern char *WMViewRealizedNotification;
+
 /* ....................................................................... */
 
 WMWindow *WMCreateWindow(WMScreen *screen, char *name);
 
 void WMSetWindowTitle(WMWindow *wPtr, char *title);
 
-WMScreen *WMWindowApplication(WMWindow *win);
-
 void WMSetWindowCloseAction(WMWindow *win, WMAction *action, void *clientData);
 
+void WMSetWindowMaxSize(WMWindow *win, unsigned width, unsigned height);
+
+void WMSetWindowMinSize(WMWindow *win, unsigned width, unsigned height);
+
 /* ....................................................................... */
 
 void WMSetButtonAction(WMButton *bPtr, WMAction *action, void *clientData);
 
 #define WMCreateCommandButton(parent) \
-	WMCreateButton((parent), WBTMomentaryPush)
+	WMCreateCustomButton((parent), WBBSpringLoadedMask\
+					|WBBPushInMask\
+					|WBBPushLightMask\
+					|WBBPushChangeMask)
 
 #define WMCreateRadioButton(parent) \
 	WMCreateButton((parent), WBTRadio)
@@ -472,6 +576,8 @@ void WMSetButtonAltImage(WMButton *bPtr, WMPixmap *image);
 
 void WMSetButtonImagePosition(WMButton *bPtr, WMImagePosition position);
 
+void WMSetButtonFont(WMButton *bPtr, WMFont *font);
+
 void WMSetButtonTextAlignment(WMButton *bPtr, WMAlignment alignment);
 
 void WMSetButtonText(WMButton *bPtr, char *text);
@@ -484,7 +590,7 @@ int WMGetButtonSelected(WMButton *bPtr);
 
 void WMSetButtonBordered(WMButton *bPtr, int isBordered);
 
-void WMSetButtonDisabled(WMButton *bPtr, int isDisabled);
+void WMSetButtonEnabled(WMButton *bPtr, Bool flag);
 
 void WMSetButtonTag(WMButton *bPtr, int tag);
 
@@ -492,6 +598,10 @@ void WMGroupButtons(WMButton *bPtr, WMButton *newMember);
 
 void WMPerformButtonClick(WMButton *bPtr);
 
+void WMSetButtonContinuous(WMButton *bPtr, Bool flag);
+
+void WMSetButtonPeriodicDelay(WMButton *bPtr, float delay, float interval);
+
 /* ....................................................................... */
 
 WMLabel *WMCreateLabel(WMWidget *parent);
@@ -528,6 +638,7 @@ void WMInsertTextFieldText(WMTextField *tPtr, char *text, int position);
 
 void WMDeleteTextFieldRange(WMTextField *tPtr, int position, int count);
 
+/* you can free the returned string */
 char *WMGetTextFieldText(WMTextField *tPtr);
 
 void WMSetTextFieldText(WMTextField *tPtr, char *text);
@@ -536,7 +647,14 @@ void WMSetTextFieldAlignment(WMTextField *tPtr, WMAlignment alignment);
 
 void WMSetTextFieldBordered(WMTextField *tPtr, Bool bordered);
 
-void WMSetTextFieldDisabled(WMTextField *tPtr, Bool disabled);
+void WMSetTextFieldEnabled(WMTextField *tPtr, Bool flag);
+
+void WMSetTextFieldSecure(WMTextField *tPtr, Bool flag);
+
+
+extern char *WMTextDidChangeNotification;
+extern char *WMTextDidBeginEditingNotification;
+extern char *WMTextDidEndEditingNotification;
 
 /* ....................................................................... */
 
@@ -547,7 +665,7 @@ void WMSetScrollerParameters(WMScroller *sPtr, float floatValue,
 
 float WMGetScrollerKnobProportion(WMScroller *sPtr);
 
-float WMGetScrollerFloatValue(WMScroller *sPtr);
+float WMGetScrollerValue(WMScroller *sPtr);
 
 WMScrollerPart WMGetScrollerHitPart(WMScroller *sPtr);
 
@@ -562,25 +680,26 @@ WMList *WMCreateList(WMWidget *parent);
 
 #define WMAddListItem(lPtr, text) WMInsertListItem((lPtr), -1, (text))
 
-WMListItem *WMInsertListItem(WMList *lPtr, int index, char *text);
+WMListItem *WMInsertListItem(WMList *lPtr, int row, char *text);
 
 WMListItem *WMAddSortedListItem(WMList *lPtr, char *text);
 
-char *WMGetListItem(WMList *lPtr, int index);
+int WMFindRowOfListItemWithTitle(WMList *lPtr, char *title);
 
-void WMRemoveListItem(WMList *lPtr, int index);
+WMListItem *WMGetListItem(WMList *lPtr, int row);
+
+void WMRemoveListItem(WMList *lPtr, int row);
 
 void WMSelectListItem(WMList *lPtr, int row);
 
 void WMSetListUserDrawProc(WMList *lPtr, WMListDrawProc *proc);
 
-void WMSetListItems(WMList *lPtr, WMListItem *items);
-
-WMListItem *WMGetListItems(WMList *lPtr);
+void WMSetListUserDrawItemHeight(WMList *lPtr, unsigned short height);
 
+/* don't free the returned data */
 WMListItem *WMGetListSelectedItem(WMList *lPtr);
 
-int WMGetListSelectedItemIndex(WMList *lPtr);
+int WMGetListSelectedItemRow(WMList *lPtr);
 
 void WMSetListAction(WMList *lPtr, WMAction *action, void *clientData);
 
@@ -588,30 +707,59 @@ void WMSetListDoubleAction(WMList *lPtr, WMAction *action, void *clientData);
 
 void WMClearList(WMList *lPtr);
 
+int WMGetListNumberOfRows(WMList *lPtr);
+
+void WMSetListPosition(WMList *lPtr, int row);
+
+void WMSetListBottomPosition(WMList *lPtr, int row);
+
+int WMGetListPosition(WMList *lPtr);
+
 /* ....................................................................... */
 
 WMBrowser *WMCreateBrowser(WMWidget *parent);
 
+void WMSetBrowserPathSeparator(WMBrowser *bPtr, char *separator);
+
+void WMSetBrowserTitled(WMBrowser *bPtr, Bool flag);
+
 void WMLoadBrowserColumnZero(WMBrowser *bPtr);
 
 int WMAddBrowserColumn(WMBrowser *bPtr);
 
+void WMRemoveBrowserItem(WMBrowser *bPtr, int column, int row);
+
 void WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, char *title);
 
-Bool WMAddBrowserItem(WMBrowser *bPtr, int column, char *text, Bool isBranch);
+WMListItem *WMAddSortedBrowserItem(WMBrowser *bPtr, int column, char *text, Bool isBranch);
+
+WMListItem *WMInsertBrowserItem(WMBrowser *bPtr, int column, int row, char *text, Bool isBranch);
 
 Bool WMSetBrowserPath(WMBrowser *bPtr, char *path);
 
+/* you can free the returned string */
 char *WMGetBrowserPath(WMBrowser *bPtr);
-
+/* you can free the returned string */
 char *WMGetBrowserPathToColumn(WMBrowser *bPtr, int column);
 
-void WMSetBrowserFillColumnProc(WMBrowser *bPtr, WMBrowserFillColumn *proc);
+void WMSetBrowserFillColumnProc(WMBrowser *bPtr,WMBrowserFillColumnProc *proc);
 
 void WMSetBrowserAction(WMBrowser *bPtr, WMAction *action, void *clientData);
 
 WMListItem *WMGetBrowserSelectedItemInColumn(WMBrowser *bPtr, int column);
 
+int WMGetBrowserFirstVisibleColumn(WMBrowser *bPtr);
+
+int WMGetBrowserSelectedColumn(WMBrowser *bPtr);
+
+int WMGetBrowserSelectedRowInColumn(WMBrowser *bPtr, int column);
+
+int WMGetBrowserNumberOfColumns(WMBrowser *bPtr);
+
+WMList *WMGetBrowserListInColumn(WMBrowser *bPtr, int column);
+
+extern char *WMBrowserDidScrollNotification;
+
 /* ....................................................................... */
 
 WMPopUpButton *WMCreatePopUpButton(WMWidget *parent);
@@ -619,6 +767,8 @@ WMPopUpButton *WMCreatePopUpButton(WMWidget *parent);
 void WMSetPopUpButtonAction(WMPopUpButton *sPtr, WMAction *action, 
 			    void *clientData);
 
+void WMSetPopUpButtonPullsDown(WMPopUpButton *bPtr, Bool flag);
+
 void WMAddPopUpButtonItem(WMPopUpButton *bPtr, char *title);
 
 void WMInsertPopUpButtonItem(WMPopUpButton *bPtr, int index, char *title);
@@ -631,6 +781,13 @@ void WMSetPopUpButtonSelectedItem(WMPopUpButton *bPtr, int index);
 
 int WMGetPopUpButtonSelectedItem(WMPopUpButton *bPtr);
 
+void WMSetPopUpButtonText(WMPopUpButton *bPtr, char *text);
+
+/* don't free the returned data */
+char *WMGetPopUpButtonItem(WMPopUpButton *bPtr, int index);
+
+int WMGetPopUpButtonNumberOfItems(WMPopUpButton *bPtr);
+
 /* ....................................................................... */
 
 WMColorWell *WMCreateColorWell(WMWidget *parent);
@@ -639,7 +796,6 @@ void WMSetColorWellColor(WMColorWell *cPtr, WMColor *color);
 
 WMColor *WMGetColorWellColor(WMColorWell *cPtr);
 
-
 /* ...................................................................... */
 
 WMScrollView *WMCreateScrollView(WMWidget *parent);
@@ -657,12 +813,57 @@ void WMSetScrollViewRelief(WMScrollView *sPtr, WMReliefType type);
 
 void WMSetScrollViewContentView(WMScrollView *sPtr, WMView *view);
 
+WMScroller *WMGetScrollViewHorizontalScroller(WMScrollView *sPtr);
+
+WMScroller *WMGetScrollViewVerticalScroller(WMScrollView *sPtr);
+
+/* ....................................................................... */
+
+WMSlider *WMCreateSlider(WMWidget *parent);
+
+int WMGetSliderMinValue(WMSlider *slider);
+
+int WMGetSliderMaxValue(WMSlider *slider);
+
+int WMGetSliderValue(WMSlider *slider);
+
+void WMSetSliderMinValue(WMSlider *slider, int value);
+
+void WMSetSliderMaxValue(WMSlider *slider, int value);
+
+void WMSetSliderValue(WMSlider *slider, int value);
+
+void WMSetSliderContinuous(WMSlider *slider, Bool flag);
+
+void WMSetSliderAction(WMSlider *slider, WMAction *action, void *data);
+
+/* ....................................................................... */
+
+/* only supports 2 subviews */
+WMSplitView *WMCreateSplitView(WMWidget *parent);
+
+void WMAddSplitViewSubview(WMSplitView *sPtr, WMView *subview);
+
+void WMAdjustSplitViewSubviews(WMSplitView *sPtr);
+
+void WMSetSplitViewConstrainProc(WMSplitView *sPtr, 
+				 WMSplitViewConstrainProc *proc);
+
+/*
+void WMSetSplitViewResizeSubviewsProc(WMSplitView *sPtr, 
+				      WMSplitViewResizeSubviewsProc *proc);
+*/
+
+int WMGetSplitViewDividerThickness(WMSplitView *sPtr);
+
+
 /* ....................................................................... */
 
 int WMRunAlertPanel(WMScreen *app, char *title, char *msg, 
 		    char *defaultButton, char *alternateButton, 
 		    char *otherButton);
 
+/* you can free the returned string */
 char *WMRunInputPanel(WMScreen *app, char *title, char *msg, 
 		      char *defaultText, char *okButton, char *cancelButton);
 
@@ -678,7 +879,10 @@ void WMDestroyAlertPanel(WMAlertPanel *panel);
 
 void WMDestroyInputPanel(WMInputPanel *panel);
 
-WMOpenPanel *WMCreateOpenPanel(WMScreen *app);
+/* ....................................................................... */
+
+/* only 1 instance per WMScreen */
+WMOpenPanel *WMGetOpenPanel(WMScreen *app);
 
 void WMSetFilePanelCanChooseDirectories(WMFilePanel *panel, int flag);
 
@@ -686,12 +890,29 @@ void WMSetFilePanelCanChooseFiles(WMFilePanel *panel, int flag);
 
 void WMSetFilePanelDirectory(WMFilePanel *panel, char *path);
 
+/* you can free the returned string */
 char *WMGetFilePanelFile(WMFilePanel *panel);
 
-void WMDestroyFilePanel(WMFilePanel *panel);
+void WMFreeFilePanel(WMFilePanel *panel);
 
 int WMRunModalOpenPanelForDirectory(WMFilePanel *panel, char *path, char *name,
 				    char **fileTypes);
 
+/* ...................................................................... */
+
+/* only 1 instance per WMScreen */
+WMFontPanel *WMGetFontPanel(WMScreen *scr);
+
+void WMShowFontPanel(WMFontPanel *panel);
+
+void WMHideFontPanel(WMFontPanel *panel);
+
+void WMSetFontPanelFont(WMFontPanel *panel, WMFont *font);
+
+/* you can free the returned string */
+char *WMGetFontPanelFontName(WMFontPanel *panel);
+
+WMFont *WMGetFontPanelFont(WMFontPanel *panel);
 
 #endif
+
