@@ -14,7 +14,7 @@
 #if defined(HAVE_SELECT)
  
 void
-usleep(unsigned int microsecs)
+wusleep(unsigned int microsecs)
 {
     struct timeval tv;
     fd_set rd, wr, ex;
@@ -31,7 +31,7 @@ usleep(unsigned int microsecs)
 # ifdef HAVE_POLL 
 
 void 
-usleep(unsigned int microsecs)
+wusleep(unsigned int microsecs)
 {
     poll((struct poll *) 0, (size_t) 0, microsecs/1000);
 }
