@@ -5,31 +5,26 @@
 #include "WINGsP.h"
 
 
-typedef struct ColumnList {
-    char *title;
-    char *selectedText;		       /* can't be freed */
-    WMListItem *column;
-    int topRow;
-    struct ColumnList *nextPtr;
-} ColumnList;
+char *WMBrowserDidScrollNotification = "WMBrowserDidScrollNotification";
 
 
 typedef struct W_Browser {
     W_Class widgetClass;
     W_View *view;
 
-    WMList **visibleColumns;
-    WMLabel **visibleTitles;
-    
-    ColumnList *columns;
+    char **titles;
+    WMList **columns;
+
     short columnCount;
-    short visibleColumnCount;
+    short usedColumnCount;	       /* columns actually being used */
     short minColumnWidth;
  
     short maxVisibleColumns;
     short firstVisibleColumn;
 
     short titleHeight;
+    
+    short selectedColumn;
 
     WMSize columnSize;
     
@@ -39,7 +34,7 @@ typedef struct W_Browser {
     void *doubleClientData;
     WMAction *doubleAction;
 
-    WMBrowserFillColumn *fillColumn;
+    WMBrowserFillColumnProc *fillColumn;
     
     WMScroller *scroller;
 
@@ -51,8 +46,6 @@ typedef struct W_Browser {
 	unsigned int hasScroller:1;
 	
 	/* */
-	
-	unsigned int redrawPending:1;
 	unsigned int loaded:1;
 	unsigned int loadingColumn:1;
     } flags;
@@ -61,7 +54,6 @@ typedef struct W_Browser {
 
 #define COLUMN_SPACING 	4
 #define TITLE_SPACING 2
-#define SCROLLER_SPACING 2
 
 #define DEFAULT_WIDTH		305
 #define DEFAULT_HEIGHT		200
@@ -80,17 +72,17 @@ static void setupScroller(WMBrowser *bPtr);
 
 static void scrollToColumn(WMBrowser *bPtr, int column);
 
-static void setupVisibleColumns(WMBrowser *bPtr);
-
 static void paintItem(WMList *lPtr, Drawable d, char *text, int state, 
 		      WMRect *rect);
 
 static void loadColumn(WMBrowser *bPtr, int column);
 
+
+static void resizeBrowser(WMWidget*, unsigned int, unsigned int);
+
 W_ViewProcedureTable _BrowserViewProcedures = {
     NULL,
-	NULL,
-	NULL,
+	resizeBrowser,
 	NULL
 };
 
@@ -100,6 +92,7 @@ WMBrowser*
 WMCreateBrowser(WMWidget *parent)
 {
     WMBrowser *bPtr;
+    int i;
 
     bPtr = wmalloc(sizeof(WMBrowser));
     memset(bPtr, 0, sizeof(WMBrowser));
@@ -111,201 +104,316 @@ WMCreateBrowser(WMWidget *parent)
 	free(bPtr);
 	return NULL;
     }
+    bPtr->view->self = bPtr;
 
     WMCreateEventHandler(bPtr->view, ExposureMask|StructureNotifyMask
 			 |ClientMessageMask, handleEvents, bPtr);
-
+	
     /* default configuration */
-    W_ResizeView(bPtr->view, DEFAULT_WIDTH, DEFAULT_HEIGHT);
     bPtr->flags.hasScroller = DEFAULT_HAS_SCROLLER;
 
-    bPtr->maxVisibleColumns = 2;
-    
-    bPtr->columnSize.width = 150;
-    bPtr->columnSize.height = 150;
-
     bPtr->titleHeight = 20;
     bPtr->flags.isTitled = 1;
+    bPtr->maxVisibleColumns = 2;
 
+    resizeBrowser(bPtr, DEFAULT_WIDTH, DEFAULT_HEIGHT);
+    
     bPtr->pathSeparator = wstrdup(DEFAULT_SEPARATOR);
 
     if (bPtr->flags.hasScroller)
 	setupScroller(bPtr);
 
-    setupVisibleColumns(bPtr);
+    for (i=0; i<bPtr->maxVisibleColumns; i++) {
+	WMAddBrowserColumn(bPtr);
+    }
+    bPtr->usedColumnCount = 0;
+    
+    bPtr->selectedColumn = -1;
 
     return bPtr;
 }
 
 
+int 
+WMGetBrowserNumberOfColumns(WMBrowser *bPtr)
+{
+    return bPtr->usedColumnCount;
+}
+
+void
+WMSetBrowserPathSeparator(WMBrowser *bPtr, char *separator)
+{
+    if (bPtr->pathSeparator)
+	free(bPtr->pathSeparator);
+    bPtr->pathSeparator = wstrdup(separator);
+}
+
+
+
+static void
+drawTitleOfColumn(WMBrowser *bPtr, int column)
+{
+    WMScreen *scr = bPtr->view->screen;
+    int x;
+    
+    x=(column-bPtr->firstVisibleColumn)*(bPtr->columnSize.width+COLUMN_SPACING);
+
+    XFillRectangle(scr->display, bPtr->view->window, W_GC(scr->darkGray), x, 0,
+		   bPtr->columnSize.width, bPtr->titleHeight);
+    W_DrawRelief(scr, bPtr->view->window, x, 0,
+		 bPtr->columnSize.width, bPtr->titleHeight, WRSunken);
+
+    if (column < bPtr->usedColumnCount && bPtr->titles[column])
+	W_PaintText(bPtr->view, bPtr->view->window, scr->boldFont, x, 
+		    (bPtr->titleHeight-scr->boldFont->height)/2,
+		    bPtr->columnSize.width, WACenter, W_GC(scr->white),
+		    False, bPtr->titles[column], strlen(bPtr->titles[column]));
+}
+
+
 void
 WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, char *title)
 {
-    ColumnList *clPtr;
-    int i;
-
     assert(column >= 0);
-    assert(column < bPtr->columnCount);
-    
-    i = column;
-    clPtr = bPtr->columns;
-    while (i-- > 0) {
-	clPtr = clPtr->nextPtr;
-    }
-    
-    if (clPtr->title)
-	free(clPtr->title);
-    if (title)
-	clPtr->title = wstrdup(title);
-    else
-	clPtr->title = NULL;
-        
-    if (COLUMN_IS_VISIBLE(bPtr, column)) {	
-	i = column - bPtr->firstVisibleColumn;
+    assert(column < bPtr->usedColumnCount);
 
-	WMSetLabelText(bPtr->visibleTitles[i], title);
+    if (bPtr->titles[column])
+	free(bPtr->titles[column]);
+
+    bPtr->titles[column] = wstrdup(title);
+   
+    if (COLUMN_IS_VISIBLE(bPtr, column) && bPtr->flags.isTitled) {
+	drawTitleOfColumn(bPtr, column);
     }
 }
 
 
-
-static void
-freeColumnItems(WMBrowser *bPtr, WMListItem *list)
+WMList*
+WMGetBrowserListInColumn(WMBrowser *bPtr, int column)
 {
-    WMListItem *tmp;
+    if (column < 0 || column >= bPtr->usedColumnCount)
+	return NULL;
     
-    while (list) {
-	free(list->text);
-	
-	tmp = list->nextPtr;
-	free(list);
-	list = tmp;
-    }
+    return bPtr->columns[column];
 }
 
 
 void 
-WMSetBrowserFillColumnProc(WMBrowser *bPtr, WMBrowserFillColumn *proc)
+WMSetBrowserFillColumnProc(WMBrowser *bPtr, WMBrowserFillColumnProc *proc)
 {
     bPtr->fillColumn = proc;
 }
 
 
+int 
+WMGetBrowserFirstVisibleColumn(WMBrowser *bPtr)
+{
+    return bPtr->firstVisibleColumn;
+}
+
+
 static void
 removeColumn(WMBrowser *bPtr, int column)
 {
-    ColumnList *clPtr, *tmp = NULL;
     int i;
+    WMList **clist;
+    char **tlist;
 
-    if (column >= bPtr->columnCount)
+    if (column >= bPtr->usedColumnCount)
 	return;
-    
-    i = column;
-    clPtr = bPtr->columns;
-    while (i-- > 0) {
-	tmp = clPtr;
-	clPtr = clPtr->nextPtr;
-    }
 
-    if (tmp)
-	tmp->nextPtr = NULL;
-    
-    while (clPtr) {
-	bPtr->columnCount--;
-	freeColumnItems(bPtr, clPtr->column);
-	if (clPtr->title)
-	    free(clPtr->title);
-	if (COLUMN_IS_VISIBLE(bPtr, column)) {
-	    int index;
+    if (column < bPtr->maxVisibleColumns) {
+	int tmp;
+	for (i=column; i < bPtr->maxVisibleColumns; i++) {
+	    if (bPtr->titles[i])
+		free(bPtr->titles[i]);
+	    bPtr->titles[i] = NULL;
 	    
-	    index = column-bPtr->firstVisibleColumn;
-	    WMHangData(bPtr->visibleColumns[index], NULL);
-	    WMSetListItems(bPtr->visibleColumns[index], NULL);
-	    if (bPtr->flags.isTitled)
-		WMSetLabelText(bPtr->visibleTitles[index], NULL);
+	    WMClearList(bPtr->columns[i]);
+	    bPtr->usedColumnCount--;
 	}
-	column++;
-	tmp = clPtr->nextPtr;
-	free(clPtr);
-	clPtr = tmp;
+	tmp = bPtr->columnCount;
+	for (i=bPtr->maxVisibleColumns; i < tmp; i++) {
+	    if (bPtr->titles[i])
+		free(bPtr->titles[i]);
+	    bPtr->titles[i] = NULL;
+	    
+	    WMDestroyWidget(bPtr->columns[i]);
+	    bPtr->columns[i] = NULL;
+	    bPtr->columnCount--;
+	    bPtr->usedColumnCount--;
+	}
+    } else {
+	for (i=column; i < bPtr->columnCount; i++) {
+	    if (bPtr->titles[i])
+		free(bPtr->titles[i]);
+	    bPtr->titles[i] = NULL;
+	    
+	    WMDestroyWidget(bPtr->columns[i]);
+	    bPtr->columns[i] = NULL;
+	    bPtr->columnCount--;
+	    bPtr->usedColumnCount--;
+	}	
     }
+    clist = wmalloc(sizeof(WMList*)*bPtr->columnCount);
+    tlist = wmalloc(sizeof(char*)*bPtr->columnCount);
+    memcpy(clist, bPtr->columns, sizeof(WMList*)*bPtr->columnCount);
+    memcpy(tlist, bPtr->titles, sizeof(char*)*bPtr->columnCount);
+    free(bPtr->titles);
+    free(bPtr->columns);
+    bPtr->titles = tlist;
+    bPtr->columns = clist;
 }
 
 
+
 WMListItem*
 WMGetBrowserSelectedItemInColumn(WMBrowser *bPtr, int column)
 {
-    return WMGetListSelectedItem(bPtr->visibleColumns[column-bPtr->firstVisibleColumn]);
+    if ((column < 0) || (column > bPtr->columnCount))
+	return NULL;
+
+    return WMGetListSelectedItem(bPtr->columns[column]);
 }
 
 
 
-static void
-insertSortedItem(ColumnList *clPtr, WMListItem *item)
+int
+WMGetBrowserSelectedColumn(WMBrowser *bPtr)
 {
-    WMListItem *tmp;
+    return bPtr->selectedColumn;
+}
+
+
+int
+WMGetBrowserSelectedRowInColumn(WMBrowser *bPtr, int column)
+{
+    if (column >= 0 && column < bPtr->columnCount) {
+	return WMGetListSelectedItemRow(bPtr->columns[column]);
+    } else {
+	return -1;
+    }
+} 
+
+
+void
+WMSetBrowserTitled(WMBrowser *bPtr, Bool flag)
+{
+    int i;
+    int columnX, columnY;
+
+    if (bPtr->flags.isTitled == flag)
+	return;
+
+    columnX = 0;
     
-    if (!clPtr->column) {
-	clPtr->column = item;
-    } else if (strcmp(clPtr->column->text, item->text) > 0) {
-	item->nextPtr = clPtr->column;
-	clPtr->column = item;
-    } else {	
-	int added = 0;
-	
-	tmp = clPtr->column;
-	while (tmp->nextPtr) {
-	    if (strcmp(tmp->nextPtr->text, item->text) >= 0) {
-		item->nextPtr = tmp->nextPtr;
-		tmp->nextPtr = item;
-		added = 1;
-		break;
-	    }
-	    tmp = tmp->nextPtr;
+    if (!bPtr->flags.isTitled) {
+	columnY = TITLE_SPACING + bPtr->titleHeight;
+
+	bPtr->columnSize.height -= columnY;
+
+	for (i=0; i<bPtr->columnCount; i++) {
+	    WMResizeWidget(bPtr->columns[i], bPtr->columnSize.width,
+			   bPtr->columnSize.height);
+
+	    columnX = WMWidgetView(bPtr->columns[i])->pos.x;
+
+	    WMMoveWidget(bPtr->columns[i], columnX, columnY);
 	}
-	if (!added) {
-	    tmp->nextPtr = item;
+    } else {
+	bPtr->columnSize.height += TITLE_SPACING + bPtr->titleHeight;
+
+	for (i=0; i<bPtr->columnCount; i++) {
+	    WMResizeWidget(bPtr->columns[i], bPtr->columnSize.width,
+			   bPtr->columnSize.height);
+
+	    columnX = WMWidgetView(bPtr->columns[i])->pos.x;
+
+	    WMMoveWidget(bPtr->columns[i], columnX, 0);
 	}
     }
+    
+    bPtr->flags.isTitled = flag;
 }
 
 
-Bool
-WMAddBrowserItem(WMBrowser *bPtr, int column, char *text, Bool isBranch)
+WMListItem*
+WMAddSortedBrowserItem(WMBrowser *bPtr, int column, char *text, Bool isBranch)
 {
-    ColumnList *clPtr;
+    WMListItem *item;
+
+    if (column < 0 || column >= bPtr->columnCount)
+	return NULL;
+	
+    item = WMAddSortedListItem(bPtr->columns[column], text);
+    item->isBranch = isBranch;
+
+    return item;
+}
+
+
+
+WMListItem*
+WMInsertBrowserItem(WMBrowser *bPtr, int column, int row, char *text,
+		    Bool isBranch)
+{
+    WMListItem *item;
+
+    if (column < 0 || column >= bPtr->columnCount)
+	return NULL;
+
+    item = WMInsertListItem(bPtr->columns[column], row, text);
+    item->isBranch = isBranch;
+
+    return item;
+}
+
+
+
+
+static void 
+resizeBrowser(WMWidget *w, unsigned int width, unsigned int height)
+{
+    WMBrowser *bPtr = (WMBrowser*)w;
+    int cols = bPtr->maxVisibleColumns;
+    int colX, colY;
     int i;
 
-    assert(column >= 0);
-    assert(column < bPtr->columnCount);
+    assert(width > 0);
+    assert(height > 0);
     
-    i = column;
-    clPtr = bPtr->columns;
-    while (i-- > 0) {
-	clPtr = clPtr->nextPtr;
-    }
-
-    if (!bPtr->flags.loadingColumn && COLUMN_IS_VISIBLE(bPtr, column)) {
-	WMListItem *item;
-
-	i = column - bPtr->firstVisibleColumn;
-	
-	item = WMAddSortedListItem(bPtr->visibleColumns[i], text);
-	item->isBranch = isBranch;
-	
-	clPtr->column = WMGetListItems(bPtr->visibleColumns[i]);
+    bPtr->columnSize.width = (width-(cols-1)*COLUMN_SPACING) / cols;
+    bPtr->columnSize.height = height;
+    
+    if (bPtr->flags.isTitled) {
+	bPtr->columnSize.height -= TITLE_SPACING + bPtr->titleHeight;
+	colY = TITLE_SPACING + bPtr->titleHeight;
     } else {
-	WMListItem *item;
+	colY = 0;
+    }
     
-	item = wmalloc(sizeof(WMListItem));
-	memset(item, 0, sizeof(WMListItem));
-	item->text = wstrdup(text);
-	item->isBranch = isBranch;
+    if (bPtr->flags.hasScroller) {
+	bPtr->columnSize.height -= SCROLLER_WIDTH + 4;
 
-	insertSortedItem(clPtr, item);
+	if (bPtr->scroller) {
+	    WMResizeWidget(bPtr->scroller, width-2, 1);
+	    WMMoveWidget(bPtr->scroller, 1, height-SCROLLER_WIDTH-1);
+	}
     }
 
-    return True;
+    colX = 0;
+    for (i = 0; i < bPtr->columnCount; i++) {
+	WMResizeWidget(bPtr->columns[i], bPtr->columnSize.width,
+		       bPtr->columnSize.height);
+	
+	WMMoveWidget(bPtr->columns[i], colX, colY);
+	
+	if (COLUMN_IS_VISIBLE(bPtr, i)) {
+	    colX += bPtr->columnSize.width+COLUMN_SPACING;
+	}
+    }
+
+    W_ResizeView(bPtr->view, width, height);
 }
 
 
@@ -328,7 +436,7 @@ paintItem(WMList *lPtr, Drawable d, char *text, int state, WMRect *rect)
 	XClearArea(scr->display, d, x, y, width, height, False);
 	
     W_PaintText(view, d, scr->normalFont,  x+2, y, width,
-		WALeft, scr->normalFontGC, False, text, strlen(text));
+		WALeft, W_GC(scr->black), False, text, strlen(text));
     
     if (state & WLDSIsBranch) {
 	XDrawLine(scr->display, d, W_GC(scr->darkGray), x+width-11, y+3,
@@ -391,7 +499,7 @@ scrollCallback(WMWidget *scroller, void *self)
 	    float floatValue;
 	    float value = bPtr->columnCount - bPtr->maxVisibleColumns;
 	    
-	    floatValue = WMGetScrollerFloatValue(bPtr->scroller);
+	    floatValue = WMGetScrollerValue(bPtr->scroller);
 	    
 	    floatValue = (floatValue*value)/value;
 	    
@@ -421,9 +529,7 @@ setupScroller(WMBrowser *bPtr)
     WMScroller *sPtr;
     int y;
 
-    y = bPtr->columnSize.height + SCROLLER_SPACING + 1;
-    if (bPtr->flags.isTitled)
-	y += TITLE_SPACING + bPtr->titleHeight;
+    y = bPtr->view->size.height - SCROLLER_WIDTH - 1;
     
     sPtr = WMCreateScroller(bPtr);
     WMSetScrollerAction(sPtr, scrollCallback, bPtr);
@@ -451,63 +557,40 @@ WMSetBrowserHasScroller(WMBrowser *bPtr, int hasScroller)
 }
 
 
-static char*
-selectItem(ColumnList *clist, char *text)
-{
-    WMListItem *item, *theItem=NULL;
-
-    item = clist->column;
-    while (item) {
-	if (strcmp(item->text, text)==0) {
-	    theItem = item;
-	    item->selected = True;
-	} else {
-	    item->selected = False;
-	}
-	item = item->nextPtr;
-    }
-
-    if (theItem) 
-	return theItem->text;
-    else
-	return NULL;
-}
-
 
 Bool
 WMSetBrowserPath(WMBrowser *bPtr, char *path)
 {
     int i;
-    ColumnList *clist;
     char *str = wstrdup(path);
     char *tmp;
+    int item;
     Bool ok = True;
 
     removeColumn(bPtr, 1);
 
     i = 0;
-    clist = bPtr->columns;
     tmp = strtok(str, bPtr->pathSeparator);
     while (tmp) {
 	/* select it in the column */
-	clist->selectedText = selectItem(clist, tmp);
-
-	if (!clist->selectedText) {
+	item = WMFindRowOfListItemWithTitle(bPtr->columns[i], tmp);
+	if (item<0) {
 	    ok = False;
 	    break;
 	}
+	WMSelectListItem(bPtr->columns[i], item);
+	
 	/* load next column */
 	WMAddBrowserColumn(bPtr);
 
 	loadColumn(bPtr, i+1);
 	
 	tmp = strtok(NULL, bPtr->pathSeparator);
-	
-	clist = clist->nextPtr;
+
 	i++;
     }
     free(str);
-	    
+
     scrollToColumn(bPtr, bPtr->columnCount-bPtr->maxVisibleColumns);
 
     return ok;
@@ -525,31 +608,31 @@ char*
 WMGetBrowserPathToColumn(WMBrowser *bPtr, int column)
 {
     int i, size;
-    ColumnList *clist;
     char *path;
+    WMListItem *item;
     
-    if (column >= bPtr->columnCount)
-	column = bPtr->columnCount-1;
+    if (column >= bPtr->usedColumnCount)
+	column = bPtr->usedColumnCount-1;
     
-    clist = bPtr->columns;
+    /* calculate size of buffer */
     size = 0;
     for (i = 0; i <= column; i++) {
-	if (!clist->selectedText)
+	item = WMGetListSelectedItem(bPtr->columns[i]);
+	if (!item)
 	    break;
-	size += strlen(clist->selectedText);
-	clist = clist->nextPtr;
+	size += strlen(item->text);
     }
     
-    clist = bPtr->columns;
+    /* get the path */
     path = wmalloc(size+(column+1)*strlen(bPtr->pathSeparator)+1);
     /* ignore first / */
     *path = 0;
     for (i = 0; i <= column; i++) {
 	strcat(path, bPtr->pathSeparator);
-	if (!clist->selectedText)
+	item = WMGetListSelectedItem(bPtr->columns[i]);
+	if (!item)
 	    break;
-	strcat(path, clist->selectedText);
-	clist = clist->nextPtr;
+	strcat(path, item->text);
     }
 
     return path;
@@ -570,17 +653,20 @@ loadColumn(WMBrowser *bPtr, int column)
 static void
 paintBrowser(WMBrowser *bPtr)
 {
-    int y;
+    int i;
 
     if (!bPtr->view->flags.mapped)
 	return;
-    
-    y = bPtr->columnSize.height + SCROLLER_SPACING;
-    if (bPtr->flags.isTitled)
-	y += TITLE_SPACING + bPtr->titleHeight;    
 
-    W_DrawRelief(bPtr->view->screen, bPtr->view->window, 0, y,
+    W_DrawRelief(bPtr->view->screen, bPtr->view->window, 0, 
+		 bPtr->view->size.height-SCROLLER_WIDTH-2,
 		 bPtr->view->size.width, 22, WRSunken);
+    
+    if (bPtr->flags.isTitled) {
+	for (i=0; i<bPtr->maxVisibleColumns; i++) {
+	    drawTitleOfColumn(bPtr, i+bPtr->firstVisibleColumn);
+	}
+    }
 }
 
 
@@ -605,58 +691,36 @@ handleEvents(XEvent *event, void *data)
 }
 
 
-extern int _listTopItem();
-extern void _listSetTopItem();
-
-
 
 static void
 scrollToColumn(WMBrowser *bPtr, int column)
 {
-    int i, lim;
-    ColumnList *listPtr;
+    int i;
+    int x;
+    int notify = 0;
+
+    
+    if (column != bPtr->firstVisibleColumn)
+	notify = 1;
 
     if (column < 0)
 	column = 0;
-        
-    if (column + bPtr->maxVisibleColumns < bPtr->columnCount)
-	lim = bPtr->maxVisibleColumns;
-    else
-	lim = bPtr->columnCount - column;
-    
-    listPtr = bPtr->columns;
-    i = column;
-    while (i-- > 0)
-	listPtr = listPtr->nextPtr;
 
-    for (i = 0; i < bPtr->maxVisibleColumns; i++) {
-	ColumnList *tmp;
-	
-	if (i < lim) {
-	    tmp = WMGetHangedData(bPtr->visibleColumns[i]);
-	    if (tmp!=listPtr || !tmp) {
-		if (tmp)
-		    tmp->topRow = _listTopItem(bPtr->visibleColumns[i]);
-		
-		WMHangData(bPtr->visibleColumns[i], listPtr);
-		
-		WMSetListItems(bPtr->visibleColumns[i], listPtr->column);
-		_listSetTopItem(bPtr->visibleColumns[i], listPtr->topRow);
-
-		if (bPtr->flags.isTitled)
-		    WMSetLabelText(bPtr->visibleTitles[i], listPtr->title);
-	    }	    
-	    listPtr = listPtr->nextPtr;
+    x = 0;
+    bPtr->firstVisibleColumn = column;
+    for (i = 0; i < bPtr->usedColumnCount; i++) {
+	if (COLUMN_IS_VISIBLE(bPtr, i)) {
+	    WMMoveWidget(bPtr->columns[i], x,
+			 WMWidgetView(bPtr->columns[i])->pos.y);
+	    if (!WMWidgetView(bPtr->columns[i])->flags.realized)
+		WMRealizeWidget(bPtr->columns[i]);
+	    WMMapWidget(bPtr->columns[i]);
+	    x += bPtr->columnSize.width+COLUMN_SPACING;
 	} else {
-	    WMSetListItems(bPtr->visibleColumns[i], NULL);
-	    WMHangData(bPtr->visibleColumns[i], NULL);
-	    if (bPtr->flags.isTitled)
-		WMSetLabelText(bPtr->visibleTitles[i], "");
+	    WMUnmapWidget(bPtr->columns[i]);
 	}
     }
 
-    bPtr->firstVisibleColumn = column;
-    
     /* update the scroller */
     if (bPtr->columnCount > bPtr->maxVisibleColumns) {
 	float value, proportion;
@@ -668,6 +732,12 @@ scrollToColumn(WMBrowser *bPtr, int column)
     } else {
 	WMSetScrollerParameters(bPtr->scroller, 0, 1);
     }
+    
+    if (bPtr->view->flags.mapped)
+	paintBrowser(bPtr);
+
+    if (notify)
+	WMPostNotificationName(WMBrowserDidScrollNotification, bPtr, NULL);
 }
 
 
@@ -677,38 +747,33 @@ listCallback(void *self, void *clientData)
     WMBrowser *bPtr = (WMBrowser*)clientData;
     WMList *lPtr = (WMList*)self;
     WMListItem *item;
-    ColumnList *clPtr;
     int i;
-
-    for (i = 0; i < bPtr->maxVisibleColumns; i++) {
-	if (self == bPtr->visibleColumns[i])
-	    break;
-    }
     
-    clPtr = WMGetHangedData(lPtr);
     item = WMGetListSelectedItem(lPtr);
     if (!item)
 	return;
     
-    clPtr->selectedText = item->text;
+    for (i=0; i<bPtr->columnCount; i++) {
+	if (lPtr == bPtr->columns[i])
+	    break;
+    }
+    assert(i<bPtr->columnCount);
+    
+    bPtr->selectedColumn = i;
 
     /* columns at right must be cleared */
-    removeColumn(bPtr, i+bPtr->firstVisibleColumn+1);
-
+    removeColumn(bPtr, i+1);
     /* open directory */
     if (item->isBranch) {	
 	WMAddBrowserColumn(bPtr);
-	
-	loadColumn(bPtr, bPtr->columnCount-1);
-
-	if (bPtr->columnCount < bPtr->maxVisibleColumns)
-	    i = 0;
-	else
-	    i = bPtr->columnCount-bPtr->maxVisibleColumns;
-
-	scrollToColumn(bPtr, i);
+	loadColumn(bPtr, bPtr->usedColumnCount-1);
     }
-    
+    if (bPtr->usedColumnCount < bPtr->maxVisibleColumns)
+	i = 0;
+    else
+	i = bPtr->usedColumnCount-bPtr->maxVisibleColumns;
+    scrollToColumn(bPtr, i);
+
     /* call callback for click */
     if (bPtr->action)
 	(*bPtr->action)(bPtr, bPtr->clientData);
@@ -732,85 +797,76 @@ WMLoadBrowserColumnZero(WMBrowser *bPtr)
 }
 
 
-static void
-setupVisibleColumns(WMBrowser *bPtr)
+void
+WMRemoveBrowserItem(WMBrowser *bPtr, int column, int row)
 {
-    WMScreen *scr = bPtr->view->screen;
-    int columnY, columnX;
-    int i;
+    WMList *list;
     
-    if (bPtr->visibleColumns==NULL) {
-	bPtr->visibleColumns = 
-	    wmalloc(sizeof(WMList*)*bPtr->maxVisibleColumns);
-	bPtr->visibleTitles =
-	    wmalloc(sizeof(WMLabel*)*bPtr->maxVisibleColumns);
-    } else {
-	printf("reconfigurevisiblecolumns not supported");
-	abort();
-    }
+    if (column < 0 || column >= bPtr->usedColumnCount)
+	return;
     
-    columnY = 0;
-    columnX = 0;
-        
-    if (bPtr->flags.isTitled) {
-	columnY = TITLE_SPACING + bPtr->titleHeight;
-    }
-
-    for (i = 0; i < bPtr->maxVisibleColumns; i++) {
-	WMList *list;
-	WMLabel *label;
-
-	list = WMCreateList(bPtr);
-	WMSetListAction(list, listCallback, bPtr);
-	WMSetListUserDrawProc(list, paintItem);
-	WMMoveWidget(list, columnX, columnY);
-	WMResizeWidget(list,bPtr->columnSize.width, bPtr->columnSize.height);
-	WMMapWidget(list);
-	
-	if (bPtr->flags.isTitled) {
-	    label = WMCreateLabel(bPtr);
-	    WMMoveWidget(label, columnX, 0);
-	    WMResizeWidget(label, bPtr->columnSize.width, bPtr->titleHeight);
-	    WMSetWidgetBackgroundColor(label, scr->darkGray);
-	    WMSetLabelFont(label, scr->boldFont);
-	    WMSetLabelTextColor(label, scr->white);
-	    WMSetLabelRelief(label, WRSunken);
-	    WMSetLabelText(label, "");
-	    WMSetLabelTextAlignment(label, WACenter);
-	    WMMapWidget(label);
-	} else {
-	    label = NULL;
-	}
-	
-	bPtr->visibleColumns[i] = list;
-	bPtr->visibleTitles[i] = label;
-	
-	columnX += bPtr->columnSize.width + COLUMN_SPACING;
-    }
+    list = WMGetBrowserListInColumn(bPtr, column);
+    
+    if (row < 0 || row >= WMGetListNumberOfRows(list))
+	return;
+    
+    removeColumn(bPtr, column+1);
+    if (bPtr->usedColumnCount < bPtr->maxVisibleColumns)
+	scrollToColumn(bPtr, 0);
+    else
+	scrollToColumn(bPtr, bPtr->usedColumnCount-bPtr->maxVisibleColumns);
+    
+    WMRemoveListItem(list, row);
 }
 
 
 int
 WMAddBrowserColumn(WMBrowser *bPtr)
 {
-    ColumnList *tmp, *listPtr;
+    WMList *list;
+    WMList **clist;
+    char **tlist;
+    int colY;
+    int index;
 
-    listPtr = wmalloc(sizeof(ColumnList));
-    memset(listPtr, 0, sizeof(ColumnList));
-
-    bPtr->columnCount++;
     
-    /* add column to the end of the list */
-    tmp = bPtr->columns;
-    if (tmp == NULL) {
-	bPtr->columns = listPtr;
-    } else {
-	while (tmp->nextPtr!=NULL) {
-	    tmp = tmp->nextPtr;
-	}
-	tmp->nextPtr = listPtr;
+    if (bPtr->usedColumnCount < bPtr->columnCount) {
+	return bPtr->usedColumnCount++;
     }
 
+    bPtr->usedColumnCount++;
+
+    if (bPtr->flags.isTitled) {
+	colY = TITLE_SPACING + bPtr->titleHeight;
+    } else {
+	colY = 0;
+    }
+
+    index = bPtr->columnCount;
+    bPtr->columnCount++;
+    clist = wmalloc(sizeof(WMList*)*bPtr->columnCount);
+    tlist = wmalloc(sizeof(char*)*bPtr->columnCount);
+    memcpy(clist, bPtr->columns, sizeof(WMList*)*(bPtr->columnCount-1));
+    memcpy(tlist, bPtr->titles, sizeof(char*)*(bPtr->columnCount-1));
+    if (bPtr->columns)
+	free(bPtr->columns);
+    if (bPtr->titles)
+	free(bPtr->titles);
+    bPtr->columns = clist;
+    bPtr->titles = tlist;
+
+    bPtr->titles[index] = NULL;
+
+    list = WMCreateList(bPtr);
+    WMSetListAction(list, listCallback, bPtr);
+    WMSetListUserDrawProc(list, paintItem);
+    bPtr->columns[index] = list;
+
+    WMResizeWidget(list, bPtr->columnSize.width, bPtr->columnSize.height);
+    WMMoveWidget(list, (bPtr->columnSize.width+COLUMN_SPACING)*index, colY);
+    if (COLUMN_IS_VISIBLE(bPtr, index)) 
+	WMMapWidget(list);
+
     /* update the scroller */
     if (bPtr->columnCount > bPtr->maxVisibleColumns) {
 	float value, proportion;
@@ -821,7 +877,7 @@ WMAddBrowserColumn(WMBrowser *bPtr)
 	WMSetScrollerParameters(bPtr->scroller, value, proportion);
     }
 
-    return bPtr->columnCount-1;
+    return index;
 }
 
 
@@ -829,23 +885,13 @@ WMAddBrowserColumn(WMBrowser *bPtr)
 static void
 destroyBrowser(WMBrowser *bPtr)
 {
-    ColumnList *listPtr;
     int i;
 
-    i = 0;
-    while (bPtr->columns != NULL) {
-	free(bPtr->columns->title);
-	/* do not free column data that is used in WMLists, because
-	 * they will be freed by the WMLists */
-	if (i < bPtr->firstVisibleColumn 
-	    || i >= bPtr->firstVisibleColumn+bPtr->maxVisibleColumns)
-	    freeColumnItems(bPtr, bPtr->columns->column);
-	
-	listPtr = bPtr->columns->nextPtr;
-	free(bPtr->columns);
-	bPtr->columns = listPtr;
-	i++;
+    for (i=0; i<bPtr->columnCount; i++) {
+	if (bPtr->titles[i])
+	    free(bPtr->titles[i]);
     }
+    free(bPtr->titles);
     
     free(bPtr->pathSeparator);
     
