@@ -5,7 +5,6 @@
 #include <X11/Xutil.h>
 
 
-
 /********** data ************/
 
 
@@ -267,6 +266,20 @@ static char *POPUP_INDICATOR[] = {
 #define POPUP_INDICATOR_HEIGHT 	8
 
 
+
+static char *PULLDOWN_INDICATOR[] = {
+"=#######=",
+"=%===== =",
+"==%=== ==",
+"==%=== ==",
+"===%= ===",
+"===%= ===",
+"====%===="
+};
+#define PULLDOWN_INDICATOR_WIDTH	9
+#define PULLDOWN_INDICATOR_HEIGHT 	7
+
+
 #define STIPPLE_WIDTH 8
 #define STIPPLE_HEIGHT 8
 static unsigned char STIPPLE_BITS[] = {
@@ -286,11 +299,16 @@ extern W_ViewProcedureTable _BrowserViewProcedures;
 extern W_ViewProcedureTable _PopUpButtonViewProcedures;
 extern W_ViewProcedureTable _ColorWellViewProcedures;
 extern W_ViewProcedureTable _ScrollViewViewProcedures;
+extern W_ViewProcedureTable _SliderViewProcedures;
+extern W_ViewProcedureTable _SplitViewViewProcedures;
 
 /*
  * All widget classes defined must have an entry here.
  */
-W_ViewProcedureTable *procedureTables[16];
+static W_ViewProcedureTable *procedureTables[16];
+
+static W_ViewProcedureTable **userProcedureTable = NULL;
+static int userWidgetCount=0;
 
 
 /*****  end data  ******/
@@ -300,12 +318,6 @@ W_ViewProcedureTable *procedureTables[16];
 static void
 initProcedureTable()
 {
-    static int inited = 0;
-    
-    if (inited)
-	return;
-    inited = 1;
-
     procedureTables[WC_Window] = &_WindowViewProcedures;
     procedureTables[WC_Frame] = &_FrameViewProcedures;
     procedureTables[WC_Label] = &_LabelViewProcedures;
@@ -317,6 +329,8 @@ initProcedureTable()
     procedureTables[WC_PopUpButton] = &_PopUpButtonViewProcedures;
     procedureTables[WC_ColorWell] = &_ColorWellViewProcedures;
     procedureTables[WC_ScrollView] = &_ScrollViewViewProcedures;
+    procedureTables[WC_Slider] = &_SliderViewProcedures;
+    procedureTables[WC_SplitView] = &_SplitViewViewProcedures;
 }
 
 
@@ -428,13 +442,11 @@ loadPixmaps(WMScreen *scr)
 
 
 WMScreen*
-WMCreateSimpleApplicationScreen(Display *display, char *applicationName,
-				int *argc, char **argv)
+WMCreateSimpleApplicationScreen(Display *display)
 {
     WMScreen *scr;
     
-    scr = WMCreateScreen(display, DefaultScreen(display), 
-			 applicationName, argc, argv);
+    scr = WMCreateScreen(display, DefaultScreen(display));
     
     scr->aflags.hasAppIcon = 0;
     scr->aflags.simpleApplication = 1;
@@ -445,25 +457,29 @@ WMCreateSimpleApplicationScreen(Display *display, char *applicationName,
 
 
 WMScreen*
-WMCreateScreen(Display *display, int screen, char *applicationName,
-	       int *argc, char **argv)
-
+WMCreateScreen(Display *display, int screen)
 {
     return WMCreateScreenWithRContext(display, screen,
-				      RCreateContext(display, screen, NULL),
-				      applicationName, argc, argv);
+				      RCreateContext(display, screen, NULL));
 }
 
 
 WMScreen*
-WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
-			   char *applicationName, int *argc, char **argv)
+WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 {
     W_Screen *scrPtr;
     XGCValues gcv;
     Pixmap stipple;
+    static int initialized = 0;
 
-    initProcedureTable();
+    if (!initialized) {
+
+	initialized = 1;
+	
+	initProcedureTable();
+
+	assert(W_ApplicationInitialized());
+    }
     
     scrPtr = malloc(sizeof(W_Screen));
     if (!scrPtr)
@@ -476,19 +492,6 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
     scrPtr->screen = screen;
     scrPtr->rcontext = context;
 
-    if (argc && argv) {
-	int i;
-
-	scrPtr->argc = *argc;
-	scrPtr->argv = wmalloc((scrPtr->argc+1)*sizeof(char*));
-	for (i=0; i<scrPtr->argc; i++) {
-	    scrPtr->argv[i] = wstrdup(argv[i]);
-	}
-	scrPtr->argv[i] = NULL;
-    }
-    
-    scrPtr->applicationName = wstrdup(applicationName);
-
     scrPtr->depth = DefaultDepth(display, screen);
 
     scrPtr->visual = DefaultVisual(display, screen);
@@ -512,6 +515,13 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
     scrPtr->xorGC = XCreateGC(display, scrPtr->rootWin, GCFunction
 			      |GCGraphicsExposures|GCForeground, &gcv);
 
+    gcv.function = GXxor;
+    gcv.foreground = W_PIXEL(scrPtr->gray);
+    gcv.subwindow_mode = IncludeInferiors;
+    scrPtr->ixorGC = XCreateGC(display, scrPtr->rootWin, GCFunction
+			       |GCGraphicsExposures|GCForeground
+			       |GCSubwindowMode, &gcv);
+
     gcv.function = GXcopy;
     scrPtr->copyGC = XCreateGC(display, scrPtr->rootWin, GCFunction
 			       |GCGraphicsExposures, &gcv);
@@ -529,7 +539,12 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
     scrPtr->stippleGC = XCreateGC(display, scrPtr->rootWin, 
 				  GCForeground|GCBackground|GCStipple
 				  |GCFillStyle|GCGraphicsExposures, &gcv);
-    
+
+    gcv.foreground = W_PIXEL(scrPtr->black);
+    gcv.background = W_PIXEL(scrPtr->white);
+    scrPtr->textFieldGC = XCreateGC(display, scrPtr->rootWin,
+				    GCForeground|GCBackground, &gcv);
+
     /* we need a 1bpp drawable for the monoGC, so borrow this one */
     scrPtr->monoGC = XCreateGC(display, stipple, 0, NULL);
 
@@ -537,18 +552,8 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
     
     
     scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 12);
-    gcv.font = W_FONTID(scrPtr->normalFont);
-    gcv.foreground = W_PIXEL(scrPtr->black);
-    gcv.background = W_PIXEL(scrPtr->white);
-    scrPtr->normalFontGC = XCreateGC(display, scrPtr->rootWin, GCFont
-				     |GCForeground|GCGraphicsExposures
-				     |GCBackground, &gcv);
 
     scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 12);
-    gcv.font = W_FONTID(scrPtr->boldFont);
-    gcv.foreground = W_PIXEL(scrPtr->black);
-    scrPtr->boldFontGC = XCreateGC(display, scrPtr->rootWin, GCFont
-				   |GCForeground|GCGraphicsExposures, &gcv);
 
     scrPtr->checkButtonImageOn = makePixmap(scrPtr, CHECK_BUTTON_ON,
 					    CHECK_BUTTON_ON_WIDTH,
@@ -616,6 +621,9 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
 					POPUP_INDICATOR_WIDTH,
 					POPUP_INDICATOR_HEIGHT, True);
 
+    scrPtr->pullDownIndicator = makePixmap(scrPtr, PULLDOWN_INDICATOR,
+					   PULLDOWN_INDICATOR_WIDTH,
+					   PULLDOWN_INDICATOR_HEIGHT, True);
     loadPixmaps(scrPtr);
     
     scrPtr->internalMessage = XInternAtom(display, "_WINGS_MESSAGE", False);
@@ -628,8 +636,6 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context,
 
     scrPtr->rootView = W_CreateRootView(scrPtr);
 
-    
-    
     return scrPtr;
 }
 
@@ -659,7 +665,7 @@ WMDestroyWidget(WMWidget *widget)
 void
 WMSetFocusToWidget(WMWidget *widget)
 {
-    W_SetFocusToView(W_VIEW(widget));
+    W_SetFocusOfTopLevel(W_TopLevelOfView(W_VIEW(widget)), W_VIEW(widget));
 }
 
 
@@ -671,20 +677,13 @@ WMSetFocusToWidget(WMWidget *widget)
 void
 WMRealizeWidget(WMWidget *w)
 {
-    if (procedureTables[W_CLASS(w)]->realize)
-	(*procedureTables[W_CLASS(w)]->realize)(w);
-    else
-	W_RealizeView(W_VIEW(w));
+    W_RealizeView(W_VIEW(w));
 }
 
 void
 WMMapWidget(WMWidget *w)
 {        
-    if (!W_VIEW(w)->flags.realized) {
-	W_VIEW(w)->flags.mapWhenRealized = 1;
-    } else {
-	W_MapView(W_VIEW(w));
-    }
+    W_MapView(W_VIEW(w));
 }
 
 
@@ -723,29 +722,79 @@ WMUnmapWidget(WMWidget *w)
 
 void
 WMSetWidgetBackgroundColor(WMWidget *w, WMColor *color)
-{
-    if (procedureTables[W_CLASS(w)]->setBackgroundColor)
+{    
+    if (W_CLASS(w) < WC_UserWidget
+	&& procedureTables[W_CLASS(w)]->setBackgroundColor) {
+	
 	(*procedureTables[W_CLASS(w)]->setBackgroundColor)(w, color);
-    else
+	
+    } else if (W_CLASS(w) >= WC_UserWidget
+	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor) {
+	
+	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor)(w, color);
+	
+    } else {
 	W_SetViewBackgroundColor(W_VIEW(w), color);
+    }
 }
 
+
 void
 WMMoveWidget(WMWidget *w, int x, int y)
 {
-    if (procedureTables[W_CLASS(w)]->move)
+    if (W_CLASS(w) < WC_UserWidget
+	&& procedureTables[W_CLASS(w)]->move) {
+	
 	(*procedureTables[W_CLASS(w)]->move)(w, x, y);
-    else
+	
+    } else if (W_CLASS(w) >= WC_UserWidget
+	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->move) {
+
+	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->move)(w, x, y);
+
+    } else {
 	W_MoveView(W_VIEW(w), x, y);
+    }
 }
 
+
 void
 WMResizeWidget(WMWidget *w, unsigned int width, unsigned int height)
 {
-    if (procedureTables[W_CLASS(w)]->resize)
+    if (W_CLASS(w) < WC_UserWidget
+	&& procedureTables[W_CLASS(w)]->resize) {
+	
 	(*procedureTables[W_CLASS(w)]->resize)(w, width, height);
-    else
+	
+    } else if (W_CLASS(w) >= WC_UserWidget
+	       && userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize) {
+
+	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize)(w, width, height);
+
+    } else {
 	W_ResizeView(W_VIEW(w), width, height);
+    }
+}
+
+
+
+W_Class
+W_RegisterUserWidget(W_ViewProcedureTable *procTable)
+{
+    W_ViewProcedureTable **newTable;
+    
+    userWidgetCount++;
+    newTable = wmalloc(sizeof(W_ViewProcedureTable*)*userWidgetCount);
+    memcpy(newTable, userProcedureTable, 
+	   sizeof(W_ViewProcedureTable*)*(userWidgetCount-1));
+
+    newTable[userWidgetCount-1] = procTable;
+    
+    free(userProcedureTable);
+    
+    userProcedureTable = newTable;
+
+    return userWidgetCount + WC_UserWidget - 1;
 }
 
 
@@ -797,3 +846,18 @@ WMScreenMainLoop(WMScreen *scr)
 	WMHandleEvent(&event);
     }
 }
+
+
+
+Display*
+WMScreenDisplay(WMScreen *scr)
+{
+    return scr->display;
+}
+
+
+void 
+WMRedisplayWidget(WMWidget *w)
+{
+    W_RedisplayView(W_VIEW(w));
+}
