@@ -77,7 +77,6 @@ static void handleMotion(Scroller *sPtr, int mouseX, int mouseY);
 W_ViewProcedureTable _ScrollerViewProcedures = {
     NULL,
 	resizeScroller,
-	NULL,
 	NULL
 };
 
@@ -99,6 +98,7 @@ WMCreateScroller(WMWidget *parent)
 	free(sPtr);
 	return NULL;
     }
+    sPtr->view->self = sPtr;
     
     sPtr->flags.documentFullyVisible = 1;
     
@@ -199,7 +199,7 @@ WMGetScrollerKnobProportion(WMScroller *sPtr)
 
 
 float
-WMGetScrollerFloatValue(WMScroller *sPtr)
+WMGetScrollerValue(WMScroller *sPtr)
 {
     CHECK_CLASS(sPtr, WC_Scroller);
     
@@ -475,9 +475,11 @@ paintScroller(Scroller *sPtr)
 			       view->size.width-4, 
 			       length-(int)(knobP+knobL));
 	}
-	
-	paintArrow(sPtr, d, 0);
-	paintArrow(sPtr, d, 1);
+
+	if (sPtr->flags.arrowsPosition != WSANone) {
+	    paintArrow(sPtr, d, 0);
+	    paintArrow(sPtr, d, 1);
+	}
     }
 
 #ifdef DOUBLE_BUFFER
