@@ -155,7 +155,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     if (napp) 
 	napp->last_workspace = wwin->screen_ptr->current_workspace;
     
-    if (!wwin->window_flags.focusable)
+    if (wwin->window_flags.no_focusable)
 	return;
 
     if (wwin->flags.mapped) {
@@ -268,7 +268,7 @@ wShadeWindow(WWindow  *wwin)
 	    XResizeWindow(dpy, wwin->frame->core->window, w, h);
 	    XSync(dpy, 0);
             if (SHADE_DELAY > 0)
-                wmsleep(SHADE_DELAY*1000L);
+                wusleep(SHADE_DELAY*1000L);
 	    h-=s;
 	    y-=s;
 	}
@@ -335,7 +335,7 @@ wUnshadeWindow(WWindow  *wwin)
 		XMoveWindow(dpy, wwin->client_win, 0, y);
 		XSync(dpy, 0);
                 if (SHADE_DELAY > 0)
-                    wmsleep(SHADE_DELAY*2000L/3);
+                    wusleep(SHADE_DELAY*2000L/3);
 		h+=s;
 		y+=s;
 	    }
@@ -479,7 +479,7 @@ animateResize(WScreen *scr, int x, int y, int w, int h,
         XDrawLines(dpy, scr->root_win, scr->frame_gc, points, 5, CoordModeOrigin);
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY > 0)	
- 	wmsleep(MINIATURIZE_ANIMATION_DELAY);
+ 	wusleep(MINIATURIZE_ANIMATION_DELAY);
 #endif
 
 	XDrawLines(dpy, scr->root_win, scr->frame_gc, points, 5, CoordModeOrigin);
@@ -527,7 +527,7 @@ animateResize(WScreen *scr, int x, int y, int w, int h,
 	}
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY > 0)	
-	wmsleep(MINIATURIZE_ANIMATION_DELAY);
+	wusleep(MINIATURIZE_ANIMATION_DELAY);
 #endif
 	for (j=0; j<FRAMES; j++) {
 	    XDrawRectangle(dpy, scr->root_win, scr->frame_gc, 
@@ -552,7 +552,7 @@ animateResize(WScreen *scr, int x, int y, int w, int h,
     }
     XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY > 0)	
-    wmsleep(MINIATURIZE_ANIMATION_DELAY);
+    wusleep(MINIATURIZE_ANIMATION_DELAY);
 #endif
     for (j=0; j<FRAMES; j++) {
 	XDrawRectangle(dpy, scr->root_win, scr->frame_gc, 
@@ -672,6 +672,13 @@ wIconifyWindow(WWindow *wwin)
     if (wwin->flags.miniaturized)
 	return;
     
+    if (wwin->transient_for!=None) {
+	WWindow *owner = wWindowFor(wwin->transient_for);
+	
+	if (owner && owner->flags.miniaturized)
+	    return;
+    }
+    
     present = wwin->frame->workspace==wwin->screen_ptr->current_workspace;
  
     /* if the window is in another workspace, simplify process */
@@ -735,7 +742,7 @@ wIconifyWindow(WWindow *wwin)
 	    
 	    tmp = wwin->prev;
 	    while (tmp) {
-		if (tmp->window_flags.focusable 
+		if (!tmp->window_flags.no_focusable
 		    && !(tmp->flags.hidden||tmp->flags.miniaturized))
 		    break;
 		tmp = tmp->prev;
@@ -756,6 +763,8 @@ wIconifyWindow(WWindow *wwin)
     if (wwin->flags.selected)
         wIconSelect(wwin->icon);
 
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+    
 #ifdef WMSOUND
     if (present) {
 	wSoundPlay(WMSOUND_ICONIFY);
@@ -820,9 +829,11 @@ wDeiconifyWindow(WWindow  *wwin)
 #endif
     
     if (wPreferences.auto_arrange_icons) {
-        wArrangeIcons(wwin->screen_ptr);
+        wArrangeIcons(wwin->screen_ptr, True);
     }
 
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+
 #ifdef WMSOUND
     wSoundPlay(WMSOUND_DEICONIFY);
 #endif
@@ -840,6 +851,7 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
 	XUnmapWindow(dpy, wwin->icon->core->window);
 	wwin->flags.hidden = 1;
 	wwin->icon->mapped = 0;
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
 	return;
     }
 
@@ -886,6 +898,8 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
 #endif
     wwin->flags.skip_next_animation = 0;
 
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+
 #ifdef WMSOUND
     wSoundPlay(WMSOUND_HIDE);
 #endif
@@ -918,7 +932,7 @@ wHideOtherApplications(WWindow *awin)
             && !(wwin->flags.miniaturized||wwin->flags.hidden)
             && !wwin->flags.internal_window
             && (!wwin->flags.inspector_open || wwin->inspector->frame!=awin)
-	    && wwin->window_flags.hide_others) {
+	    && !wwin->window_flags.no_hide_others) {
 	    
 #ifdef REDUCE_APPICONS
 	    match = 0;
@@ -934,7 +948,7 @@ wHideOtherApplications(WWindow *awin)
 #endif
 
 	    if (wwin->main_window==None) {
-		if (wwin->window_flags.miniaturizable) {
+		if (!wwin->window_flags.no_miniaturizable) {
 		    wwin->flags.skip_next_animation = 1;
 		    wIconifyWindow(wwin);
 		}
@@ -949,7 +963,7 @@ wHideOtherApplications(WWindow *awin)
 		    tapp->flags.skip_next_animation = 1;
 		    wHideApplication(tapp);
 		} else {
-		    if (wwin->window_flags.miniaturizable) {
+		    if (!wwin->window_flags.no_miniaturizable) {
 		    	wwin->flags.skip_next_animation = 1;
 		   	 wIconifyWindow(wwin);
 		    }
@@ -1001,9 +1015,7 @@ wHideApplication(WApplication *wapp)
     else
 	wapp->last_focused = NULL;
     while (wlist) {
-#ifndef REDUCE_APPICONS
-	if (wlist->main_window == wapp->main_window) {
-#else
+#ifdef REDUCE_APPICONS
 	matchwmhints = matchworkspace = 0;
 	if (!nowmhints) {
 	    tapp = wApplicationOf(wlist->main_window);
@@ -1029,6 +1041,11 @@ wHideApplication(WApplication *wapp)
 	}
         if ((wlist->main_window == wapp->main_window || matchwmhints) && 
 	    matchworkspace) {
+#ifdef I_HATE_THIS
+	}
+#endif
+#else
+	if (wlist->main_window == wapp->main_window) {
 #endif
 	    if (wlist->flags.focused) {
 		hadfocus = 1;
@@ -1047,7 +1064,7 @@ wHideApplication(WApplication *wapp)
 	if (wPreferences.focus_mode==WKF_CLICK) {
 	    wlist = scr->focused_window;
 	    while (wlist) {
-		if (wlist->window_flags.focusable && !wlist->flags.hidden
+		if (!wlist->window_flags.no_focusable && !wlist->flags.hidden
 		    && (wlist->flags.mapped || wlist->flags.shaded))
 		  break;
 		wlist = wlist->prev;
@@ -1065,9 +1082,11 @@ wHideApplication(WApplication *wapp)
 
 
 static void
-unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
+unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
+	     int bringToCurrentWS)
 {
-    wWindowChangeWorkspace(wwin, wwin->screen_ptr->current_workspace);
+    if (bringToCurrentWS)
+	wWindowChangeWorkspace(wwin, wwin->screen_ptr->current_workspace);
 
     wwin->flags.hidden=0;
     wwin->flags.mapped=1;
@@ -1098,6 +1117,8 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
         wClientSetState(pwin, NormalState, None);
     }
 
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
+
 #ifdef WMSOUND
     wSoundPlay(WMSOUND_UNHIDE);
 #endif
@@ -1169,6 +1190,7 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
 			wlist->icon->mapped = 1;
 		    }
 		    wlist->flags.hidden = 0;
+		    UpdateSwitchMenu(scr, wlist, ACTION_CHANGE_STATE);
 		    if (wlist->frame->workspace != scr->current_workspace)
 			wWindowChangeWorkspace(wlist, scr->current_workspace);
 		}
@@ -1178,7 +1200,8 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
 	    } else if (wlist->flags.hidden) {
 		unhideWindow(wapp->app_icon->icon, wapp->app_icon->x_pos,
 			     wapp->app_icon->y_pos, wlist, 
-			     wapp->flags.skip_next_animation);
+			     wapp->flags.skip_next_animation,
+			     bringToCurrentWS);
 	    } else {
 		if (bringToCurrentWS
 		    && wlist->frame->workspace != scr->current_workspace) {
@@ -1195,11 +1218,11 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
     
     if (focused)
 	wSetFocusTo(scr, focused);
-    else if (wapp->last_focused)
+    else if (wapp->last_focused && wapp->last_focused->flags.mapped)
 	wSetFocusTo(scr, wapp->last_focused);
     wapp->last_focused = NULL;
     if (wPreferences.auto_arrange_icons) {
-        wArrangeIcons(scr);
+        wArrangeIcons(scr, True);
     }
 }
 
@@ -1255,7 +1278,7 @@ wRefreshDesktop(WScreen *scr)
 
 
 void
-wArrangeIcons(WScreen *scr)
+wArrangeIcons(WScreen *scr, Bool arrangeAll)
 {
     WWindow *wwin = scr->focused_window;
     WAppIcon *aicon;
@@ -1323,22 +1346,25 @@ wArrangeIcons(WScreen *scr)
             (wwin->frame->workspace==scr->current_workspace ||
              wwin->window_flags.omnipresent ||
              wPreferences.sticky_icons)) {
-	    if (wwin->icon_x != x || wwin->icon_y != y) {
+	    
+	    if (arrangeAll || !wwin->flags.icon_moved) {
+		if (wwin->icon_x != x || wwin->icon_y != y) {
 #ifdef ANIMATIONS
-		if (wPreferences.no_animations) {
-		    XMoveWindow(dpy, wwin->icon->core->window, x, y);
-		} else {
-		    SlideWindow(wwin->icon->core->window, wwin->icon_x, 
-				wwin->icon_y, x, y);
-		}
+		    if (wPreferences.no_animations) {
+			XMoveWindow(dpy, wwin->icon->core->window, x, y);
+		    } else {
+			SlideWindow(wwin->icon->core->window, wwin->icon_x, 
+				    wwin->icon_y, x, y);
+		    }
 #else
-		XMoveWindow(dpy, wwin->icon->core->window, x, y);
+		    XMoveWindow(dpy, wwin->icon->core->window, x, y);
 #endif /* ANIMATIONS */
+		}
+		wwin->icon_x = x;
+		wwin->icon_y = y;
+		wwin->flags.icon_moved = 0;
+		x += wPreferences.icon_size;
 	    }
-	    wwin->icon_x = x;
-	    wwin->icon_y = y;
-	    wwin->flags.icon_moved = 0;
-	    x += wPreferences.icon_size;
 	}
         /* we reversed the order, so we use next */
         wwin = wwin->next;
