@@ -60,25 +60,13 @@ static void iconDblClick(WObjDescriptor *desc, XEvent *event);
 static void iconExpose(WObjDescriptor *desc, XEvent *event);
 
 
-/*
- *---------------------------------------------------------------------- 
- * wAppIconCreateFromIcon--
- * 	Create an icon for use in the Dock, by "converting" a normal
- * icon to a dock icon. Tries to guess how the app is launched and
- * if it can't, fail.
- * 
- * Returns:
- * 	A WAppIcon structure or NULL, if the icon can't be docked.
- * 
- *---------------------------------------------------------------------- 
- */
+
 WAppIcon*
-wAppIconCreateFromIcon(WIcon *icon)
+wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
+		      char *wm_class)
 {
     WAppIcon *dicon;
-    WScreen *scr = icon->core->screen_ptr;
-    char **argv;
-    int argc;
+    char *path;
 
     dicon = wmalloc(sizeof(WAppIcon));
     wretain(dicon);
@@ -93,57 +81,20 @@ wAppIconCreateFromIcon(WIcon *icon)
     }
     scr->app_icon_list = dicon;
 
-    dicon->icon = icon;
-
-    /* check WM_COMMAND */
-    if (!wApplicationOf(icon->owner->main_window)) {
-	dicon->main_window = icon->owner->client_win;
-    } else {
-	dicon->main_window = icon->owner->main_window;
+    if (command) {
+	dicon->command = wstrdup(command);	
     }
-    if (!XGetCommand(dpy, dicon->main_window, &argv, &argc) || !argv) {
-	wrelease(dicon);
-	return NULL;
-    }
-    dicon->command = FlattenStringList(argv, argc);
-    XFreeStringList(argv);
-    dicon->wm_class = wstrdup(icon->owner->wm_class);
-    dicon->wm_instance = wstrdup(icon->owner->wm_instance);
-#ifdef REDUCE_APPICONS
-    dicon->num_apps = 0;
-#endif
-    return dicon;
-}
-
-
-WAppIcon*
-wAppIconCreateForCommand(WScreen *scr, char *command, char *wm_instance,
-			 char *wm_class)
-{
-    WAppIcon *dicon;
-
-    dicon = wmalloc(sizeof(WAppIcon));
-    wretain(dicon);
-    memset(dicon, 0, sizeof(WAppIcon));
-    dicon->yindex = -1;
-    dicon->xindex = -1;
-
-    dicon->prev = NULL;
-    dicon->next = scr->app_icon_list;
-    if (scr->app_icon_list) {
-        scr->app_icon_list->prev = dicon;
-    }
-    scr->app_icon_list = dicon;
-
-    if (command)
-	dicon->command = wstrdup(command);
-
     if (wm_class)
       dicon->wm_class = wstrdup(wm_class);
     if (wm_instance)
       dicon->wm_instance = wstrdup(wm_instance);
 
-    dicon->icon = wIconCreateWithName(scr, wm_instance, wm_class);
+    path = wDefaultGetIconFile(scr, wm_instance, wm_class);
+    if (path)
+	path = FindImage(wPreferences.icon_path, path);
+    dicon->icon = wIconCreateWithIconFile(scr, path);
+    if (path)
+	free(path);
 #ifdef REDUCE_APPICONS
     dicon->num_apps = 0;
 #endif
@@ -687,27 +638,29 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
     if ((desc->click_timestamp>0) &&
 	(event->xbutton.time-desc->click_timestamp <= wPreferences.dblclick_time)) {
-	desc->click_timestamp = -1;
-	
-	iconDblClick(desc, event);
+	if (event->xbutton.button == desc->click_button) {
+ 	    desc->click_timestamp = -1;
+ 	    iconDblClick(desc, event);
+ 	}
 	return;
     }
     desc->click_timestamp = event->xbutton.time;
+    desc->click_button = event->xbutton.button;
 
     if (event->xbutton.button == Button3) {
 	WObjDescriptor *desc;
 	WApplication *wapp = wApplicationOf(aicon->icon->owner->main_window);
-	
-	openApplicationMenu(wapp, event->xbutton.x_root, event->xbutton.y_root);
-	
-	/* allow drag select */
+
+	openApplicationMenu(wapp, event->xbutton.x_root, 
+			    event->xbutton.y_root);
+
+	/* allow drag select of menu */
 	desc = &scr->icon_menu->menu->descriptor;
-	event->xany.send_event = True;
+	event->xbutton.send_event = True;
 	(*desc->handle_mousedown)(desc, event);
-	
 	return;
     }
-	
+
 #ifdef DEBUG
     puts("Moving icon");
 #endif
@@ -718,8 +671,8 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
     
     if (XGrabPointer(dpy, icon->core->window, True, ButtonMotionMask
-		     |ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None,
-		     None, CurrentTime) !=GrabSuccess) {
+		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
 	wwarning("pointer grab failed for appicon move");
     }
 
@@ -730,8 +683,8 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
 
     while (!done) {
-	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonMotionMask
-		   |ExposureMask, &ev);
+	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
+		    |ButtonMotionMask|ExposureMask, &ev);
 	switch (ev.type) {
 	 case Expose:
 	    WMHandleEvent(&ev);
@@ -742,7 +695,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					     |ButtonReleaseMask, 
+					    |ButtonReleaseMask|ButtonPressMask, 
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -835,7 +788,12 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
 	    break;
 
+	 case ButtonPress:
+	    break;
+
 	 case ButtonRelease:
+	    if (ev.xbutton.button != desc->click_button)
+		break;
 	    XUngrabPointer(dpy, CurrentTime);
 
             if (docking) {
