@@ -306,8 +306,12 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
     switch (event->atom) {
      case XA_WM_NAME:
 	/* window title was changed */
-	
-	if (wwin->frame && MyXFetchName(dpy, wwin->client_win, &tmp)) {
+
+	if (wwin->frame) {
+	    if (!wFetchName(dpy, wwin->client_win, &tmp)) {
+		/* the hint was removed */
+		tmp = wstrdup(DEF_WINDOW_TITLE);
+	    }
 	    wFrameWindowChangeTitle(wwin->frame, tmp);
 	    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
 	}
@@ -320,7 +324,7 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    char *new_title;
 	    
 	    /* icon title was changed */
-	    MyXGetIconName(dpy, wwin->client_win, &new_title);
+	    wGetIconName(dpy, wwin->client_win, &new_title);
 	    wIconChangeTitle(wwin->icon, new_title);
 	}
 	break;
@@ -519,15 +523,15 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		}
 		wwin->transient_for = new_owner;
 		if (new_owner==None) {
-		    if (!wwin->window_flags.miniaturizable) {
-			wwin->window_flags.miniaturizable = 1;
-			wwin->window_flags.miniaturize_button = 1;
+		    if (wwin->window_flags.no_miniaturizable) {
+			wwin->window_flags.no_miniaturizable = 0;
+			wwin->window_flags.no_miniaturize_button = 0;
 			if (wwin->frame)
 			    wWindowConfigureBorders(wwin);
 		    }
-		} else if (wwin->window_flags.miniaturizable) {
-		    wwin->window_flags.miniaturizable = 0;
-		    wwin->window_flags.miniaturize_button = 0;
+		} else if (!wwin->window_flags.no_miniaturizable) {
+		    wwin->window_flags.no_miniaturizable = 1;
+		    wwin->window_flags.no_miniaturize_button = 1;
 		    if (wwin->frame)
 			wWindowConfigureBorders(wwin);
 		}
@@ -539,9 +543,9 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	if (event->atom==_XA_WM_PROTOCOLS) {
 	    PropGetProtocols(wwin->client_win, &wwin->protocols);
 	    if (wwin->protocols.DELETE_WINDOW) {
-		wwin->window_flags.broken_close = 0;
+		wwin->window_flags.kill_close = 0;
 	    } else {
-		wwin->window_flags.broken_close = 1;		
+		wwin->window_flags.kill_close = 1;		
 	    }
 	    if (wwin->frame)
 		wWindowUpdateButtonImages(wwin);
