@@ -109,6 +109,7 @@ static int getColor();
 static int getKeybind();
 static int getModMask();
 static int getSpeed();
+static int getMButton();
 
 /* value setting functions */
 static int setJustify();
@@ -138,7 +139,6 @@ static int setKeyGrab();
 
 static int setClipTitleFont();
 static int setClipTitleColor();
-static int setClipColor();
 
 /*
  * Other points of interaction with the defaults DB:
@@ -191,19 +191,25 @@ WDefaultEntry staticOptionList[] = {
 
 WDefaultEntry optionList[] = {
       /* dynamic options */
-    {"SwapMenuButton",	"NO",			NULL,
-	  &wPreferences.swap_menu_buttons,getBool,	NULL
+    {"SelectWindowsMouseButton", "Left",	NULL,
+	  &wPreferences.select_button, 	getMButton,	NULL
+    },
+    {"WindowListMouseButton", "Middle",		NULL,
+	  &wPreferences.windowl_button, getMButton,	NULL
+    },
+    {"ApplicationMenuMouseButton", "Right",	NULL,
+	  &wPreferences.menu_button,	getMButton,	NULL
     },
     {"PixmapPath",	DEF_PIXMAP_PATHS,	NULL,
 	  &wPreferences.pixmap_path,	getPathList,	NULL
     },
-    {"IconPath",	DEF_ICON_PATHS,	NULL,
+    {"IconPath",	DEF_ICON_PATHS,		NULL,
 	  &wPreferences.icon_path,	getPathList,	NULL
     },
     {"ColormapMode",	"auto",			(void*)True,
 	  &wPreferences.colormap_mode,	getFocusMode,	NULL
     },
-    {"AutoFocus", 	"YES",			NULL,
+    {"AutoFocus", 	"NO",			NULL,
 	  &wPreferences.auto_focus,	getBool,	NULL
     },
     {"RaiseDelay",	"0",			NULL,
@@ -215,9 +221,6 @@ WDefaultEntry optionList[] = {
     {"Superfluous",	"NO",			NULL,
 	  &wPreferences.superfluous, 	getBool, 	NULL
     },
-    {"WorkspaceChangeDelay", "700",		NULL,
-	  &wPreferences.ws_change_delay, getInt,        NULL
-    },
     {"AdvanceToNewWorkspace", "NO",		NULL,
 	  &wPreferences.ws_advance,      getBool,       NULL
     },
@@ -233,7 +236,7 @@ WDefaultEntry optionList[] = {
     {"WrapMenus", "NO",				NULL,
 	  &wPreferences.wrap_menus,	getBool,	NULL
     },
-    {"ScrollableMenus",	"YES",			NULL,
+    {"ScrollableMenus",	"NO",			NULL,
 	  &wPreferences.scrollable_menus, getBool,	NULL
     },
     {"MenuScrollSpeed",	"medium",    		NULL,
@@ -251,7 +254,7 @@ WDefaultEntry optionList[] = {
     {"AlignSubmenus",	"NO",			NULL,
 	  &wPreferences.align_menus,	getBool,	NULL
     },
-    {"OnTopTransients",	"YES",			NULL,
+    {"OnTopTransients",	"NO",			NULL,
 	  &wPreferences.on_top_transients, getBool,	NULL
     },
     {"WindowPlacement",	"auto",		NULL,
@@ -287,11 +290,6 @@ WDefaultEntry optionList[] = {
     {"WindowPlaceOrigin", "(0, 0)",		NULL,
 	  &wPreferences.window_place_origin, getCoord,	NULL
     },
-/* TODO: finish removing this option
-    {"ConstrainWindowSize", "NO",		NULL,
-	  &wPreferences.constrain_window_size, getBool,	NULL
-    },
-*/
     {"ResizeDisplay",	"corner",	       	NULL,
 	  &wPreferences.size_display,	getGeomDisp,	NULL
     },
@@ -341,15 +339,9 @@ WDefaultEntry optionList[] = {
     {"ClipTitleColor",	"black",		(void*)CLIP_NORMAL,
 	  NULL,				getColor,	setClipTitleColor
     },
-    {"CClipTitleColor", "\"#616161\"",		(void*)CLIP_COLLAPSED,
+    {"CClipTitleColor", "\"#454045\"",		(void*)CLIP_COLLAPSED,
 	  NULL,				getColor,	setClipTitleColor
     },
-    {"AClipColor",      DEF_CLIP_ACTIVE,	(void*)CLIP_ACTIVE,
-	  NULL,				getColor,	setClipColor
-    },
-    {"IClipColor",      DEF_CLIP_INACTIVE,	(void*)CLIP_INACTIVE,
-	  NULL,				getColor,	setClipColor
-    },
     {"FTitleColor",	"white",		(void*)WS_FOCUSED,
 	  NULL,				getColor,	setWTitleColor
     },
@@ -430,6 +422,9 @@ WDefaultEntry optionList[] = {
     {"ShadeKey", "None",			(void*)WKBD_SHADE,
 	  NULL,				getKeybind,	setKeyGrab
     },
+    {"SelectKey", "None",			(void*)WKBD_SELECT,
+	  NULL,				getKeybind,	setKeyGrab
+    },
     {"FocusNextKey", "None",		       	(void*)WKBD_FOCUSNEXT,
 	  NULL,				getKeybind,	setKeyGrab
     },
@@ -580,29 +575,6 @@ wDefaultsDestroyDomain(WDDomain *domain)
     free(domain);
 }
 
-char*
-wDefaultsPathForDomain(char *domain)
-{
-    char path[PATH_MAX];
-    char *gspath;
-
-    gspath = getenv("GNUSTEP_USER_ROOT");
-    if (gspath) {
-	strcpy(path, gspath);
-	strcat(path, "/");
-    } else {
-	strcpy(path, wgethomedir());
-	strcat(path, "/GNUstep/");
-    }
-    strcat(path, DEFAULTS_DIR);
-    strcat(path, "/");
-    strcat(path, domain);
-
-    gspath = wstrdup(path);
-
-    return gspath;
-}
-
 
 WDDomain*
 wDefaultsInitDomain(char *domain, Bool requireDictionary)
@@ -611,7 +583,6 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
     struct stat stbuf;
     static int inited = 0;
     char path[PATH_MAX];
-    int global = 0;
     char *the_path;
     proplist_t shared_dict=NULL;
 
@@ -623,8 +594,9 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
     db = wmalloc(sizeof(WDDomain));
     memset(db, 0, sizeof(WDDomain));
     db->domain_name = domain;
-    db->path = wDefaultsPathForDomain(domain);
-    the_path = NULL;
+    db->path = wdefaultspathfordomain(domain);
+    the_path = db->path;
+#if 0
     if (access(db->path, R_OK)!=0) {
 	char *gsroot;
 	
@@ -641,13 +613,12 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
         strcat(path, domain);
 	
 	if (access(path, R_OK)==0) {
-	    global = True;
 	    the_path = (char*)path;
 	}
     } else {
 	the_path = db->path;
     }
-    
+#endif    
     
     if (the_path && stat(the_path, &stbuf)>=0) {
 	db->dictionary = ReadProplistFromFile(the_path);
@@ -744,9 +715,15 @@ wDefaultsCheckDomains(void *screen)
     WScreen *scr = (WScreen*)screen;
     struct stat stbuf;
     proplist_t dict;
-    
+
+#ifdef HEARTBEAT
+    puts("Checking domains...");
+#endif
     if (stat(scr->dmain->path, &stbuf)>=0
 	&& scr->dmain->timestamp < stbuf.st_mtime) {
+#ifdef HEARTBEAT
+	puts("Checking WindowMaker domain");
+#endif
 	dict = ReadProplistFromFile(scr->dmain->path);
 	if (dict) {
 	    if (!PLIsDictionary(dict)) {
@@ -769,6 +746,9 @@ wDefaultsCheckDomains(void *screen)
 
     if (stat(scr->dwindow->path, &stbuf)>=0
 	&& scr->dwindow->timestamp < stbuf.st_mtime) {
+#ifdef HEARTBEAT
+	puts("Checking WMWindowAttributes domain");
+#endif
 	dict = ReadProplistFromFile(scr->dwindow->path);
 	if (dict) {
 	    if (!PLIsDictionary(dict)) {
@@ -793,8 +773,11 @@ wDefaultsCheckDomains(void *screen)
     if (stat(scr->droot_menu->path, &stbuf)>=0
 	&& scr->droot_menu->timestamp < stbuf.st_mtime) {
 	dict = ReadProplistFromFile(scr->droot_menu->path);
+#ifdef HEARTBEAT
+	puts("Checking WMRootMenu domain");
+#endif
 	if (dict) {
-	    if (!PLIsDictionary(dict) || !PLIsString(dict)) {
+	    if (!PLIsArray(dict) && !PLIsString(dict)) {
 		PLRelease(dict);
 		dict = NULL;
 		wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
@@ -811,6 +794,7 @@ wDefaultsCheckDomains(void *screen)
 	scr->droot_menu->timestamp = stbuf.st_mtime;
     }
 
+
     WMAddTimerHandler(DEFAULTS_CHECK_INTERVAL, wDefaultsCheckDomains, screen);
 }
 
@@ -1075,27 +1059,31 @@ getBool(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     val = PLGetString(value);
 
 again:
-    if ((val[1]=='\0' && (val[0]=='y' || val[0]=='Y' || val[0]=='T' 
-			  || val[0]=='t' || val[0]=='1'))
-	|| (strcasecmp(val, "YES")==0 || strcasecmp(val, "TRUE")==0)) {
+    if ((val[1]=='\0' && (val[0]=='y' || val[0]=='Y'))
+	|| strcasecmp(val, "YES")==0) {
 
 	data = 1;
-    } else if ((val[1]=='\0' 
-	      && (val[0]=='n' || val[0]=='N' || val[0]=='F' 
-		  || val[0]=='f' || val[0]=='0')) 
-	     || (strcasecmp(val, "NO")==0 || strcasecmp(val, "FALSE")==0)) {
-
+    } else if ((val[1]=='\0' && (val[0]=='n' || val[0]=='N'))
+		|| strcasecmp(val, "NO")==0) {
       data = 0;
     } else {
-        wwarning(_("can't convert \"%s\" to boolean for key \"%s\""),
-                 val, PLGetString(entry->plkey));
-        if (second_pass==0) {
-            val = PLGetString(entry->plvalue);
-            second_pass = 1;
-            wwarning(_("using default \"%s\" instead"), val);
-            goto again;
-        }
-	return False;
+	int i;
+	if (sscanf(val, "%i", &i)==1) {
+	    if (i!=0)
+		data = 1;
+	    else
+		data = 0;
+	} else {
+	    wwarning(_("can't convert \"%s\" to boolean for key \"%s\""),
+		     val, PLGetString(entry->plkey));
+	    if (second_pass==0) {
+		val = PLGetString(entry->plvalue);
+		second_pass = 1;
+		wwarning(_("using default \"%s\" instead"), val);
+		goto again;
+	    }
+	    return False;
+	}
     }
     
     if (ret)
@@ -1490,6 +1478,54 @@ again:
 }
 
 
+static int 
+getMButton(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr, 
+	     void **ret)
+{
+    static char data;
+    char *val;
+    int second_pass=0;
+
+    STRINGP("one of Left, Middle, Right or Button1 through Button5");
+
+    val = PLGetString(value);
+
+again:
+    if (strcasecmp(val, "left")==0 || strcasecmp(val, "button1")==0)
+      data = Button1;
+    else if (strcasecmp(val, "middle")==0 || strcasecmp(val, "button2")==0)
+      data = Button2;
+    else if (strcasecmp(val, "right")==0 || strcasecmp(val, "button3")==0)
+      data = Button3;
+    else if (strcasecmp(val, "button4")==0)
+      data = Button4;
+    else if (strcasecmp(val, "button5")==0)
+      data = Button4;
+    else {
+	wwarning(_("Invalid mouse button \"%s\". "
+		   "Should be Left, Middle, Right or Button1 through Button5"),
+		 PLGetString(value));
+        if (second_pass==0) {
+            val = PLGetString(entry->plvalue);
+            second_pass = 1;
+            wwarning(_("using default \"%s\" instead"), val);
+            goto again;
+        }
+	return False;
+    }
+    
+    if (ret)
+      *ret = &data;
+    
+    if (addr)
+      *(char*)addr = data;
+    
+    return True;    
+}
+
+
+
+
 /*
  * (solid <color>)
  * (hgradient <color> <color>)
@@ -1843,6 +1879,8 @@ again:
         file = FindImage(wPreferences.pixmap_path, val);
         if (file) {
             if (fork()==0) {
+		close(ConnectionNumber(dpy));
+		
                 execlp(program, program, style, "-c", cpc, file, NULL);
                 wwarning(_("could not run \"%s\""), program);
                 exit(-1);
@@ -2107,7 +2145,7 @@ setStickyIcons(WScreen *scr, WDefaultEntry *entry, void *bar, void *foo)
 {
     if (scr->workspaces) {
         wWorkspaceForceChange(scr, scr->current_workspace);
-        wArrangeIcons(scr);
+        wArrangeIcons(scr, False);
     }
     return 0;
 }
@@ -2166,6 +2204,9 @@ setIconTile(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
     }
     scr->icon_back_texture = wTextureMakeSolid(scr, &((*texture)->any.color));
     
+    FREE_PIXMAP(scr->clip_left_button);
+    FREE_PIXMAP(scr->clip_right_button);
+    
     /*
      * Free the texture as nobody else will use it, nor refer to it.
      */
@@ -2334,20 +2375,6 @@ setClipTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 }
 
 
-static int
-setClipColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
-{
-    if (scr->clip_pixel[index]!=scr->white_pixel &&
-	scr->clip_pixel[index]!=scr->black_pixel) {
-	wFreeColor(scr, scr->clip_pixel[index]);
-    }
-
-    scr->clip_pixel[index] = color->pixel;
-
-    return REFRESH_FORE_COLOR;
-}
-
-
 static int
 setWTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 {
@@ -2607,10 +2634,10 @@ setKeyGrab(WScreen *scr, WDefaultEntry *entry, WShortKey *shortcut, long index)
     while (wwin!=NULL) {
 	XUngrabKey(dpy, AnyKey, AnyModifier, wwin->frame->core->window);
 
-	if (wwin->window_flags.bind_keys) {
+	if (!wwin->window_flags.no_bind_keys) {
 	    wWindowSetKeyGrabs(wwin);
 	}
-	wwin = wwin->next;
+	wwin = wwin->prev;
     }
     
     return 0;
