@@ -52,20 +52,6 @@ extern WPreferences wPreferences;
 
 
 
-typedef struct {
-    int done;
-    int result;
-} pair;
-
-
-static void
-handler(pid_t pid, unsigned int status, void *cdata)
-{
-    ((pair*)cdata)->result = (char)status;
-    ((pair*)cdata)->done = 1;
-}
-
-
 int
 wMessageDialog(WScreen *scr, char *title, char *message, int type)
 {
@@ -105,10 +91,10 @@ wMessageDialog(WScreen *scr, char *title, char *message, int type)
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
     
     wwin = wManageInternalWindow(scr, parent, None, NULL, 
-				 (scr->scr_width - 400)/2, 
+				 (scr->scr_width - 400)/2,
 				 (scr->scr_height - 180)/2, 400, 180);
+    wwin->client_leader = WMWidgetXID(panel->win);
 
-    
     WMMapWidget(panel->win);
 
     while (!panel->done) {
@@ -173,10 +159,11 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
 
     
     panel = WMCreateInputPanel(scr->wmscreen, title, message, *text,
-			_("OK"), _("Cancel"));
+			       _("OK"), _("Cancel"));
 
 
     parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 320, 160, 0, 0, 0);
+    XSelectInput(dpy, parent, KeyPressMask|KeyReleaseMask);
 
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
@@ -184,6 +171,8 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
 				 (scr->scr_width - 320)/2,
 				 (scr->scr_height - 160)/2, 320, 160);
 
+    wwin->client_leader = WMWidgetXID(panel->win);
+
     WMMapWidget(panel->win);
     
 
@@ -319,7 +308,7 @@ setViewedImage(IconPanel *panel, char *file)
 	char *msg;
 	char *tmp;
 
-	WMSetButtonDisabled(panel->okButton, True);
+	WMSetButtonEnabled(panel->okButton, False);
 	    
 	tmp = _("Could not load image file ");
 	msg = wmalloc(strlen(tmp)+strlen(file)+6);
@@ -331,7 +320,7 @@ setViewedImage(IconPanel *panel, char *file)
 	
 	WMSetLabelImage(panel->iconView, NULL);
     } else {
-	WMSetButtonDisabled(panel->okButton, False);
+	WMSetButtonEnabled(panel->okButton, True);
 	
 	WMSetLabelImage(panel->iconView, pixmap);
 	WMReleasePixmap(pixmap);
@@ -353,7 +342,7 @@ listCallback(void *self, void *data)
 
 	WMSetLabelImage(panel->iconView, NULL);
 
-	WMSetButtonDisabled(panel->okButton, True);
+	WMSetButtonEnabled(panel->okButton, False);
 
 	WMClearList(panel->iconList);
 	listPixmaps(panel->scr, panel->iconList, path);	
@@ -385,7 +374,8 @@ listIconPaths(WMList *lPtr)
     for (i=0; wPreferences.icon_path[i]!=NULL; i++) {
 	/* do not sort, because the order implies the order of
 	 * directories searched */
-	WMAddListItem(lPtr, wPreferences.icon_path[i]);
+	if (access(wPreferences.icon_path[i], X_OK)==0)
+	    WMAddListItem(lPtr, wPreferences.icon_path[i]);
     }
 }
 
@@ -464,12 +454,12 @@ wIconChooserDialog(WScreen *scr, char **file)
     color = WMWhiteColor(scr->wmscreen);
     WMSetLabelTextColor(panel->dirLabel, color); 
     WMSetLabelTextColor(panel->iconLabel, color);
-    WMReleaseColor(scr->wmscreen, color);
+    WMReleaseColor(color);
 
     color = WMDarkGrayColor(scr->wmscreen);
     WMSetWidgetBackgroundColor(panel->iconLabel, color);
     WMSetWidgetBackgroundColor(panel->dirLabel, color); 
-    WMReleaseColor(scr->wmscreen, color);
+    WMReleaseColor(color);
 
     WMSetLabelRelief(panel->iconLabel, WRSunken);
 
@@ -497,13 +487,13 @@ wIconChooserDialog(WScreen *scr, char **file)
     panel->fileField = WMCreateTextField(panel->win);
     WMResizeWidget(panel->fileField, 365, 20);
     WMMoveWidget(panel->fileField, 75, 210);
-    WMSetTextFieldDisabled(panel->fileField, True);
+    WMSetTextFieldEnabled(panel->fileField, False);
     
     panel->okButton = WMCreateCommandButton(panel->win);
     WMResizeWidget(panel->okButton, 80, 28);
     WMMoveWidget(panel->okButton, 360, 240);
     WMSetButtonText(panel->okButton, _("OK"));
-    WMSetButtonDisabled(panel->okButton, True);
+    WMSetButtonEnabled(panel->okButton, False);
     WMSetButtonAction(panel->okButton, buttonCallback, panel);
     
     panel->cancelButton = WMCreateCommandButton(panel->win);
@@ -687,7 +677,8 @@ wShowInfoPanel(WScreen *scr)
     WMMoveWidget(panel->copyrL, 20, 190);
     WMSetLabelTextAlignment(panel->copyrL, WALeft);
     WMSetLabelText(panel->copyrL, COPYRIGHT_TEXT);
-    font = WMSystemFontOfSize(scr->wmscreen, 10);
+    /* we want the (c) character in the helvetica font */
+    font = WMCreateFontInDefaultEncoding(scr->wmscreen, HELVETICA10_FONT);
     if (font) {
 	WMSetLabelFont(panel->copyrL, font);
 	WMReleaseFont(font);
@@ -703,8 +694,8 @@ wShowInfoPanel(WScreen *scr)
 				 (scr->scr_width - 470)/2,
 				 (scr->scr_height - 270)/2, 390, 237);
 
-    wwin->window_flags.closable = 1;
-    wwin->window_flags.close_button = 1;
+    wwin->window_flags.no_closable = 0;
+    wwin->window_flags.no_close_button = 0;
     wWindowUpdateButtonImages(wwin);
     wFrameWindowShowButton(wwin->frame, WFF_RIGHT_BUTTON);
     wwin->frame->on_click_right = destroyInfoPanel;
@@ -812,8 +803,8 @@ wShowLegalPanel(WScreen *scr)
 				 (scr->scr_width - 420)/2,
 				 (scr->scr_height - 250)/2, 420, 250);
 
-    wwin->window_flags.closable = 1;
-    wwin->window_flags.close_button = 1;
+    wwin->window_flags.no_closable = 0;
+    wwin->window_flags.no_close_button = 0;
     wWindowUpdateButtonImages(wwin);
     wFrameWindowShowButton(wwin->frame, WFF_RIGHT_BUTTON);
     wwin->frame->on_click_right = destroyLegalPanel;
