@@ -79,6 +79,12 @@ extern Atom _XA_DND_PROTOCOL;
 extern int ShapeEventBase;
 #endif
 
+/* special flags */
+extern char WRestartASAP;
+extern char WExitASAP;
+extern char WDelayedActionSet;
+
+
 /************ Local stuff ***********/
 
 static void saveTimestamp(XEvent *event);
@@ -305,19 +311,37 @@ DispatchEvent(XEvent *event)
     if (deathHandler)
 	handleDeadProcess(NULL);
     
-    if (wScreen->flags.exit_asap) {
-	/* received SIGTERM */
-	wScreenSaveState(wScreen);
-	
-	RestoreDesktop(wScreen);
-	exit(0);
-    } else if (wScreen->flags.restart_asap) {
-	/* received SIGHUP */
-	wScreenSaveState(wScreen);
+    if (WExitASAP) {
+	WExitASAP = 0;
 
-	RestoreDesktop(wScreen);
+	/* 
+	 * WMHandleEvent() can't be called from anything
+	 * executed inside here, or we can get in a infinite
+	 * recursive loop.
+	 */
+	if (wScreen) {
+	    /* received SIGTERM */
+	    wScreenSaveState(wScreen);
+	    
+	    RestoreDesktop(wScreen);
+	}
+	exit(0);
+    } else if (WRestartASAP) {
+	WRestartASAP = 0;
+	
+	if (wScreen) {
+	    /* received SIGHUP */
+	    wScreenSaveState(wScreen);
+	
+	    RestoreDesktop(wScreen);
+	}
 	Restart(NULL);
     }
+
+    /* for the case that all that is wanted to be dispatched is
+     * the stuff above */
+    if (!event)
+	return;
     
     saveTimestamp(event);    
     switch (event->type) {
@@ -585,28 +609,21 @@ handleMapRequest(Window window)
 
     if (wwin) {
 	int state;
-#if 0
-	if (wwin->window_flags.start_withdrawn) {
-	    wwin->window_flags.start_withdrawn = 0;
-	    state = WithdrawnState;
-	} else
-#endif
-	{
-	    if (wwin->wm_hints && (wwin->wm_hints->flags & StateHint))
-	      state = wwin->wm_hints->initial_state;
-	    else
-	      state = NormalState;
-        }
+	
+	if (wwin->wm_hints && (wwin->wm_hints->flags & StateHint))
+	    state = wwin->wm_hints->initial_state;
+	else
+	    state = NormalState;
 
         if (state==IconicState)
             wwin->flags.miniaturized = 1;
 
         if (state==WithdrawnState) {
-            wwin->flags.mapped = 0;
-            wClientSetState(wwin, WithdrawnState, None);
-            XSelectInput(dpy, wwin->client_win, NoEventMask);
-            XRemoveFromSaveSet(dpy, wwin->client_win);
-            wUnmanageWindow(wwin, True);
+	    wwin->flags.mapped = 0;
+	    wClientSetState(wwin, WithdrawnState, None);
+	    XSelectInput(dpy, wwin->client_win, NoEventMask);
+	    XRemoveFromSaveSet(dpy, wwin->client_win);
+	    wUnmanageWindow(wwin, True);
         } else {
             wClientSetState(wwin, NormalState, None);
             if (wwin->flags.shaded) {
@@ -726,16 +743,7 @@ handleButtonPress(XEvent *event)
     puts("got button press");
 #endif
     if (event->xbutton.window==wScreen->root_win) {
-	int menuButton, selectButton;
-	
-	if (wPreferences.swap_menu_buttons) {
-	    menuButton = Button1;
-	    selectButton = Button3;
-	} else {
-	    menuButton = Button3;
-	    selectButton = Button1;
-	}
-	if (event->xbutton.button==menuButton) {
+	if (event->xbutton.button==wPreferences.menu_button) {
 	    OpenRootMenu(wScreen, event->xbutton.x_root,
                          event->xbutton.y_root, False);
 	    /* ugly hack */
@@ -745,12 +753,8 @@ handleButtonPress(XEvent *event)
 		else
 		  event->xbutton.window = wScreen->root_menu->frame->core->window;
 	    }
-	} else if (event->xbutton.button==selectButton) {
-	    
-	    wUnselectWindows();
-	    wSelectWindows(wScreen, event);
-	    
-	} else if (event->xbutton.button==Button2) {
+	}
+	if (event->xbutton.button==wPreferences.windowl_button) {
 	    
 	    OpenSwitchMenu(wScreen, event->xbutton.x_root,
 			   event->xbutton.y_root, False);
@@ -761,6 +765,12 @@ handleButtonPress(XEvent *event)
 		  event->xbutton.window = wScreen->switch_menu->frame->core->window;
 	    }
 	}
+	if (event->xbutton.button==wPreferences.select_button) {
+	    
+	    wUnselectWindows();
+	    wSelectWindows(wScreen, event);
+	    
+	}
     }
     if (XFindContext(dpy, event->xbutton.window, wWinContext, 
 		 (XPointer *)&desc)==XCNOENT) {
@@ -783,9 +793,11 @@ handleButtonPress(XEvent *event)
     } else if (desc->parent_type == WCLASS_APPICON
 	       || desc->parent_type == WCLASS_MINIWINDOW
 	       || desc->parent_type == WCLASS_DOCK_ICON) {
-	XSync(dpy, 0);
-	XAllowEvents(dpy, AsyncPointer, CurrentTime);
-	XSync(dpy, 0);
+	if (event->xbutton.state & MOD_MASK) {
+	    XSync(dpy, 0);
+	    XAllowEvents(dpy, AsyncPointer, CurrentTime);
+	    XSync(dpy, 0);
+	}
     }
 
     if (desc->handle_mousedown!=NULL) {
@@ -845,6 +857,8 @@ handleUnmapNotify(XEvent *event)
 	return;
     }
     XGrabServer(dpy);
+    XUnmapWindow(dpy, wwin->frame->core->window);
+    wwin->flags.mapped = 0;
     XSync(dpy, 0);
     /* check if the window was destroyed */
     if (XCheckTypedWindowEvent(dpy, wwin->client_win, DestroyNotify,&ev)) {
@@ -952,7 +966,6 @@ handleClientMessage(XEvent *event)
 		 case WMFHideApplication:
 		    wHideApplication(wApplicationOf(wwin->main_window));
 		    break;
-	
 		}
 	    }
 	}
@@ -973,7 +986,7 @@ handleClientMessage(XEvent *event)
 	if (XFindContext(dpy, event->xbutton.window, wWinContext, 
 			 (XPointer *)&desc)!=XCNOENT) {
 	    struct WIcon *icon=NULL;
-	    
+
 	    if (desc->parent_type == WCLASS_MINIWINDOW) {
 		icon = (WIcon*)desc->parent;
 	    } else if (desc->parent_type == WCLASS_DOCK_ICON
@@ -996,7 +1009,7 @@ static void
 handleEnterNotify(XEvent *event)
 {
     WWindow *wwin;
-    static WMagicNumber *prev_window=NULL;
+    static WMHandlerID *prev_window=NULL;
     WObjDescriptor *desc;
 
 #ifdef DEBUG
@@ -1026,6 +1039,24 @@ handleEnterNotify(XEvent *event)
 
 	return;
     }
+    /* set focus if in focus-follows-mouse mode and the event
+     * is for the frame window and window doesn't have focus yet */
+    if ((wPreferences.focus_mode==WKF_POINTER
+	 || wPreferences.focus_mode==WKF_SLOPPY)
+	&& wwin->frame->core->window==event->xcrossing.window
+	&& !wwin->flags.focused) {
+	wSetFocusTo(wwin->screen_ptr, wwin);
+
+	if (prev_window)
+	    WMDeleteTimerHandler(prev_window);
+	prev_window = NULL;
+
+	if (wPreferences.raise_delay && !wwin->window_flags.no_focusable) {
+	    prev_window = WMAddTimerHandler(wPreferences.raise_delay,
+					   (WMCallback*)wRaiseFrame,
+					   wwin->frame->core);
+	}
+    }
     /* Install colormap for window, if the colormap installation mode
      * is colormap_follows_mouse */
     if (wPreferences.colormap_mode==WKF_POINTER) {
@@ -1034,23 +1065,6 @@ handleEnterNotify(XEvent *event)
 	else
 	    InstallWindowColormaps(NULL);
     }
-
-    /* set focus if in focus-follows-mouse mode and the event
-     * is for the frame window and window doesn't have focus yet */
-    if ((wPreferences.focus_mode==WKF_POINTER
-	 || wPreferences.focus_mode==WKF_SLOPPY)
-	&& wwin->frame->core->window==event->xcrossing.window
-	&& !wwin->flags.focused) {
-	WMDeleteTimerHandler(prev_window);
-	prev_window = NULL;
-
-	wSetFocusTo(wwin->screen_ptr, wwin);
-	if (wPreferences.raise_delay && wwin->window_flags.focusable) {
-	    prev_window = WMAddTimerHandler(wPreferences.raise_delay,
-					   (WMCallback*)wRaiseFrame,
-					   wwin->frame->core);
-	}
-    }
 }
 
 
@@ -1162,7 +1176,7 @@ handleKeyPress(XEvent *event)
     int command=-1;
 
     /* ignore CapsLock */
-    modifiers = event->xkey.state & ~IGNORE_MOD_MASK;
+    modifiers = event->xkey.state & VALID_MOD_MASK;
     
     for (i=0; i<WKBD_LAST; i++) {
 	if (wKeyBindings[i].keycode==0) 
@@ -1177,18 +1191,35 @@ handleKeyPress(XEvent *event)
     }
     
     if (command < 0) {
-	wRootMenuPerformShortcut(event);
+	if (!wRootMenuPerformShortcut(event)) {
+	    static int dontLoop = 0;
+	    
+	    if (dontLoop > 10) {
+		wwarning("problem with key event processing code");
+		return;
+	    }
+	    dontLoop++;
+	    /* if the focused window is an internal window, try redispatching
+	     * the event to the managed window, as it can be a WINGs window */
+	    if (wwin && wwin->flags.internal_window) {
+		/* client_leader contains the WINGs toplevel */
+		event->xany.window = wwin->client_leader;
+		WMHandleEvent(event);
+	    }
+	    dontLoop--;
+	}
 	return;
     }
 
 #define ISMAPPED(w) ((w) && !(w)->flags.miniaturized && ((w)->flags.mapped || (w)->flags.shaded))
+#define ISFOCUSED(w) ((w) && (w)->flags.focused)
 
     switch (command) {
      case WKBD_ROOTMENU:
 	OpenRootMenu(wScreen, event->xkey.x_root, event->xkey.y_root, True);
 	break;
      case WKBD_WINDOWMENU:
-	if (ISMAPPED(wwin))
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin))
 	    OpenWindowMenu(wwin, wwin->frame_x, 
 			   wwin->frame_y+wwin->frame->top_width, True);
 	break;
@@ -1196,29 +1227,31 @@ handleKeyPress(XEvent *event)
 	OpenSwitchMenu(wScreen, event->xkey.x_root, event->xkey.y_root, True);
 	break;
      case WKBD_MINIATURIZE:
-	if (ISMAPPED(wwin)) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
 	    CloseWindowMenu(wScreen);
 	    
 	    if (wwin->protocols.MINIATURIZE_WINDOW)
 	      	wClientSendProtocol(wwin, _XA_WINDOWMAKER_WM_MINIATURIZE_WINDOW,
 				    event->xbutton.time);
 	    else {
-		if (wwin->window_flags.miniaturizable)
+		if (!wwin->window_flags.no_miniaturizable)
 		    wIconifyWindow(wwin);
 	    }
 	}
 	break;
      case WKBD_HIDE:
-	if (ISMAPPED(wwin)) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
+	    WApplication *wapp = wApplicationOf(wwin->main_window);
 	    CloseWindowMenu(wScreen);
 	    
-	    if (wwin->main_window && wwin->window_flags.application) {
-		wHideApplication(wApplicationOf(wwin->main_window));
+	    if (wapp && !wapp->main_window_desc->window_flags.no_appicon) {
+		wHideApplication(wapp);
 	    }
 	}
 	break;
      case WKBD_MAXIMIZE:
-	if (ISMAPPED(wwin) && wwin->window_flags.resizable) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin) 
+	    && !wwin->window_flags.no_resizable) {
 	    CloseWindowMenu(wScreen);
 	    
 	    if (wwin->flags.maximized) {
@@ -1229,7 +1262,8 @@ handleKeyPress(XEvent *event)
 	}
 	break;
      case WKBD_VMAXIMIZE:
-	if (ISMAPPED(wwin) && wwin->window_flags.resizable) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)
+	    && !wwin->window_flags.no_resizable) {
 	    CloseWindowMenu(wScreen);
 	    
 	    if (wwin->flags.maximized) {
@@ -1240,14 +1274,14 @@ handleKeyPress(XEvent *event)
 	}
 	break;
      case WKBD_RAISE:
-	if (ISMAPPED(wwin)) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
 	    CloseWindowMenu(wScreen);
 	    
 	    wRaiseFrame(wwin->frame->core);
 	}
 	break;
      case WKBD_LOWER:
-	if (ISMAPPED(wwin)) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
 	    CloseWindowMenu(wScreen);
 
 	    wLowerFrame(wwin->frame->core);
@@ -1262,7 +1296,8 @@ handleKeyPress(XEvent *event)
 	    wRaiseLowerFrame(wwin->frame->core);
 	break;
      case WKBD_SHADE:
-	if (ISMAPPED(wwin) &&wwin->window_flags.shadeable) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)
+	    && !wwin->window_flags.no_shadeable) {
 	    if (wwin->flags.shaded)
 		wUnshadeWindow(wwin);
 	    else
@@ -1270,13 +1305,19 @@ handleKeyPress(XEvent *event)
 	}
 	break;
      case WKBD_CLOSE:
-	if (ISMAPPED(wwin) && wwin->window_flags.closable) {
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)
+	    && !wwin->window_flags.no_closable) {
 	    CloseWindowMenu(wScreen);
 	    if (wwin->protocols.DELETE_WINDOW)
 	      wClientSendProtocol(wwin, _XA_WM_DELETE_WINDOW,
 				  event->xkey.time);
 	}
 	break;
+    case WKBD_SELECT:
+	if (ISMAPPED(wwin) && ISFOCUSED(wwin)) {
+	    wSelectWindow(wwin);
+	}
+        break;
      case WKBD_FOCUSNEXT:
 	wwin = NextFocusWindow(wScreen);
         if (wwin != NULL) {
@@ -1374,7 +1415,8 @@ handleKeyPress(XEvent *event)
 }
 
 
-static void handleMotionNotify(XEvent *event)
+static void 
+handleMotionNotify(XEvent *event)
 {
     WMenu *menu;
 
