@@ -70,18 +70,17 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
     
     fwin->screen_ptr = scr;
     fwin->window_level = wlevel;
+
+    fwin->flags.single_texture = (flags & WFF_SINGLE_STATE) ? 1 : 0;
     
     fwin->title_texture = title_texture;
     fwin->resizebar_texture = resize_texture;
     fwin->title_pixel = color;
     fwin->title_gc = gc;
     fwin->font = font;
-
-    if (flags & WFF_IS_MENU)
-	fwin->flags.menu = 1;
     
     fwin->core = wCoreCreateTopLevel(scr, x, y, width, height, 
-				     fwin->flags.menu ? 0 : 1);
+				     FRAME_BORDER_WIDTH);
     if (wPreferences.use_saveunders) {
 	unsigned long vmask;
 	XSetWindowAttributes attribs;
@@ -114,7 +113,6 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
     int bsize;
     int width, height;
     int i;
-    int ofs = fwin->flags.menu;
     WScreen *scr = fwin->screen_ptr;
     
     width = fwin->core->width;
@@ -135,18 +133,18 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	/* if we had a titlebar and is requesting for one,
 	 * check if the size has changed and resize it */
 	if (flags & WFF_TITLEBAR) {
-	    fwin->top_width = theight + ofs;
+	    fwin->top_width = theight;
 
 	    fwin->flags.need_texture_remake = 1;
 	    
 	    if (wPreferences.new_style) {
 		if (fwin->left_button) {
-		    wCoreConfigure(fwin->left_button, 0, ofs, bsize, bsize);
+		    wCoreConfigure(fwin->left_button, 0, 0, bsize, bsize);
 		}
 		
 		if (fwin->right_button) {
-		    wCoreConfigure(fwin->right_button, width-bsize+1-ofs,
-				   ofs, bsize, bsize);
+		    wCoreConfigure(fwin->right_button, width-bsize+1,
+				   0, bsize, bsize);
 		}
 	    } else { /* !new_style */
 		if (fwin->left_button) {
@@ -155,14 +153,14 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 		}
 		
 		if (fwin->right_button) {
-		    wCoreConfigure(fwin->right_button, width-bsize-3-2*ofs,
+		    wCoreConfigure(fwin->right_button, width-bsize-3,
 				   (theight-bsize)/2, bsize, bsize);
 		}
 	    }
 	    updateTitlebar(fwin);
 	} else {
 	    /* we had a titlebar, but now we don't need it anymore */
-	    for (i=0; i < (fwin->flags.menu?1:3); i++) {
+	    for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 		FREE_PIXMAP(fwin->title_back[i]);
                 if (wPreferences.new_style) {
 		    FREE_PIXMAP(fwin->lbutton_back[i]);
@@ -189,13 +187,12 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    fwin->top_width = theight;
 	    
 	    fwin->flags.titlebar = 1;
-	    fwin->titlebar = wCoreCreate(fwin->core, ofs, ofs, width+1, 
-					 theight);
+	    fwin->titlebar = wCoreCreate(fwin->core, 0, 0, width+1, theight);
 	    
 	    if (flags & WFF_LEFT_BUTTON) {
 		fwin->flags.left_button = 1;
                 if (wPreferences.new_style) {
-                    fwin->left_button = wCoreCreate(fwin->core, ofs, ofs,
+                    fwin->left_button = wCoreCreate(fwin->core, 0, 0,
                                                     bsize, bsize);
 		    if (width < theight*4) {
 			fwin->flags.lbutton_dont_fit = 1;
@@ -223,11 +220,11 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 		fwin->flags.right_button = 1;
                 if (wPreferences.new_style) {
 		    fwin->right_button =
-			wCoreCreate(fwin->core, width-bsize+1-ofs, ofs,
+			wCoreCreate(fwin->core, width-bsize+1, 0,
 				    bsize, bsize);
                 } else {
 		    fwin->right_button =
-			wCoreCreate(fwin->titlebar, width-bsize-3-2*ofs,
+			wCoreCreate(fwin->titlebar, width-bsize-3,
 				    (theight-bsize)/2, bsize, bsize);
 		    XSetWindowBackground(dpy, fwin->right_button->window,
 					 scr->widget_texture->normal.pixel);
@@ -348,7 +345,7 @@ wFrameWindowDestroy(WFrameWindow *fwin)
     if (fwin->title)
 	free(fwin->title);
     
-    for (i=0; i < (fwin->flags.menu?1:3); i++) {
+    for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 	FREE_PIXMAP(fwin->title_back[i]);
         if (wPreferences.new_style) {
 	    FREE_PIXMAP(fwin->lbutton_back[i]);
@@ -382,7 +379,7 @@ updateTitlebar(WFrameWindow *fwin)
     theight = (*fwin->font)->height + TITLEBAR_EXTRA_HEIGHT;
 
     x = 0;
-    w = fwin->core->width - (fwin->flags.menu ? 2 : 0) + 1;
+    w = fwin->core->width + 1;
 
     if (wPreferences.new_style) {
 	if (fwin->flags.hide_left_button || !fwin->left_button
@@ -393,9 +390,6 @@ updateTitlebar(WFrameWindow *fwin)
 	    w -= fwin->left_button->width;
 	}
     }
-    if (fwin->flags.menu) {
-	x++;
-    }
     
     if (wPreferences.new_style) {
 	if (!fwin->flags.hide_right_button && fwin->right_button
@@ -407,7 +401,7 @@ updateTitlebar(WFrameWindow *fwin)
     if (wPreferences.new_style || fwin->titlebar->width!=w)
 	fwin->flags.need_texture_remake = 1;
 
-    wCoreConfigure(fwin->titlebar, x, fwin->flags.menu, w, theight);
+    wCoreConfigure(fwin->titlebar, x, 0, w, theight);
 }
 
 
@@ -589,7 +583,7 @@ wFrameWindowPaint(WFrameWindow *fwin)
 
 	fwin->flags.need_texture_remake = 0;
 
-	for (i=0; i < (fwin->flags.menu?1:3); i++) {
+	for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 	    if (fwin->title_texture[i] && fwin->titlebar) {
 		FREE_PIXMAP(fwin->title_back[i]);
                 if (wPreferences.new_style) {
@@ -607,10 +601,7 @@ wFrameWindowPaint(WFrameWindow *fwin)
 		    right = fwin->right_button && !fwin->flags.hide_right_button
 			&& !fwin->flags.rbutton_dont_fit;
 
-		    if (fwin->flags.menu)
-			width = fwin->core->width+1-2;
-		    else
-			width = fwin->core->width+1;
+		    width = fwin->core->width+1;
 		    
 		    renderTexture(fwin->screen_ptr, fwin->title_texture[i],
                                   width, fwin->titlebar->height,
@@ -784,7 +775,6 @@ void
 wFrameWindowResize(WFrameWindow *fwin, int width, int height)
 {
     int k = (wPreferences.new_style ? 4 : 3);
-    int ofs = 0;
     int resizedHorizontally = 0;
     
     XResizeWindow(dpy, fwin->core->window, width, height);
@@ -801,12 +791,6 @@ wFrameWindowResize(WFrameWindow *fwin, int width, int height)
     fwin->core->width = width;
     fwin->core->height = height;
 
-    if (fwin->flags.menu) {
-	if (!wPreferences.new_style)
-	    width -= 2;
-	ofs = 1;
-    }
-    
     if (fwin->titlebar && resizedHorizontally) {	
 	/* Check if the titlebar is wide enough to hold the buttons.
 	 * Temporarily remove them if can't
@@ -846,7 +830,7 @@ wFrameWindowResize(WFrameWindow *fwin, int width, int height)
         if (wPreferences.new_style) {	    
 	    if (fwin->right_button)
 		XMoveWindow(dpy, fwin->right_button->window, 
-			    width - fwin->right_button->width + 1 - ofs, ofs);
+			    width - fwin->right_button->width + 1, 0);
         } else {
 	    if (fwin->right_button)
 		XMoveWindow(dpy, fwin->right_button->window, 
@@ -933,14 +917,22 @@ paintButton(WCoreWindow *button, WTexture *texture, unsigned long color,
 		width = image->width;
 	    }
 	}
-	d=1;
 	XSetClipMask(dpy, copy_gc, None);
 	XSetForeground(dpy, copy_gc, scr->white_pixel);
-	XFillRectangle(dpy, button->window, copy_gc, 0, 0,
-		       button->width-1, button->height-1);
-	XSetForeground(dpy, copy_gc, scr->black_pixel);
-	XDrawRectangle(dpy, button->window, copy_gc, 0, 0,
-		       button->width-1, button->height-1);
+	d=1;
+	if (wPreferences.new_style) {
+	    XFillRectangle(dpy, button->window, copy_gc, 0, 0,
+			   button->width-1, button->height-1);
+	    XSetForeground(dpy, copy_gc, scr->black_pixel);
+	    XDrawRectangle(dpy, button->window, copy_gc, 0, 0,
+			   button->width-1, button->height-1);
+	} else {
+	    XFillRectangle(dpy, button->window, copy_gc, 0, 0,
+			   button->width, button->height);
+	    XSetForeground(dpy, copy_gc, scr->black_pixel);
+	    XDrawRectangle(dpy, button->window, copy_gc, 0, 0,
+			   button->width, button->height);
+	}
     } else {
 	XClearWindow(dpy, button->window);
 	
@@ -958,7 +950,7 @@ paintButton(WCoreWindow *button, WTexture *texture, unsigned long color,
 	    }
         } else {
 	    wDrawBevel(button, scr->widget_texture, WREL_RAISED);
-        }
+	}
     }
     
     if (image) {
@@ -969,12 +961,18 @@ paintButton(WCoreWindow *button, WTexture *texture, unsigned long color,
 	XSetClipOrigin(dpy, copy_gc, x-left, y);
         if (!wPreferences.new_style) {
 	    XSetForeground(dpy, copy_gc, scr->black_pixel);
-	    if (image->depth==1)
-		XCopyPlane(dpy, image->image, button->window, copy_gc,
-			   left, 0, width, image->height, x, y, 1);
-	    else
-		XCopyArea(dpy, image->image, button->window, copy_gc,
-			  left, 0, width, image->height, x, y);
+	    if (!pushed) {
+		if (image->depth==1)
+		    XCopyPlane(dpy, image->image, button->window, copy_gc,
+			       left, 0, width, image->height, x, y, 1);
+		else
+		    XCopyArea(dpy, image->image, button->window, copy_gc,
+			      left, 0, width, image->height, x, y);
+	    } else {
+		XSetForeground(dpy, copy_gc, scr->dark_pixel);
+		XFillRectangle(dpy, button->window, copy_gc, 0, 0,
+			       button->width, button->height);
+	    }
         } else {
 	    if (pushed) {
 		XSetForeground(dpy, copy_gc, scr->black_pixel);
@@ -1015,12 +1013,15 @@ titlebarMouseDown(WObjDescriptor *desc, XEvent *event)
     
     if ((desc->click_timestamp>0) 
 	&& (event->xbutton.time - desc->click_timestamp <= DBLCLICK_TIME)) {
-	desc->click_timestamp = -1;
-	
-	if (fwin->on_dblclick_titlebar)
-	    (*fwin->on_dblclick_titlebar)(titlebar, fwin->child, event);
+	if (event->xbutton.button == desc->click_button) {
+	    desc->click_timestamp = -1;
+
+	    if (fwin->on_dblclick_titlebar)
+	        (*fwin->on_dblclick_titlebar)(titlebar, fwin->child, event);
+	}
     } else {
 	desc->click_timestamp = event->xbutton.time;
+	desc->click_button = event->xbutton.button;
 	if (fwin->on_mousedown_titlebar)
 	    (*fwin->on_mousedown_titlebar)(titlebar, fwin->child, event);
     }
@@ -1048,9 +1049,11 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
     int done=0, execute=1;
     WTexture *texture;
     unsigned long pixel;
-    
-    
-    if (event->xbutton.time - desc->click_timestamp <= DBLCLICK_TIME) {
+
+    if ((desc->click_timestamp > 0) &&
+	(event->xbutton.time - desc->click_timestamp <= DBLCLICK_TIME) &&
+	(event->xbutton.button == desc->click_button)) {
+
 	desc->click_timestamp = -1;
 	if (button == fwin->right_button && fwin->on_dblclick_right) {
 	    (*fwin->on_dblclick_right)(button, fwin->child, event);
@@ -1059,6 +1062,7 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
 	return;
     } 
     desc->click_timestamp = event->xbutton.time;
+    desc->click_button = event->xbutton.button;
     
     if (button == fwin->left_button) {
 	image = fwin->lbutton_image;
@@ -1072,7 +1076,7 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
     
     while (!done) {
 	WMMaskEvent(dpy, LeaveWindowMask|EnterWindowMask|ButtonReleaseMask
-		    |ExposureMask, &ev);
+		    |ButtonPressMask|ExposureMask, &ev);
 	switch (ev.type) {
 	 case LeaveNotify:
 	    execute = 0;
@@ -1083,9 +1087,13 @@ buttonMouseDown(WObjDescriptor *desc, XEvent *event)
 	    execute = 1;
 	    paintButton(button, texture, pixel, image, True);
 	    break;
-	    
-	 case ButtonRelease:
-	    done = 1;
+
+	 case ButtonPress:
+	    break;
+
+  	 case ButtonRelease:
+	    if (ev.xbutton.button == desc->click_button)
+		done = 1;
 	    break;
 	    
 	 default:
