@@ -128,7 +128,7 @@ wIconCreate(WWindow *wwin)
         icon->file = wstrdup(file);
     }
 
-    MyXGetIconName(dpy, wwin->client_win, &icon->icon_name);
+    wGetIconName(dpy, wwin->client_win, &icon->icon_name);
 
     wIconUpdate(icon);
     
@@ -139,10 +139,9 @@ wIconCreate(WWindow *wwin)
 
 
 WIcon*
-wIconCreateWithName(WScreen *scr, char *instance, char *class)
+wIconCreateWithIconFile(WScreen *scr, char *iconfile)
 {
     WIcon *icon;
-    char *file;
     unsigned long vmask = 0;
     XSetWindowAttributes attribs;
 
@@ -172,13 +171,16 @@ wIconCreateWithName(WScreen *scr, char *instance, char *class)
     icon->core->stacking->window_level = WMNormalWindowLevel;
     icon->core->stacking->child_of = NULL;
 
-    icon->image = wDefaultGetImage(scr, instance, class);
+    if (iconfile) {
+	icon->image = RLoadImage(scr->rcontext, iconfile, 0);
+	if (!icon->image) {
+	    wwarning(_("error loading image file \"%s\""), iconfile, RErrorString);
+	}
 
-    file = wDefaultGetIconFile(scr, instance, class);
-    if (file) {
-        icon->file = wstrdup(file);
+	icon->image = wIconValidateIconSize(scr, icon->image);
+    
+	icon->file = wstrdup(iconfile);
     }
-
     wIconUpdate(icon);
 
     return icon;
@@ -242,7 +244,8 @@ makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed)
 {
     RImage *tile;
     Pixmap pixmap;
-    int x, y, w, h, sx, sy;
+    int x, y, sx, sy;
+    unsigned w, h;
     int theight = scr->icon_title_font->height;
     
     tile = RCloneImage(scr->icon_tile);
@@ -261,10 +264,10 @@ makeIcon(WScreen *scr, RImage *icon, int titled, int shadowed)
 	} else {
 	    h = (icon->height+theight > wPreferences.icon_size
 		 ? wPreferences.icon_size-theight : icon->height);
-	    y = theight + (wPreferences.icon_size - icon->height - theight)/2;
+	    y = theight+((int)wPreferences.icon_size-(int)(icon->height+theight))/2;
 	    sy = (icon->height - h)/2;
 	}
-	sx=sy=0;
+/*	sx=sy=0;*/
 	RCombineArea(tile, icon, sx, sy, w, h, x, y);
     }
 
@@ -317,16 +320,44 @@ wIconChangeImage(WIcon *icon, RImage *new_image)
     if (icon->image)
         RDestroyImage(icon->image);
 
-    icon->image = new_image;
+    icon->image = wIconValidateIconSize(icon->core->screen_ptr, new_image);
+
     wIconUpdate(icon);
 }
 
 
+RImage*
+wIconValidateIconSize(WScreen *scr, RImage *icon)
+{
+    RImage *tmp;
+    int w, h;
+    
+    if (!icon)
+	return NULL;
+    
+    if ((icon->width > wPreferences.icon_size
+	 || icon->height > wPreferences.icon_size) && icon) {
+	if (icon->width > icon->height) {
+	    w = wPreferences.icon_size - 4;
+	    h = w*icon->height/icon->width;
+	} else {
+	    h = wPreferences.icon_size - 4;
+	    w = h*icon->width/icon->height;
+	}
+	tmp = RScaleImage(icon, w, h);
+	RDestroyImage(icon);
+	icon = tmp;
+    }
+
+    return icon;
+}
+
+
 Bool
 wIconChangeImageFile(WIcon *icon, char *file)
 {
     WScreen *scr = icon->core->screen_ptr;
-    RImage *image, *tmp;
+    RImage *image;
     char *path;
     int error = 0;
 
@@ -338,14 +369,6 @@ wIconChangeImageFile(WIcon *icon, char *file)
     path = FindImage(wPreferences.icon_path, file);
 
     if (path && (image = RLoadImage(scr->rcontext, path, 0))) {
-	if (wPreferences.icon_size!=ICON_WIDTH) {
-	    int w = image->width*wPreferences.icon_size/ICON_WIDTH;
-	    int h = image->height*wPreferences.icon_size/ICON_HEIGHT;
-	    
-	    tmp = RScaleImage(image, w, h);
-	    RDestroyImage(image);
-	    image = tmp;
-	}
 	wIconChangeImage(icon, image);
     } else {
 	error = 1;
@@ -357,10 +380,10 @@ wIconChangeImageFile(WIcon *icon, char *file)
     return !error;
 }
 
-
+/*
 void
 wIconChangeIconWindow(WIcon *icon, Window new_window);
-
+*/
 
 static void 
 cycleColor(void *data)
@@ -382,6 +405,17 @@ cycleColor(void *data)
 }
 
 
+void
+wIconSetHighlited(WIcon *icon, Bool flag)
+{
+    if (icon->highlighted == flag) {
+	return;
+    }
+    
+    icon->highlighted = flag;
+    wIconPaint(icon);
+}
+
 void
 wIconSelect(WIcon *icon)
 {
@@ -389,15 +423,23 @@ wIconSelect(WIcon *icon)
 
     if (icon->selected) {
         icon->step = 0;
-        icon->handlerID = WMAddTimerHandler(COLOR_CYCLE_DELAY,cycleColor, icon);
+        icon->handlerID = WMAddTimerHandler(10, cycleColor, icon);
     } else {
         if (icon->handlerID) {
             WMDeleteTimerHandler(icon->handlerID);
             icon->handlerID = NULL;
         }
-	XClearArea(dpy, icon->core->window, 0, 0, icon->core->width, 
+	XClearArea(dpy, icon->core->window, 0, 0, icon->core->width,
 		   icon->core->height, True);
     }
+    
+#if 0
+    if (icon->highlighted) {
+	/*
+	XDrawRectangle(dpy, icon->core->window,
+		      */
+    }
+#endif
 }
 
 
@@ -420,6 +462,7 @@ wIconUpdate(WIcon *icon)
     
     /* use client specified icon window */
     if (icon->icon_win!=None) {
+	XWindowAttributes attr;
 	int resize=0;
 	int width, height, depth;
 	int theight;
@@ -464,6 +507,21 @@ wIconUpdate(WIcon *icon)
 	
 	icon->pixmap = pixmap;
 
+	if (XGetWindowAttributes(dpy, icon->icon_win, &attr)) {
+	    if (attr.all_event_masks & ButtonPressMask) {
+		XGrabButton(dpy, Button1, MOD_MASK, icon->core->window, True,
+			    ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+			    wCursor[WCUR_ARROW]);
+		XGrabButton(dpy, Button1, MOD_MASK|LockMask, icon->core->window,
+			    True, ButtonPressMask, GrabModeSync, GrabModeAsync,
+			    None, wCursor[WCUR_ARROW]);
+#ifdef NUMLOCK_HACK
+		wHackedGrabButton(Button1, MOD_MASK, icon->core->window, True, 
+				  ButtonPressMask, GrabModeSync, GrabModeAsync,
+				  None, wCursor[WCUR_ARROW]);
+#endif
+	    }
+	}
     } else if (wwin && wwin->wm_hints
 	       && (wwin->wm_hints->flags & IconPixmapHint)) {
 	int x, y;
@@ -512,8 +570,8 @@ wIconUpdate(WIcon *icon)
 	XSetClipOrigin(dpy, scr->copy_gc, x, y);
 	
 	if (d != scr->w_depth) {
-	    XSetForeground(dpy, scr->copy_gc, scr->white_pixel);
-	    XSetBackground(dpy, scr->copy_gc, scr->black_pixel);
+	    XSetForeground(dpy, scr->copy_gc, scr->black_pixel);
+	    XSetBackground(dpy, scr->copy_gc, scr->white_pixel);
 	    XCopyPlane(dpy, wwin->wm_hints->icon_pixmap, pixmap, scr->copy_gc,
 		      0, 0, w, h, x, y, 1);
 	} else {
@@ -545,18 +603,21 @@ wIconUpdate(WIcon *icon)
                           wwarning(_("could not find default icon \"%s\""),file);
                           goto make_icons;
                       }
-                      free(path);
 
                       image = RLoadImage(scr->rcontext, path, 0);
                       if (!image) {
-                          wwarning(_("could not load default icon \"%s\""),file);
+                          wwarning(_("could not load default icon \"%s\":%s"),
+				   file, RErrorString);
                       }
+                      free(path);
                   }
 	make_icons:
+		  
+		  image = wIconValidateIconSize(scr, image);
                   scr->def_icon_pixmap = makeIcon(scr, image, False, False);
                   scr->def_ticon_pixmap = makeIcon(scr, image, True, False);
-                  if (image)
-                      RDestroyImage(image);
+		  if (image)
+		      RDestroyImage(image);
               }
 
               if (icon->show_title) {
@@ -658,6 +719,7 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 
     
     if ((desc->click_timestamp>0) &&
+	(event->xbutton.button == desc->click_button) &&
 	(event->xbutton.time-desc->click_timestamp <= wPreferences.dblclick_time)) {
 	desc->click_timestamp = -1;
 
@@ -665,6 +727,7 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 	return;
     }
     desc->click_timestamp = event->xbutton.time;
+    desc->click_button = event->xbutton.button;
     
 #ifdef DEBUG
     puts("Moving miniwindow");
@@ -681,15 +744,15 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
     }
 
     if (XGrabPointer(dpy, icon->core->window, False, ButtonMotionMask
-		     |ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None, 
-		     None, CurrentTime) !=GrabSuccess) {
+		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
 #ifdef DEBUG0
 	wwarning("pointer grab failed for icon move");
 #endif
     }
     while(1) {
-	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonMotionMask
-		   |ExposureMask, &ev);
+	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
+		   |ButtonMotionMask|ExposureMask, &ev);
 	switch (ev.type) {
 	 case Expose:
 	    WMHandleEvent(&ev);
@@ -700,7 +763,7 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					     |ButtonReleaseMask, 
+					    |ButtonReleaseMask|ButtonPressMask, 
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -712,7 +775,13 @@ miniwindowMouseDown(WObjDescriptor *desc, XEvent *event)
 	    XMoveWindow(dpy, icon->core->window, x, y);
 	    break;
 
+	 case ButtonPress:
+	    break;
+
 	 case ButtonRelease:
+	    if (ev.xbutton.button != desc->click_button)
+		break;
+
 	    if (wwin->icon_x!=x || wwin->icon_y!=y)
 	      wwin->flags.icon_moved = 1;
 
