@@ -209,7 +209,7 @@ NextFocusWindow(WScreen *scr)
     d = 0xffffffff;
     while (tmp) {
         if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && tmp->window_flags.focusable &&
+            && !tmp->window_flags.no_focusable &&
             !tmp->window_flags.skip_window_list) {
 	    if (min->client_win > tmp->client_win)
 	      min = tmp;
@@ -242,7 +242,7 @@ PrevFocusWindow(WScreen *scr)
     d = 0xffffffff;
     while (tmp) {
 	if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && tmp->window_flags.focusable &&
+            && !tmp->window_flags.no_focusable &&
             !tmp->window_flags.skip_window_list) {
 	    if (max->client_win < tmp->client_win)
 	      max = tmp;
@@ -260,13 +260,12 @@ PrevFocusWindow(WScreen *scr)
     return closest;
 }
 
-#if 1
+
 /*
  *  XFetchName Wrapper
  *
  */
-
-Status MyXFetchName(dpy, win, winname)
+Bool wFetchName(dpy, win, winname)
 Display *dpy;
 Window win;
 char **winname;
@@ -275,24 +274,32 @@ char **winname;
     char **list;
     int num;
 
-	if (XGetWMName(dpy, win, &text_prop) != 0 && text_prop.value
-	    && text_prop.nitems > 0) {
-		if (text_prop.encoding == XA_STRING)
-			*winname = (char *)text_prop.value;
-		else {
-			text_prop.nitems = strlen((char *)text_prop.value);
-			if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
-				Success && num > 0 && *list) {
-				XFree(text_prop.value);
-				*winname = wstrdup(*list);
-				XFreeStringList(list);
-			} else
-				*winname = (char *)text_prop.value;
+    if (XGetWMName(dpy, win, &text_prop)) {
+	if (text_prop.value && text_prop.nitems > 0) {
+	    if (text_prop.encoding == XA_STRING) {
+		*winname = (char *)text_prop.value;
+	    } else {
+		text_prop.nitems = strlen((char *)text_prop.value);
+		if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
+		    Success && num > 0 && *list) {
+		    XFree(text_prop.value);
+		    *winname = wstrdup(*list);
+		    XFreeStringList(list);
+		} else {
+		    *winname = (char *)text_prop.value;
 		}
-		return 1;
-    }
+	    }
+	} else {
+	    /* the title is set, but it was set to none */
+	    *winname = wstrdup("");
+	}
+	return True;
+    } else {
+	/* the hint is probably not set */
 	*winname = NULL;
-    return 0;  
+	
+	return False;
+    }
 }
 
 /*
@@ -300,7 +307,7 @@ char **winname;
  *
  */
 
-Status MyXGetIconName(dpy, win, iconname)
+Bool wGetIconName(dpy, win, iconname)
 Display *dpy;
 Window win;
 char **iconname;
@@ -323,12 +330,11 @@ char **iconname;
 	    } else
 		*iconname = (char *)text_prop.value;
 	}
-	return 1;
+	return True;
     }
     *iconname = NULL;
-    return 0;
+    return False;
 }
-#endif /* I18N_MB */
 
 
 static void eatExpose()
@@ -421,7 +427,7 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
         XMoveWindow(dpy, win, (int)x, (int)y);
         XFlush(dpy);
         if (apars[wPreferences.icon_slide_speed].delay > 0) {
-            wmsleep(apars[wPreferences.icon_slide_speed].delay*1000L);
+            wusleep(apars[wPreferences.icon_slide_speed].delay*1000L);
         }
     }
     XMoveWindow(dpy, win, to_x, to_y);
@@ -641,7 +647,7 @@ next_token(char *word, char **next)
 
 
 void
-ParseCommand(WScreen *scr, char *command, char ***argv, int *argc)
+ParseCommand(char *command, char ***argv, int *argc)
 {
     LinkedList *list = NULL;
     char *token, *line;
@@ -1066,10 +1072,11 @@ char*
 GetShortcutString(char *text)
 {
     char *buffer = NULL;
-    char *tmp;
+/*    char *tmp;*/
     char *k;
     int modmask = 0;
-    KeySym ksym;
+/*    KeySym ksym;*/
+    int control = 0;
     
     /* get modifiers */
     while ((k = strchr(text, '+'))!=NULL) {
@@ -1080,25 +1087,44 @@ GetShortcutString(char *text)
 	if (mod<0) {
 	    return wstrdup("bug");
 	}
-	
+		
 	modmask |= mod;
-
-	if (mod!=ControlMask) {
+	
+	if (strcasecmp(text, "Meta")==0) {
+	    buffer = catenate(buffer, "M+");
+	} else if (strcasecmp(text, "Alt")==0) {
+	    buffer = catenate(buffer, "A+");
+	} else if (strcasecmp(text, "Shift")==0) {
+	    buffer = catenate(buffer, "Sh+");
+	} else if (strcasecmp(text, "Mod1")==0) {
+	    buffer = catenate(buffer, "M1+"); 
+	} else if (strcasecmp(text, "Mod2")==0) {
+	    buffer = catenate(buffer, "M2+"); 
+	} else if (strcasecmp(text, "Mod3")==0) {
+	    buffer = catenate(buffer, "M3+");
+	} else if (strcasecmp(text, "Mod4")==0) {
+	    buffer = catenate(buffer, "M4+"); 
+	} else if (strcasecmp(text, "Mod5")==0) {
+	    buffer = catenate(buffer, "M5+"); 
+	} else if (strcasecmp(text, "Control")==0) {
+	    control = 1;
+	} else {
 	    buffer = catenate(buffer, text);
 	}
 	text = k+1;
     }
 
-    if (modmask & ControlMask) {
+    if (control) {
 	buffer = catenate(buffer, "^");
-	puts(buffer);
     }
+    buffer = catenate(buffer, text);
+    
     /* get key */
-    ksym = XStringToKeysym(text);
+/*    ksym = XStringToKeysym(text);
     tmp = keysymToString(ksym, modmask);
     puts(tmp);
     buffer = catenate(buffer, tmp);
     free(tmp);
-
+*/
     return buffer;
 }
