@@ -423,13 +423,13 @@ doWindowMove(WWindow *wwin, LinkedList *list, int dx, int dy)
 	    
 	    /* don't let windows become unreachable */
 	    
-	    if (x + (int) tmpw->frame->core->width < 20)
-		x = 20 - tmpw->frame->core->width;
+	    if (x + (int)tmpw->frame->core->width < 20)
+		x = 20 - (int)tmpw->frame->core->width;
 	    else if (x + 20 > scr_width)
 		x = scr_width - 20;
-	    
-	    if (y + (int) tmpw->frame->core->height < 20)
-		y = 0;
+
+	    if (y + (int)tmpw->frame->core->height < 20)
+		y = 20 - (int)tmpw->frame->core->height;
 	    else if (y + 20 > scr_height)
 		y = scr_height - 20;
 	    
@@ -448,10 +448,10 @@ drawTransparentFrame(WWindow *wwin, int x, int y, int width, int height)
     int h = 0;
     int bottom = 0;
     
-    if (wwin->window_flags.titlebar && !wwin->flags.shaded) {
+    if (!wwin->window_flags.no_titlebar && !wwin->flags.shaded) {
 	h = wwin->screen_ptr->title_font->height + TITLEBAR_EXTRA_HEIGHT;
     }
-    if (wwin->window_flags.resizebar && !wwin->flags.shaded) {
+    if (!wwin->window_flags.no_resizebar && !wwin->flags.shaded) {
 	/* Can't use wwin-frame->bottom_width because, in some cases 
 	 (e.g. interactive placement), frame does not point to anything. */
 	bottom = RESIZEBAR_HEIGHT - 1;
@@ -491,13 +491,13 @@ drawFrames(WWindow *wwin, LinkedList *list, int dx, int dy)
 	    
 	    /* don't let windows become unreachable */
 	    
-	    if (x + (int) tmpw->frame->core->width < 20)
-		x = 20 - tmpw->frame->core->width;
+	    if (x + (int)tmpw->frame->core->width < 20)
+		x = 20 - (int)tmpw->frame->core->width;
 	    else if (x + 20 > scr_width)
 		x = scr_width - 20;
 	    
-	    if (y + (int) tmpw->frame->core->height < 20)
-		y = 0;
+	    if (y + (int)tmpw->frame->core->height < 20)
+		y = 20 - (int)tmpw->frame->core->height;
 	    else if (y + 20 > scr_height)
 		y = scr_height - 20;
 	    
@@ -544,8 +544,8 @@ crossWorkspace(WScreen *scr, WWindow *wwin, int opaque_move,
     
     if (!opaque_move) {
 	XGrabPointer(dpy, scr->root_win, True, PointerMotionMask
-		     |ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None,
-		     wCursor[WCUR_MOVE], CurrentTime);
+		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, wCursor[WCUR_MOVE], CurrentTime);
     }
 }
 
@@ -612,8 +612,8 @@ wMouseMoveWindow(WWindow *wwin, XEvent *ev)
 	} else {
 	    Window win;
 	    
-	    WMMaskEvent(dpy, KeyPressMask | ButtonMotionMask | ButtonReleaseMask
-		       | ExposureMask, &event);
+	    WMMaskEvent(dpy, KeyPressMask | ButtonMotionMask
+		| ButtonReleaseMask | ButtonPressMask | ExposureMask, &event);
 	    
 	    if (event.type == MotionNotify) {
 		/* compress MotionNotify events */
@@ -721,7 +721,7 @@ wMouseMoveWindow(WWindow *wwin, XEvent *ev)
 	    } else if (abs(orig_x - event.xmotion.x_root) >= MOVE_THRESHOLD
 		       || abs(orig_y - event.xmotion.y_root) >= MOVE_THRESHOLD) {
 		XChangeActivePointerGrab(dpy, ButtonMotionMask
-					 | ButtonReleaseMask,
+					 | ButtonReleaseMask | ButtonPressMask,
 					 wCursor[WCUR_MOVE], CurrentTime);
 		started = 1;
 		XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, 
@@ -743,8 +743,14 @@ wMouseMoveWindow(WWindow *wwin, XEvent *ev)
 	    
 	    showPosition(wwin, x, y);
 	    break;
+
+	 case ButtonPress:
+	    break;
 	    
 	 case ButtonRelease:
+	    if (event.xbutton.button != ev->xbutton.button)
+		break;
+
 	    if (started) {
 		if (!opaque_move) {
 		    drawFrames(wwin, wSelectedWindows,
@@ -806,7 +812,7 @@ getResizeDirection(WWindow *wwin, int x, int y, int dx, int dy,
     if (!(flags & RESIZEBAR)) {
 	int xdir = (abs(x) < (wwin->client.width/2)) ? LEFT : RIGHT;
 	int ydir = (abs(y) < (wwin->client.height/2)) ? UP : DOWN;
-	if (abs(dx) < MOVE_THRESHOLD || abs(dy) < MOVE_THRESHOLD) {
+	if (abs(dx) < 2 || abs(dy) < 2) {
 	    if (abs(dy) > abs(dx))
 		xdir = 0;
 	    else
@@ -889,13 +895,13 @@ wMouseResizeWindow(WWindow *wwin, XEvent *ev)
     ry2 = fy + fh - 1;
     shiftl = XKeysymToKeycode(dpy, XK_Shift_L);
     shiftr = XKeysymToKeycode(dpy, XK_Shift_R);
-    if (wwin->window_flags.titlebar)
+    if (!wwin->window_flags.no_titlebar)
     	h = wwin->screen_ptr->title_font->height + TITLEBAR_EXTRA_HEIGHT;
     else
 	h = 0;
     while (1) {
 	WMMaskEvent(dpy, KeyPressMask | ButtonMotionMask | ButtonReleaseMask
-		   | ExposureMask, &event);
+		   | ButtonPressMask | ExposureMask, &event);
 	switch (event.type) {
 	 case KeyPress:
 	    showGeometry(wwin, fx, fy, fx + fw, fy + fh, res);
@@ -968,7 +974,7 @@ wMouseResizeWindow(WWindow *wwin, XEvent *ev)
 					 orig_y - event.xmotion.y_root, flags);
 
 		XChangeActivePointerGrab(dpy, ButtonMotionMask
-					 | ButtonReleaseMask,
+					 | ButtonReleaseMask | ButtonPressMask,
 					 wCursor[WCUR_RESIZE], CurrentTime);
 		XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, 
 			      CurrentTime);
@@ -1005,7 +1011,13 @@ wMouseResizeWindow(WWindow *wwin, XEvent *ev)
 	    }
 	    break;
 
+	 case ButtonPress:
+	    break;
+
  	 case ButtonRelease:
+	    if (event.xbutton.button != ev->xbutton.button)
+		break;
+
 	    if (started) {
 		showGeometry(wwin, fx, fy, fx + fw, fy + fh, res);
 		
@@ -1047,7 +1059,7 @@ wUnselectWindows()
             wIconSelect(wwin->icon);
 
         XSetWindowBorder(dpy, wwin->frame->core->window,
-                         wwin->screen_ptr->black_pixel);
+                         wwin->screen_ptr->frame_border_pixel);
 	wwin->flags.selected = 0;
 	list_remove_head(&wSelectedWindows);
     }
@@ -1065,7 +1077,7 @@ wSelectWindow(WWindow *wwin)
     } else {
 	wwin->flags.selected = 0;
 	XSetWindowBorder(dpy, wwin->frame->core->window, 
-			 wwin->screen_ptr->black_pixel);
+			 wwin->screen_ptr->frame_border_pixel);
 	wSelectedWindows = list_remove_elem(wSelectedWindows, wwin);
     }
 }
@@ -1111,8 +1123,9 @@ wSelectWindows(WScreen *scr, XEvent *ev)
     puts("Selecting windows");
 #endif
     if (XGrabPointer(dpy, scr->root_win, False, ButtonMotionMask
-		     | ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None,
-		     wCursor[WCUR_DEFAULT], CurrentTime) != Success) {
+		     | ButtonReleaseMask | ButtonPressMask, GrabModeAsync,
+		     GrabModeAsync, None, wCursor[WCUR_DEFAULT],
+		     CurrentTime) != Success) {
 	return;
     }
     XGrabServer(dpy);
@@ -1121,7 +1134,8 @@ wSelectWindows(WScreen *scr, XEvent *ev)
     
     XDrawRectangle(dpy, root, gc, xp, yp, w, h);
     while (1) {
-	WMMaskEvent(dpy, ButtonReleaseMask | PointerMotionMask, &event);
+	WMMaskEvent(dpy, ButtonReleaseMask | PointerMotionMask
+		    | ButtonPressMask, &event);
 	
 	switch (event.type) {
 	 case MotionNotify:
@@ -1142,8 +1156,14 @@ wSelectWindows(WScreen *scr, XEvent *ev)
 	    }
 	    XDrawRectangle(dpy, root, gc, x, y, w, h);
 	    break;
-	    
+
+	 case ButtonPress:
+	    break;
+
 	 case ButtonRelease:
+	    if (event.xbutton.button != ev->xbutton.button)
+		break;
+
 	    XDrawRectangle(dpy, root, gc, x, y, w, h);
 	    XUngrabServer(dpy);
 	    XUngrabPointer(dpy, CurrentTime);
@@ -1180,11 +1200,11 @@ InteractivePlaceWindow(WWindow *wwin, int *x_ret, int *y_ret)
 	*y_ret = 0;
 	return;
     }
-    if (wwin->window_flags.titlebar) {
+    if (!wwin->window_flags.no_titlebar) {
 	h = scr->title_font->height + TITLEBAR_EXTRA_HEIGHT;
 	height += h;
     }
-    if (wwin->window_flags.resizebar) {
+    if (!wwin->window_flags.no_resizebar) {
 	height += RESIZEBAR_HEIGHT;
     }
     XGrabKeyboard(dpy, root, False, GrabModeAsync, GrabModeAsync, CurrentTime);
