@@ -183,6 +183,7 @@ execCommand(WMenu *menu, WMenuEntry *entry)
 	 * Ok. -Alfredo
          */
         if (fork()==0) {
+	    close(ConnectionNumber(dpy));
 #ifdef HAVE_SETPGID
             setpgid(0, 0);
 #endif
@@ -257,7 +258,7 @@ refreshCommand(WMenu *menu, WMenuEntry *entry)
 static void
 arrangeIconsCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wArrangeIcons(menu->frame->screen_ptr);
+    wArrangeIcons(menu->frame->screen_ptr, True);
 }
 
 static void
@@ -324,14 +325,15 @@ raiseMenus(WMenu *menu)
 
 
 
-void
+Bool
 wRootMenuPerformShortcut(XEvent *event)
 {
     Shortcut *ptr;
     int modifiers;
+    int done = 0;
     
     /* ignore CapsLock */
-    modifiers = event->xkey.state & ~IGNORE_MOD_MASK;
+    modifiers = event->xkey.state & VALID_MOD_MASK;
     
     for (ptr = shortcutList; ptr!=NULL; ptr = ptr->next) {
 	if (ptr->keycode==0) 
@@ -339,8 +341,10 @@ wRootMenuPerformShortcut(XEvent *event)
 	
 	if (ptr->keycode==event->xkey.keycode && (ptr->modifier==modifiers)) {
 	    (*ptr->entry->callback)(ptr->menu, ptr->entry);
+	    done = True;
 	}
     }
+    return done;
 }
 
 
@@ -352,8 +356,12 @@ wRootMenuBindShortcuts(Window window)
     ptr = shortcutList;
     while (ptr) {
 	if (ptr->modifier!=AnyModifier) {
-	    XGrabKey(dpy, ptr->keycode, ptr->modifier|IGNORE_MOD_MASK,
-		window, True, GrabModeAsync, GrabModeAsync);
+	    XGrabKey(dpy, ptr->keycode, ptr->modifier|LockMask,
+		     window, True, GrabModeAsync, GrabModeAsync);
+#ifdef NUMLOCK_HACK
+	    wHackedGrabKey(ptr->keycode, ptr->modifier,
+			   window, True, GrabModeAsync, GrabModeAsync);	    
+#endif
 	}
 	XGrabKey(dpy, ptr->keycode, ptr->modifier, window, True, 
 		 GrabModeAsync, GrabModeAsync);
@@ -372,7 +380,7 @@ rebindKeygrabs(WScreen *scr)
     while (wwin!=NULL) {
 	XUngrabKey(dpy, AnyKey, AnyModifier, wwin->frame->core->window);
 
-	if (wwin->window_flags.bind_keys) {
+	if (!wwin->window_flags.no_bind_keys) {
 	    wWindowSetKeyGrabs(wwin);
 	}
 	wwin = wwin->next;
@@ -411,7 +419,6 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     char *k;
     char buf[128], *b;
 
-
     ptr = wmalloc(sizeof(Shortcut));
     
     strcpy(buf, shortcutDefinition);
@@ -439,16 +446,16 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     ksym = XStringToKeysym(b);
     
     if (ksym==NoSymbol) {
-	wwarning(_("%s:invalid kbd shortcut specification \"%s\""), file,
-		 shortcutDefinition);
+	wwarning(_("%s:invalid kbd shortcut specification \"%s\" for entry %s"),
+		 file, shortcutDefinition, entry->text);
 	free(ptr);
 	return False;
     }
     
     ptr->keycode = XKeysymToKeycode(dpy, ksym);
     if (ptr->keycode==0) {
-	wwarning(_("%s:invalid key in shortcut \"%s\""), file,
-		 shortcutDefinition);
+	wwarning(_("%s:invalid key in shortcut \"%s\" for entry %s"), file,
+		 shortcutDefinition, entry->text);
 	free(ptr);
 	return False;
     }
@@ -647,7 +654,7 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
                 if (first<0)
                     first=i;
             } else {
-                wwarning(_("%s:could not stat menu"), path[i]);
+                wsyserror(_("%s:could not stat menu"), path[i]);
                 /*goto finish;*/
             }
 
@@ -714,7 +721,7 @@ addWorkspaceMenu(WScreen *scr, WMenu *menu, char *title)
     } else {
 	scr->flags.added_workspace_menu = 1;
 	
-	wsmenu = wWorkspaceMenuMake(scr);
+	wsmenu = wWorkspaceMenuMake(scr, True);
 	scr->workspace_menu = wsmenu;
 	entry=wMenuAddCallback(menu, title, NULL, NULL);
 	wMenuEntrySetCascade(menu, entry, wsmenu);
@@ -745,11 +752,6 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
 	    char *path;
 	    
             path = wfindfile(DEF_CONFIG_PATHS, params);
-            /* Using below form will not find files in DEF_CONFIG_PATH, and
-             * return error for them. The above will anyway use wexpandpath()
-             * to do ~ and $HOME expansion, so is better to use it. Dan.
-             */
-	    /*path = wexpandpath(params);*/
 	    if (!path) {
 		path = wstrdup(params);
 	    }
@@ -836,10 +838,11 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
 		     title);
 	} else {
 	    if (addShortcut(file_name, shortcut, menu, entry)) {
-		/*
+	      
 		entry->rtext = GetShortcutString(shortcut);
-		 */
+		 /*
 		entry->rtext = wstrdup(shortcut);
+		  */
 	    }
 	}
     }
@@ -1094,7 +1097,8 @@ readMenuFile(WScreen *scr, char *file_name)
 	    }
 	    break;
 	} else {
-	    wwarning(_("%s:invalid menu file"), file_name);
+	    wwarning(_("%s:invalid menu file. MENU command is missing"), 
+		     file_name);
 	    break;
 	}
     }
@@ -1248,7 +1252,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 
             buffer = wmalloc(strlen(path[i])+strlen(dentry->d_name)+4);
             if (!buffer) {
-                wwarning(_("out of memory while constructing directory menu %s"),
+                wsyserror(_("out of memory while constructing directory menu %s"),
                          path[i]);
                 break;
             }
@@ -1311,7 +1315,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
             length += strlen(command) + 6;
         buffer = wmalloc(length);
         if (!buffer) {
-            wwarning(_("out of memory while constructing directory menu %s"),
+            wsyserror(_("out of memory while constructing directory menu %s"),
                      path[d->index]);
             break;
         }
@@ -1355,7 +1359,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 
         buffer = wmalloc(length);
         if (!buffer) {
-            wwarning(_("out of memory while constructing directory menu %s"),
+            wsyserror(_("out of memory while constructing directory menu %s"),
                      path[f->index]);
             break;
         }
@@ -1397,7 +1401,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 }
 
 
-/************  Menu Configuration From Defaults DB   *************/
+/************  Menu Configuration From WMRootMenu   *************/
 
 static WMenu*
 makeDefaultMenu(WScreen *scr)
@@ -1411,8 +1415,6 @@ makeDefaultMenu(WScreen *scr)
 }
 
 
-
-
 /*
  *---------------------------------------------------------------------- 
  * configureMenu--
@@ -1424,7 +1426,7 @@ static WMenu*
 configureMenu(WScreen *scr, proplist_t definition)
 {
     WMenu *menu = NULL;
-    proplist_t elem, entry;
+    proplist_t elem;
     int i, count;
     proplist_t title, command, params;
     char *tmp, *mtitle;
@@ -1436,8 +1438,8 @@ configureMenu(WScreen *scr, proplist_t definition)
 
 	/* menu definition is a string. Probably a path, so parse the file */
 	
-	tmp = wstrdup(PLGetString(definition));
-	
+	tmp = wexpandpath(PLGetString(definition));
+
 	if (Locale) {
 	    path = wmalloc(strlen(tmp)+32);
 
@@ -1445,13 +1447,20 @@ configureMenu(WScreen *scr, proplist_t definition)
 	    strcat(path, ".");
 	    strcat(path, Locale);
 
+	    /* look for menu.xy */
 	    if (stat(path, &stat_buf)<0) {		
 		int i;
 		i = strlen(Locale);
 		if (i>2) {
 		    path[strlen(path)-(i-2)]=0;
+		    /* look for menu.xy_zw */
 		    if (stat(path, &stat_buf)<0) {
 			free(path);
+			/* If did not find any localized menus, try
+			 * only menu. This can also mean that
+			 * the path in WMRootMenu was already the
+			 * path for the localized menu (eg: menu = "menu.ab")
+			 */
 			path = NULL;
 		    }
 		}
@@ -1462,14 +1471,14 @@ configureMenu(WScreen *scr, proplist_t definition)
 	    path = wfindfile(DEF_CONFIG_PATHS, tmp);
 
 	if (!path) {
-	    wwarning(_("%s:could not find menu file \"%s\""), "defaults DB",
+	    wsyserror(_("%s:could not find menu file \"%s\""), "WMRootMenu",
 		     tmp);
 	    free(tmp);
 	    return NULL;
 	}
 	
 	if (stat(path, &stat_buf)<0) {
-	    wsyserror(_("%s:could not access menu \"%s\""), "defaults DB", path);
+	    wsyserror(_("%s:could not access menu \"%s\""), "WMRootMenu", path);
 	    free(path);
 	    free(tmp);
 	    return NULL;
@@ -1487,20 +1496,19 @@ configureMenu(WScreen *scr, proplist_t definition)
 
 	return menu;
     }
-    
+
     count = PLGetNumberOfElements(definition);
     if (count==0)
 	return NULL;
-    
+
     elem = PLGetArrayElement(definition, 0);
     if (!PLIsString(elem)) {
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
-		 "defaults DB", tmp);
+		 "WMRootMenu", tmp);
 	free(tmp);
 	return NULL;
     }
-    
     mtitle = PLGetString(elem);
     
     menu = wMenuCreate(scr, mtitle, False);
@@ -1509,41 +1517,50 @@ configureMenu(WScreen *scr, proplist_t definition)
     for (i=1; i<count; i++) {
 	elem = PLGetArrayElement(definition, i);
 
-	if (!PLIsArray(elem))
+	if (!PLIsArray(elem) || PLGetNumberOfElements(elem) < 2)
 	    goto error;
-
-	entry = PLGetArrayElement(elem, 0);
 	
-	if (PLIsArray(entry)) {
+	if (PLIsArray(PLGetArrayElement(elem,1))) {
 	    WMenu *submenu;
 	    WMenuEntry *mentry;
 	    
 	    /* submenu */
-	    submenu = configureMenu(scr, entry);
+	    submenu = configureMenu(scr, elem);
 	    if (submenu) {
 		mentry = wMenuAddCallback(menu, submenu->frame->title, NULL, 
 					  NULL);
 		wMenuEntrySetCascade(menu, mentry, submenu);
 	    }
 	} else {
+	    int idx = 0;
+	    char *shortcut;
 	    /* normal entry */
-	    
-	    title = entry;
-	    command = PLGetArrayElement(elem, 1);
-	    params = PLGetArrayElement(elem, 2);
+
+	    title = PLGetArrayElement(elem, idx++);
+	    shortcut = PLGetArrayElement(elem, idx++);
+	    if (strcmp(PLGetString(shortcut), "SHORTCUT")==0) {
+		shortcut = PLGetArrayElement(elem, idx++);
+		command = PLGetArrayElement(elem, idx++);
+	    } else {
+		command = shortcut;
+		shortcut = NULL;
+	    }
+	    params = PLGetArrayElement(elem, idx++);
 
 	    if (!title || !command)
 		goto error;
 	    
-	    addMenuEntry(menu, PLGetString(title), NULL, PLGetString(command),
-			 params ? PLGetString(params) : NULL, "defaults DB");
+	    addMenuEntry(menu, PLGetString(title), 
+			 shortcut ? PLGetString(shortcut) : NULL, 
+			 PLGetString(command),
+			 params ? PLGetString(params) : NULL, "WMRootMenu");
 	}
 	continue;
 
       error:
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
-		 "defaults DB", tmp);
+		 "WMRootMenu", tmp);
 	free(tmp);
     }
     
@@ -1611,8 +1628,19 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
 /*
     definition = PLGetDomain(domain);
  */
-    if (definition)
-	menu = configureMenu(scr, definition);
+    if (definition) {
+	if (PLIsArray(definition)) {
+	    if (!scr->root_menu 
+		|| scr->droot_menu->timestamp > scr->root_menu->timestamp) {
+		menu = configureMenu(scr, definition);
+		if (menu)
+		    menu->timestamp = scr->droot_menu->timestamp;
+	    } else
+		menu = NULL;
+	} else {
+	    menu = configureMenu(scr, definition);
+	}
+    }
 
     if (!menu) {
 	/* menu hasn't changed or could not be read */
