@@ -148,6 +148,11 @@ allocButtonPixmaps(WScreen *scr)
     if (pix)
       pix->shared = 1;
     scr->b_pixmaps[WBUT_ICONIFY] = pix;
+    
+    pix = wPixmapCreateFromXPMData(scr, PRED_KILL_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_KILL] = pix;
 }
 
 
@@ -313,8 +318,7 @@ allocGCs(WScreen *scr)
 
     /* window title text GC */
     gcv.graphics_exposures = False;
-    scr->window_title_gc = XCreateGC(dpy, scr->w_win, GCGraphicsExposures, 
-				     &gcv);
+    scr->window_title_gc = XCreateGC(dpy, scr->w_win,GCGraphicsExposures,&gcv);
 
     /* icon title GC */
     scr->icon_title_gc = XCreateGC(dpy, scr->w_win, GCGraphicsExposures, &gcv);
@@ -326,6 +330,10 @@ allocGCs(WScreen *scr)
     gcv.graphics_exposures = False;
     gcm = GCGraphicsExposures;
     scr->info_text_gc = XCreateGC(dpy, scr->root_win, gcm, &gcv);
+
+    /* frame boder color */
+    wGetColor(scr, FRAME_BORDER_COLOR, &color);
+    scr->frame_border_pixel = color.pixel;
 }
 
 
@@ -338,25 +346,43 @@ createPixmaps(WScreen *scr)
     Pixmap p, m;
 
     /* load pixmaps */
-    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_INDICATOR_XBM_DATA, 
-				   (char*)MENU_INDICATOR_XBM_DATA,
-				   MENU_INDICATOR_XBM_SIZE,
-				   MENU_INDICATOR_XBM_SIZE,
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_RADIO_INDICATOR_XBM_DATA,
+				   (char*)MENU_RADIO_INDICATOR_XBM_DATA,
+				   MENU_RADIO_INDICATOR_XBM_SIZE,
+				   MENU_RADIO_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
       pix->shared = 1;
-    scr->menu_indicator = pix;
+    scr->menu_radio_indicator = pix;
 
     
-    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_INDICATOR2_XBM_DATA,
-				   (char*)MENU_INDICATOR2_XBM_DATA,
-				   MENU_INDICATOR2_XBM_SIZE,
-				   MENU_INDICATOR2_XBM_SIZE,
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_CHECK_INDICATOR_XBM_DATA,
+				   (char*)MENU_CHECK_INDICATOR_XBM_DATA,
+				   MENU_CHECK_INDICATOR_XBM_SIZE,
+				   MENU_CHECK_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
       pix->shared = 1;
-    scr->menu_indicator2 = pix;
+    scr->menu_check_indicator = pix;
 
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_MINI_INDICATOR_XBM_DATA,
+				   (char*)MENU_MINI_INDICATOR_XBM_DATA,
+				   MENU_MINI_INDICATOR_XBM_SIZE,
+				   MENU_MINI_INDICATOR_XBM_SIZE,
+				   scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_mini_indicator = pix;
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_HIDE_INDICATOR_XBM_DATA,
+				   (char*)MENU_HIDE_INDICATOR_XBM_DATA,
+				   MENU_HIDE_INDICATOR_XBM_SIZE,
+				   MENU_HIDE_INDICATOR_XBM_SIZE,
+				   scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_hide_indicator = pix;
+    
 
     image = wDefaultGetImage(scr, "Logo", "WMPanel");
 
@@ -378,6 +404,7 @@ createPixmaps(WScreen *scr)
     if (!wPreferences.flags.nodock || !wPreferences.flags.noclip) {
 	scr->dock_dots = make3Dots(scr);
     }
+    
     /* titlebar button pixmaps */
     allocButtonPixmaps(scr);
 }
@@ -490,6 +517,13 @@ wScreenInit(int screen_number)
     
     /* read defaults that don't change until a restart */
     wReadStaticDefaults(domain ? domain->dictionary : NULL);
+
+    /* check sanity of some values */
+    if (wPreferences.icon_size < 16) {
+	wwarning(_("icon size is configured to %i, but it's too small. Using 16, instead\n"),
+		 wPreferences.icon_size);
+	wPreferences.icon_size = 16;
+    }
     
     scr = wmalloc(sizeof(WScreen));
     memset(scr, 0, sizeof(WScreen));
@@ -532,7 +566,22 @@ wScreenInit(int screen_number)
 	wwarning(_("could not initialize graphics library context: %s"),
 		 RErrorString);
 	wAbort(False);
+    } else {
+	char **formats;
+	int i = 0;
+
+	formats = RSupportedFileFormats();
+	if (formats) {
+	    for (i=0; formats[i]!=NULL; i++) {
+		if (strcmp(formats[i], "TIFF")==0) {
+		    scr->flags.supports_tiff = 1;
+		    break;
+		}
+	    }
+	    RFreeStringList(formats);
+	}
     }
+    
     scr->w_win = scr->rcontext->drawable;
     scr->w_visual = scr->rcontext->visual;
     scr->w_depth = scr->rcontext->depth;
@@ -540,19 +589,18 @@ wScreenInit(int screen_number)
 
     scr->black_pixel = scr->rcontext->black;
     scr->white_pixel = scr->rcontext->white;    
-    
+
     /* create screen descriptor for WINGs */
     scr->wmscreen = WMCreateScreenWithRContext(dpy, screen_number, 
-					       scr->rcontext, "WindowMaker",
-					       NULL, NULL);
+					       scr->rcontext);
 
     color = WMGrayColor(scr->wmscreen);
     scr->light_pixel = WMColorPixel(color);
-    WMReleaseColor(scr->wmscreen, color);
+    WMReleaseColor(color);
 
     color = WMDarkGrayColor(scr->wmscreen);
     scr->dark_pixel = WMColorPixel(color);
-    WMReleaseColor(scr->wmscreen, color);
+    WMReleaseColor(color);
 
     /* create GCs with default values */
     allocGCs(scr);
@@ -615,7 +663,7 @@ wScreenRestoreState(WScreen *scr)
 
     make_keys();
 
-    scr->session_state = PLGetProplistWithPath(wDefaultsPathForDomain("WMState"));
+    scr->session_state = PLGetProplistWithPath(wdefaultspathfordomain("WMState"));
 
     if (!wPreferences.flags.noclip) {
         state = PLGetDictionaryEntry(scr->session_state, dClip);
@@ -699,7 +747,7 @@ wScreenSaveState(WScreen *scr)
 
     wMenuSaveState(scr);
 
-    path = PLMakeString(wDefaultsPathForDomain("WMState"));
+    path = PLMakeString(wdefaultspathfordomain("WMState"));
     PLSetFilename(scr->session_state, path);
     PLSave(scr->session_state, YES);
     PLRelease(path);
@@ -713,12 +761,12 @@ wScreenBringInside(WScreen *scr, int *x, int *y, int width, int height)
 {
     int moved = 0;
     int tol_w, tol_h;
-    
+
     if (width > 20)
 	tol_w = width/2;
     else
 	tol_w = 20;
-    
+
     if (height > 20)
 	tol_h = height/2;
     else
@@ -730,7 +778,7 @@ wScreenBringInside(WScreen *scr, int *x, int *y, int width, int height)
 	*x = scr->scr_width - tol_w - 1, moved = 1;
 
     if (*y < -height + 10)
-	*y = -tol_w, moved = 1;
+	*y = -tol_h, moved = 1;
     else if (*y >= scr->scr_height - 10)
 	*y = scr->scr_height - tol_h - 1, moved = 1;
 
