@@ -35,6 +35,7 @@
 #include <X11/Intrinsic.h>
 #include <X11/cursorfont.h>
 #include <X11/Xproto.h>
+#include <X11/keysym.h>
 
 #include "WindowMaker.h"
 #include "GNUstep.h"
@@ -102,10 +103,23 @@ extern Atom _XA_DND_PROTOCOL;
 extern Atom _XA_DND_SELECTION;
 #endif
 
-
 /* cursors */
 extern Cursor wCursor[WCUR_LAST];
 
+/* special flags */
+extern char WRestartASAP;
+extern char WExitASAP;
+extern char WDelayedActionSet;
+
+
+
+#ifdef NUMLOCK_HACK
+static unsigned int _NumLockMask = 0;
+static unsigned int _ScrollLockMask = 0;
+#endif
+
+
+
 static void manageAllWindows();
 
 extern void NotifyDeadProcess(pid_t pid, unsigned char status);
@@ -164,6 +178,27 @@ handleXIO(Display *dpy)
 }
 
 
+
+/*
+ *----------------------------------------------------------------------
+ * delayedAction-
+ * 	Action to be executed after the signal() handler is exited.
+ *----------------------------------------------------------------------
+ */
+static void
+delayedAction(void *cdata)
+{
+    WDelayedActionSet = 0;
+    /* 
+     * Make the event dispatcher do whatever it needs to do,
+     * including handling zombie processes, restart and exit 
+     * signals.
+     */
+    DispatchEvent(NULL);
+}
+
+
+
 /*
  *----------------------------------------------------------------------
  * handleSig--
@@ -173,6 +208,7 @@ handleXIO(Display *dpy)
 static RETSIGTYPE
 handleSig(int sig)
 {
+    static int already_crashed = 0;
 #ifndef NO_EMERGENCY_AUTORESTART
     char *argv[2];
     
@@ -190,17 +226,25 @@ handleSig(int sig)
 #else
         wwarning(_("got signal %i - restarting\n"), sig);
 #endif
-        if (wScreen) {
-	    wScreen->flags.restart_asap = 1;
-	    return;
-        }
+
+	WRestartASAP = 1;
+	/* setup idle handler, so that this will be handled when
+	 * the select() is returned becaused of the signal, even if
+	 * there are no X events in the queue */
+	if (!WDelayedActionSet) {
+	    WDelayedActionSet = 1;
+	    WMAddIdleHandler(delayedAction, NULL);
+	}
+        return;
     } else if (sig==SIGTERM) {
 	printf(_("%s: Received signal SIGTERM. Exiting..."), ProgName);
 	
-	if (wScreen) {
-	    wScreen->flags.exit_asap = 1;
-            return;
+	WExitASAP = 1;
+	if (!WDelayedActionSet) {
+	    WDelayedActionSet = 1;
+	    WMAddIdleHandler(delayedAction, NULL);
 	}
+	return;
     }
 
 #ifdef SYS_SIGLIST_DECLARED
@@ -209,9 +253,14 @@ handleSig(int sig)
     wfatal(_("got signal %i\n"), sig);
 #endif
     
-    
-#ifndef NO_EMERGENCY_AUTORESTART
     if (sig==SIGSEGV || sig==SIGFPE || sig==SIGBUS) {
+	if (already_crashed) {
+	    wfatal(_("crashed while trying to do some post-crash cleanup. Aborting immediatelly."));
+	    abort();
+	}
+	already_crashed = 1;
+	
+#ifndef NO_EMERGENCY_AUTORESTART
     	/* restart another window manager so that the X session doesn't
 	 * go to space */
 
@@ -228,8 +277,8 @@ handleSig(int sig)
 
     	argv[0] = "twm";
     	execvp("twm", argv);
-    }
 #endif /* !NO_EMERGENCY_AUTORESTART */
+    }
     
     wAbort(sig==SIGSEGV);
 }
@@ -252,6 +301,14 @@ buryChild(int foo)
     pid = waitpid(-1, &status, WNOHANG);
     if (pid>0) {
 	NotifyDeadProcess(pid, WEXITSTATUS(status));
+	/* 
+	 * Make sure that the kid will be buried even if there are
+	 * no events in the X event queue
+	 */
+	if (!WDelayedActionSet) {
+	    WDelayedActionSet = 1;
+	    WMAddIdleHandler(delayedAction, NULL);
+	}
     }
 }
 
@@ -277,6 +334,120 @@ getWorkspaceState(Window root, WWorkspaceState **state)
 }
 
 
+#ifdef NUMLOCK_HACK
+static void
+getOffendingModifiers()
+{
+    int i;
+    XModifierKeymap *modmap;
+    KeyCode nlock, slock;
+    static mask_table[8] = {
+	ShiftMask,LockMask,ControlMask,Mod1Mask,
+	    Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
+    };
+
+    nlock = XKeysymToKeycode(dpy, XK_Num_Lock);
+    slock = XKeysymToKeycode(dpy, XK_Scroll_Lock);
+
+    /*
+     * Find out the masks for the NumLock and ScrollLock modifiers,
+     * so that we can bind the grabs for when they are enabled too.
+     */
+    modmap = XGetModifierMapping(dpy);
+    
+    if (modmap!=NULL && modmap->max_keypermod>0) {
+	for (i=0; i<8*modmap->max_keypermod; i++) {
+	    if (modmap->modifiermap[i]==nlock)
+		_NumLockMask = mask_table[i/modmap->max_keypermod];
+	    else if (modmap->modifiermap[i]==slock)
+		_ScrollLockMask = mask_table[i/modmap->max_keypermod];
+	}
+    }
+
+    if (modmap)
+	XFreeModifiermap(modmap);
+}
+
+
+void 
+wHackedGrabKey(int keycode, unsigned int modifiers,
+	       Window grab_window, Bool owner_events, int pointer_mode,
+	       int keyboard_mode)
+{
+    if (modifiers == AnyModifier)
+	return;
+    
+    /* grab all combinations of the modifier with CapsLock, NumLock and
+     * ScrollLock. How much memory/CPU does such a monstrosity consume
+     * in the server?
+     */
+    XGrabKey(dpy, keycode, modifiers|_NumLockMask, 
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    XGrabKey(dpy, keycode, modifiers|_ScrollLockMask,
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask,
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    XGrabKey(dpy, keycode, modifiers|_NumLockMask|LockMask,
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    XGrabKey(dpy, keycode, modifiers|_ScrollLockMask|LockMask,
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask|LockMask,
+	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    /* phew, I guess that's all, right? */
+}
+
+
+void 
+wHackedGrabButton(unsigned int button, unsigned int modifiers, 
+		  Window grab_window, Bool owner_events, 
+		  unsigned int event_mask, int pointer_mode, 
+		  int keyboard_mode, Window confine_to, Cursor cursor)
+{
+    if (modifiers==AnyModifier)
+	return;
+    
+    /* same as above, but for mouse buttons */
+    XGrabButton(dpy, button, modifiers|_NumLockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+    XGrabButton(dpy, button, modifiers|_ScrollLockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+    XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+    XGrabButton(dpy, button, modifiers|_NumLockMask|LockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+    XGrabButton(dpy, button, modifiers|_ScrollLockMask|LockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+    XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask|LockMask,
+		grab_window, owner_events, event_mask, pointer_mode,
+		keyboard_mode, confine_to, cursor);
+}
+
+
+void
+wHackedUngrabButton(unsigned int button, unsigned int modifiers, 
+		    Window grab_window)
+{
+    XUngrabButton(dpy, button, modifiers|_NumLockMask,
+		  grab_window);
+    XUngrabButton(dpy, button, modifiers|_ScrollLockMask,
+		  grab_window);
+    XUngrabButton(dpy, button, modifiers|_NumLockMask|_ScrollLockMask,
+		  grab_window);
+    XUngrabButton(dpy, button, modifiers|_NumLockMask|LockMask,
+		  grab_window);
+    XUngrabButton(dpy, button, modifiers|_ScrollLockMask|LockMask,
+		  grab_window);
+    XUngrabButton(dpy, button, modifiers|_NumLockMask|_ScrollLockMask|LockMask,
+		  grab_window);
+}
+
+#endif /* NUMLOCK_HACK */
+
        
 /*
  *---------------------------------------------------------- 
@@ -294,6 +465,10 @@ StartUp()
     WWorkspaceState *ws_state;
     struct sigaction sig_action;
     int i;
+
+#ifdef NUMLOCK_HACK
+    getOffendingModifiers();
+#endif
     
     memset(&wKeyBindings, 0, sizeof(wKeyBindings));
 
@@ -345,28 +520,39 @@ StartUp()
     wCursor[WCUR_TEXT]     = XCreateFontCursor(dpy, XC_xterm); /* odd name???*/
     
     /* emergency exit... */
-    signal(SIGINT, handleSig);
-    signal(SIGTERM, handleSig);
-    signal(SIGQUIT, handleSig);
-    signal(SIGSEGV, handleSig);
-    signal(SIGBUS, handleSig);
-    signal(SIGFPE, handleSig);
-    signal(SIGHUP, handleSig);
-
-    /* handle X shutdowns a such */
-    XSetIOErrorHandler(handleXIO);
-    
-    /* handle dead children */
-    sig_action.sa_handler = buryChild;
+    sig_action.sa_handler = handleSig;
     sigemptyset(&sig_action.sa_mask);
-    sig_action.sa_flags = SA_NOCLDSTOP|SA_RESTART;
-    sigaction(SIGCHLD, &sig_action, NULL);
+
+    /* Here we don't care about SA_RESTART since these signals will close
+     * wmaker anyway.
+     * -Dan */
+    sig_action.sa_flags = 0;
+    sigaction(SIGINT, &sig_action, NULL);
+    sigaction(SIGTERM, &sig_action, NULL);
+    sigaction(SIGQUIT, &sig_action, NULL);
+    sigaction(SIGSEGV, &sig_action, NULL);
+    sigaction(SIGBUS, &sig_action, NULL);
+    sigaction(SIGFPE, &sig_action, NULL);
+
+    /* Here we set SA_RESTART for safety, because SIGHUP may not be handled
+     * immediately.
+     * -Dan */
+    sig_action.sa_flags = SA_RESTART;
+    sigaction(SIGHUP, &sig_action, NULL);
 
     /* ignore dead pipe */
     sig_action.sa_handler = ignoreSig;
-    sigemptyset(&sig_action.sa_mask);
+    sig_action.sa_flags = SA_RESTART;
     sigaction(SIGPIPE, &sig_action, NULL);
 
+    /* handle dead children */
+    sig_action.sa_handler = buryChild;
+    sig_action.sa_flags = SA_NOCLDSTOP|SA_RESTART;
+    sigaction(SIGCHLD, &sig_action, NULL);
+
+    /* handle X shutdowns a such */
+    XSetIOErrorHandler(handleXIO);
+
     /* set hook for out event dispatcher in WINGs event dispatcher */
     WMHookEventHandler(DispatchEvent);
 
@@ -557,17 +743,19 @@ manageAllWindows(WScreen *scr)
 	    }
 	}
     }
-    scr->flags.startup = 0;
     XUngrabServer(dpy);
     XFree(children);
+    scr->flags.startup = 0;
+    scr->flags.startup2 = 1;
     while (XPending(dpy)) {
 	XEvent ev;
 	WMNextEvent(dpy, &ev);
 	WMHandleEvent(&ev);
     }
-    wWorkspaceChange(scr, 0);
+    wWorkspaceForceChange(scr, 0);
     if (!wPreferences.flags.noclip)
         wDockShowIcons(scr->workspaces[scr->current_workspace]->clip);
+    scr->flags.startup2 = 0;
 }
 
 
