@@ -114,7 +114,7 @@ DoKaboom(WScreen *scr, Window win, int x, int y)
 	    }
 	}
 #if (MINIATURIZE_ANIMATION_DELAY > 0)	
-	wmsleep(MINIATURIZE_ANIMATION_DELAY*2);
+	wusleep(MINIATURIZE_ANIMATION_DELAY*2);
 #endif
 	for (i=0; i<j; i++) {
 	    if (ax[i]>=0) {
