@@ -82,8 +82,6 @@ extern Time LastTimestamp;
 /* local functions */
 static FocusMode getFocusMode(WWindow *wwin);
 
-static void setupGrabs(WWindow *wwin);
-static void removeGrabs(WWindow *wwin);
 static int getSavedState(Window window, WSavedState **state);
 
 static void setupGNUstepHints(WWindowAttributes *attribs, 
@@ -150,6 +148,7 @@ wWindowCreate()
     wwin->client_descriptor.self = wwin;
     wwin->client_descriptor.parent_type = WCLASS_WINDOW;
     wwin->client_descriptor.click_timestamp = 0;
+    wwin->client_descriptor.click_button = -1;
     return wwin;
 }
 
@@ -186,7 +185,7 @@ wWindowDestroy(WWindow *wwin)
 	RemoveFromStackList(wwin->icon->core);
         wIconDestroy(wwin->icon);
         if (wPreferences.auto_arrange_icons)
-            wArrangeIcons(wwin->screen_ptr);
+            wArrangeIcons(wwin->screen_ptr, True);
     }
     wrelease(wwin);
 }
@@ -198,30 +197,30 @@ static void
 setupGNUstepHints(WWindowAttributes *attribs, GNUstepWMAttributes *gs_hints)
 {
     if (gs_hints->flags & GSWindowStyleAttr) {
-	attribs->titlebar = 
-	    ((gs_hints->window_style & WMTitledWindowMask)?1:0);
+	attribs->no_titlebar =
+	    ((gs_hints->window_style & WMTitledWindowMask)?0:1);
 	
-	attribs->close_button = attribs->closable = 
-	    ((gs_hints->window_style & WMClosableWindowMask)?1:0);
+	attribs->no_close_button = attribs->no_closable = 
+	    ((gs_hints->window_style & WMClosableWindowMask)?0:1);
 	
-	attribs->miniaturize_button = attribs->miniaturizable =
-	    ((gs_hints->window_style & WMMiniaturizableWindowMask)?1:0);
+	attribs->no_miniaturize_button = attribs->no_miniaturizable =
+	    ((gs_hints->window_style & WMMiniaturizableWindowMask)?0:1);
 	
-	attribs->resizebar = attribs->resizable = 
-	    ((gs_hints->window_style & WMResizableWindowMask)?1:0);
+	attribs->no_resizebar = attribs->no_resizable =
+	    ((gs_hints->window_style & WMResizableWindowMask)?0:1);
     } else {
 	/* setup the defaults */
-	attribs->titlebar = 1;
-	attribs->closable = 1;
-	attribs->miniaturizable = 1;
-	attribs->resizable = 1;
-	attribs->close_button = 1;
-	attribs->miniaturize_button = 1;
-	attribs->resizebar = 1;
+	attribs->no_titlebar = 0;
+	attribs->no_closable = 0;
+	attribs->no_miniaturizable = 0;
+	attribs->no_resizable = 0;
+	attribs->no_close_button = 0;
+	attribs->no_miniaturize_button = 0;
+	attribs->no_resizebar = 0;
     }
     
     if (gs_hints->extra_flags & GSNoApplicationIconFlag) {
-	attribs->application = 0;
+	attribs->no_appicon = 1;
     }
 }
 
@@ -240,19 +239,22 @@ setupMWMHints(WWindowAttributes *attribs, MWMHints *mwm_hints)
 # ifdef DEBUG
 	fprintf(stderr,"0x%08x: has decor hints [ ", (int)window);
 # endif
-	attribs->titlebar = 0;
-	attribs->close_button = 0;
-	attribs->miniaturize_button = 0;
-	attribs->resizebar = 0;
+	attribs->no_titlebar = 1;
+	attribs->no_close_button = 1;
+	attribs->no_miniaturize_button = 1;
+	attribs->no_resizebar = 1;
 	
 	if (mwm_hints->decorations & MWM_DECOR_ALL) {
 # ifdef DEBUG
 	    fprintf(stderr,"ALL ");
 # endif
-	    attribs->titlebar = 1;
-	    attribs->close_button = 1;
-	    attribs->miniaturize_button = 1;
-	    attribs->resizebar = 1;
+	    attribs->no_titlebar = 0;
+	    attribs->no_close_button = 0;
+	    attribs->no_closable = 0;
+	    attribs->no_miniaturize_button = 0;
+	    attribs->no_miniaturizable = 0;
+	    attribs->no_resizebar = 0;
+	    attribs->no_resizable = 0;
 	}
 /*
 	if (mwm_hints->decorations & MWM_DECOR_BORDER) {
@@ -266,15 +268,16 @@ setupMWMHints(WWindowAttributes *attribs, MWMHints *mwm_hints)
 # ifdef DEBUG
 	    fprintf(stderr,"RESIZEH ");
 # endif
-	    attribs->resizebar = 1;
+	    attribs->no_resizebar = 0;
 	}
 
 	if (mwm_hints->decorations & MWM_DECOR_TITLE) {
 # ifdef DEBUG
 	    fprintf(stderr,"TITLE+close ");
 # endif
-	    attribs->titlebar = 1;
-	    attribs->close_button = 1;
+	    attribs->no_titlebar = 0;
+	    attribs->no_close_button = 0;
+	    attribs->no_closable = 0;
 	}
 /*
 	if (mwm_hints->decorations & MWM_DECOR_MENU) {
@@ -288,7 +291,8 @@ setupMWMHints(WWindowAttributes *attribs, MWMHints *mwm_hints)
 # ifdef DEBUG
 	    fprintf(stderr,"MINIMIZE ");
 # endif
-	    attribs->miniaturize_button = 1;
+	    attribs->no_miniaturize_button = 0;
+	    attribs->no_miniaturizable = 0;
 	}
 /*
 	if (mwm_hints->decorations & MWM_DECOR_MAXIMIZE) {
@@ -307,23 +311,23 @@ setupMWMHints(WWindowAttributes *attribs, MWMHints *mwm_hints)
 # ifdef DEBUG
 	fprintf(stderr,"0x%08x: has function hints [ ",(int)window);
 # endif
-	attribs->closable = 0;
-	attribs->miniaturizable = 0;
-	attribs->resizable = 0;
+	attribs->no_closable = 1;
+	attribs->no_miniaturizable = 1;
+	attribs->no_resizable = 1;
 	
 	if (mwm_hints->functions & MWM_FUNC_ALL) {
 # ifdef DEBUG 
 	    fprintf(stderr,"ALL ");
 # endif
-	    attribs->closable = 1;
-	    attribs->miniaturizable = 1;
-	    attribs->resizable = 1;
+	    attribs->no_closable = 0;
+	    attribs->no_miniaturizable = 0;
+	    attribs->no_resizable = 0;
 	}
 	if (mwm_hints->functions & MWM_FUNC_RESIZE) {
 # ifdef DEBUG 
 	    fprintf(stderr,"RESIZE ");
 # endif
-	    attribs->resizable = 1;
+	    attribs->no_resizable = 0;
 	}
 	/*
 	if (mwm_hints->functions & MWM_FUNC_MOVE) {
@@ -336,20 +340,20 @@ setupMWMHints(WWindowAttributes *attribs, MWMHints *mwm_hints)
 # ifdef DEBUG 
 	    fprintf(stderr,"MINIMIZE ");
 # endif
-	    attribs->miniaturizable = 1;
+	    attribs->no_miniaturizable = 0;
 	}
 	if (mwm_hints->functions & MWM_FUNC_MAXIMIZE) {
 # ifdef DEBUG 
 	    fprintf(stderr,"MAXIMIZE ");
 	    /* a window must be resizable to be maximizable */
-	    attribs->resizable = 1;
+	    attribs->no_resizable = 0;
 # endif
 	}
 	if (mwm_hints->functions & MWM_FUNC_CLOSE) {
 # ifdef DEBUG 
 	    fprintf(stderr,"CLOSE ");
 # endif
-	    attribs->closable = 1;
+	    attribs->no_closable = 0;
 	}
 # ifdef DEBUG 
 	fprintf(stderr,"]\n");
@@ -456,7 +460,7 @@ wManageWindow(WScreen *scr, Window window)
     XSetWindowBorderWidth(dpy, window, 0);
 
     /* fill in property/hint data */
-    if (!MyXFetchName(dpy, window, &title)) {
+    if (!wFetchName(dpy, window, &title)) {
 	title = NULL;
     }
     /* get hints from GNUstep app */
@@ -511,12 +515,13 @@ wManageWindow(WScreen *scr, Window window)
 	} else {
 	    WWindow *owner;
 	    owner = wWindowFor(wwin->transient_for);
-	    if (owner /*&& owner->main_window*/) {
+	    if (owner && owner->main_window!=None) {
 		wwin->main_window = owner->main_window;
-            } else {
+            } /*else {
 		wwin->main_window = None;
-            }
-	    /* don't let transients be miniaturized if their owners
+            }*/
+
+	    /* don't let transients start miniaturized if their owners
 	     * are not */
 	    if (owner && !owner->flags.miniaturized && iconic) {
 		iconic = 0;
@@ -544,8 +549,6 @@ wManageWindow(WScreen *scr, Window window)
      * 
      *--------------------------------------------------
      */
-    memset(&wwin->window_flags, 0xff, sizeof(WWindowAttributes));
-    
     /* sets global default stuff */
     wDefaultFillAttributes(scr, wwin->wm_instance, wwin->wm_class, 
 			   &wwin->window_flags, True);
@@ -556,15 +559,17 @@ wManageWindow(WScreen *scr, Window window)
      * - use GNUstep/external window attributes
      * - set hints specified for the app in the resource DB
      */
+    wwin->window_flags.broken_close = 0;
+    
     if (wwin->protocols.DELETE_WINDOW)
-      wwin->window_flags.broken_close = 0;
+	wwin->window_flags.kill_close = 0;
     else
-      wwin->window_flags.broken_close = 1;
+	wwin->window_flags.kill_close = 1;
 
     /* transients can't be iconified or maximized */
     if (wwin->transient_for) {
-	wwin->window_flags.miniaturizable = 0;
-	wwin->window_flags.miniaturize_button = 0;
+	wwin->window_flags.no_miniaturizable = 1;
+	wwin->window_flags.no_miniaturize_button = 1;
 #ifdef DEBUG
 	printf("%x is transient for %x\n", (unsigned)window, 
 	       (unsigned)wwin->transient_for);
@@ -575,8 +580,8 @@ wManageWindow(WScreen *scr, Window window)
     if (wwin->normal_hints->flags & (PMinSize|PMaxSize)
 	&& (wwin->normal_hints->min_width==wwin->normal_hints->max_width)
 	&& (wwin->normal_hints->min_height==wwin->normal_hints->max_height)) {
-	wwin->window_flags.resizable = 0;
-	wwin->window_flags.resizebar = 0;
+	wwin->window_flags.no_resizable = 1;
+	wwin->window_flags.no_resizebar = 1;
     }
     
     /* set GNUstep window attributes */
@@ -588,17 +593,12 @@ wManageWindow(WScreen *scr, Window window)
 	    /* setup defaults */
 	    window_level = WMNormalWindowLevel;
 	}
-#ifdef MWM_HINTS
-    } else if (motif_hints) {
-#ifdef DEBUG
-    	fprintf(stderr,"0x%08x: looking into motif hints\n",(int)window);
-#endif
-	setupMWMHints(&wwin->window_flags, motif_hints);
-	
-	/* setup other defaults */
-	window_level = WMNormalWindowLevel;
-#endif /* MWM_HINTS */
     } else {
+#ifdef MWM_HINTS
+	if (motif_hints) {
+	    setupMWMHints(&wwin->window_flags, motif_hints);
+	}
+#endif /* MWM_HINTS */
 	if (wwin->window_flags.floating)
 	    window_level = WMFloatingWindowLevel;
 	else
@@ -617,7 +617,7 @@ wManageWindow(WScreen *scr, Window window)
     wDefaultFillAttributes(scr, wwin->wm_instance, wwin->wm_class, 
 			   &wwin->window_flags, False);
 
-    wwin->window_flags.shadeable = wwin->window_flags.titlebar;
+    wwin->window_flags.no_shadeable = wwin->window_flags.no_titlebar;
 
     /*
      *------------------------------------------------------------ 
@@ -627,7 +627,7 @@ wManageWindow(WScreen *scr, Window window)
      *------------------------------------------------------------
      */
     if (wwin->window_flags.start_miniaturized
-	&& wwin->window_flags.miniaturizable) {
+	&& !wwin->window_flags.no_miniaturizable) {
 	wwin->flags.miniaturized = 1;
         iconic = 1;
     }
@@ -643,10 +643,10 @@ wManageWindow(WScreen *scr, Window window)
                        wwin->wm_hints->initial_state==WithdrawnState)) {
         if (win_state->state->hidden>0)
             wwin->flags.hidden = win_state->state->hidden;
-        if (win_state->state->shaded>0 && wwin->window_flags.shadeable)
+        if (win_state->state->shaded>0 && !wwin->window_flags.no_shadeable)
             wwin->flags.shaded = win_state->state->shaded;
         if (win_state->state->miniaturized>0 &&
-            wwin->window_flags.miniaturizable) {
+            !wwin->window_flags.no_miniaturizable) {
             wwin->flags.miniaturized = win_state->state->miniaturized;
             iconic = 1;
         }
@@ -704,15 +704,6 @@ wManageWindow(WScreen *scr, Window window)
     }
 
     wWindowConstrainSize(wwin, &width, &height);
-/*
-    if (wPreferences.constrain_window_size) {
-	if (width > wwin->screen_ptr->scr_width)
-	    width = wwin->screen_ptr->scr_width;
-	
-	if (height > wwin->screen_ptr->scr_height)
-	    height = wwin->screen_ptr->scr_height;
-    }
-*/
 
     resized_client = 0;
     if (wwin->client.width != width) {
@@ -753,9 +744,9 @@ wManageWindow(WScreen *scr, Window window)
      */
     
     foo = WFF_LEFT_BUTTON | WFF_RIGHT_BUTTON;
-    if (wwin->window_flags.titlebar)
+    if (!wwin->window_flags.no_titlebar)
 	foo |= WFF_TITLEBAR;
-    if (wwin->window_flags.resizebar)
+    if (!wwin->window_flags.no_resizebar)
 	foo |= WFF_RESIZEBAR;
 
     wwin->frame = wFrameWindowCreate(scr, window_level, 
@@ -776,9 +767,9 @@ wManageWindow(WScreen *scr, Window window)
     
     /* hide unused buttons */
     foo = 0;
-    if (!wwin->window_flags.close_button)
+    if (wwin->window_flags.no_close_button)
 	foo |= WFF_RIGHT_BUTTON;
-    if (!wwin->window_flags.miniaturize_button)
+    if (wwin->window_flags.no_miniaturize_button)
 	foo |= WFF_LEFT_BUTTON;
     if (foo!=0)
 	wFrameWindowHideButton(wwin->frame, foo);
@@ -831,6 +822,15 @@ wManageWindow(WScreen *scr, Window window)
 
     if (wwin->main_window!=None) {
 	WApplication *app;
+	WWindow *leader;
+
+	/* Leader windows do not necessary set themselves as leaders.
+	 * If this is the case, point the leader of this window to
+	 * itself */
+	leader = wWindowFor(wwin->main_window);
+	if (leader && leader->main_window==None) {
+	    leader->main_window = leader->client_win;       
+	}
 
 	app = wApplicationCreate(scr, wwin->main_window);
         if (app) {
@@ -927,26 +927,17 @@ wManageWindow(WScreen *scr, Window window)
     
     wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
 
-    if (wwin->window_flags.bind_mouse) {
-	if (wPreferences.auto_focus) {
-	    XGrabButton(dpy, AnyButton, MOD_MASK,
-			wwin->client_win, True,
-			ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-			wCursor[WCUR_ARROW]);
-	} else {
-	    setupGrabs(wwin);
-	}
-    }
 			    
     if (!iconic && workspace == scr->current_workspace) {
-	if (wPreferences.focus_mode==WKF_CLICK 
-	    && wPreferences.auto_focus)
-	  wSetFocusTo(scr, wwin);
+	if (/*wPreferences.focus_mode==WKF_CLICK &&*/ 
+	    wPreferences.auto_focus)
+	    wSetFocusTo(scr, wwin);
     } else {
 	wwin->flags.ignore_next_unmap = 1;
     }
+    wWindowResetMouseGrabs(wwin);
     
-    if (wwin->window_flags.bind_keys) {
+    if (!wwin->window_flags.no_bind_keys) {
 	wWindowSetKeyGrabs(wwin);
     }
 
@@ -988,12 +979,11 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
 
     wwin = wWindowCreate();
 
-    wwin->window_flags.titlebar = 1;
     wwin->flags.internal_window = 1;
-    wwin->window_flags.focusable = 1;
-    wwin->window_flags.bind_mouse = 1;
-    wwin->window_flags.bind_keys = 1;
     wwin->window_flags.omnipresent = 1;
+    wwin->window_flags.no_shadeable = 1;
+    
+    wwin->focus_mode = WFM_PASSIVE;
     
     wwin->client_win = window;
     wwin->screen_ptr = scr;
@@ -1009,11 +999,8 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
     wwin->frame_y = wwin->client.y;
 
     
-        
     foo = WFF_RIGHT_BUTTON;
     foo |= WFF_TITLEBAR;
-    if (wwin->window_flags.resizebar)
-	foo |= WFF_RESIZEBAR;
 
     wwin->frame = wFrameWindowCreate(scr, WMFloatingWindowLevel,
 				     wwin->frame_x, wwin->frame_y,
@@ -1031,14 +1018,9 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
 
     /* setup button images */
     wWindowUpdateButtonImages(wwin);
-    
-    /* hide unused buttons */
-    foo = 0;
-    if (!wwin->window_flags.close_button) 
-	foo |= WFF_RIGHT_BUTTON;
-    
-    wFrameWindowHideButton(wwin->frame, foo);
-    
+
+    /* hide buttons */
+    wFrameWindowHideButton(wwin->frame, WFF_RIGHT_BUTTON);
     
     wwin->frame->child = wwin;
 
@@ -1105,14 +1087,8 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
     if (wPreferences.focus_mode==WKF_CLICK 
 	&& wPreferences.auto_focus)
 	  wSetFocusTo(scr, wwin);
-    
-    if (wPreferences.auto_focus) {
-    	XGrabButton(dpy, AnyButton, MOD_MASK, wwin->client_win, 
-		    True, ButtonPressMask, GrabModeSync, GrabModeAsync, None,
-		    wCursor[WCUR_ARROW]);
-    } else {
-	setupGrabs(wwin);
-    }
+
+    wWindowResetMouseGrabs(wwin);
 
     wWindowSetKeyGrabs(wwin);
 
@@ -1140,6 +1116,9 @@ wUnmanageWindow(WWindow *wwin, int restore)
 {
     WCoreWindow *frame = wwin->frame->core;
     WWindow *owner;
+    WWindow *newFocusedWindow;
+    int wasNotFocused;
+    WScreen *scr = wwin->screen_ptr;
 
     /* First close attribute editor window if open */
     if (wwin->flags.inspector_open) {
@@ -1158,15 +1137,15 @@ wUnmanageWindow(WWindow *wwin, int restore)
     if (wPreferences.raise_delay) 
        WMDeleteTimerWithClientData(wwin->frame->core);
 
-    XSync(dpy, 0);
+    XFlush(dpy);
 
-    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_REMOVE);
+    UpdateSwitchMenu(scr, wwin, ACTION_REMOVE);
 
     /* reparent the window back to the root */
     if (restore)
 	wClientRestore(wwin);
 
-    if (wwin->transient_for!=wwin->screen_ptr->root_win) {
+    if (wwin->transient_for!=scr->root_win) {
 	owner = wWindowFor(wwin->transient_for);
 	if (owner) {
 	    owner->flags.semi_focused = 0;
@@ -1174,11 +1153,13 @@ wUnmanageWindow(WWindow *wwin, int restore)
 	}
     }
 
+    wasNotFocused = !wwin->flags.focused;
+    
     /* remove from window focus list */
     if (!wwin->prev && !wwin->next) {
 	/* was the only window */
-	wwin->screen_ptr->focused_window = NULL;
-	wSetFocusTo(wwin->screen_ptr, NULL);
+	scr->focused_window = NULL;
+	newFocusedWindow = NULL;
     } else {
 	WWindow *tmp;
 
@@ -1187,8 +1168,8 @@ wUnmanageWindow(WWindow *wwin, int restore)
 	if (wwin->next)
 	  wwin->next->prev = wwin->prev;
 	else {
-	    wwin->screen_ptr->focused_window = wwin->prev;
-	    wwin->screen_ptr->focused_window->next = NULL;
+	    scr->focused_window = wwin->prev;
+	    scr->focused_window->next = NULL;
 	}
 	
 	/* if in click to focus mode and the window
@@ -1197,23 +1178,24 @@ wUnmanageWindow(WWindow *wwin, int restore)
 	tmp = NULL;
 	if (wPreferences.focus_mode==WKF_CLICK) {
 	    tmp = wWindowFor(wwin->transient_for);
-	    if (tmp && (!tmp->flags.mapped || !tmp->window_flags.focusable)) {
+	    if (tmp && (!tmp->flags.mapped || tmp->window_flags.no_focusable)) {
 		tmp = NULL;
 	    }
 	}
 	/* otherwise, focus the next one in the focus list */
 	if (!tmp) {
-	    tmp = wwin->screen_ptr->focused_window;
+	    tmp = scr->focused_window;
 	    while (tmp) {
-		if (tmp->window_flags.focusable 
+		if (!tmp->window_flags.no_focusable 
 		    && (tmp->flags.mapped || tmp->flags.shaded))
 		    break;
 		tmp = tmp->prev;
 	    }
 	}
 	if (wPreferences.focus_mode==WKF_CLICK) {
-	    wSetFocusTo(wwin->screen_ptr, tmp);
-	} else if (wPreferences.focus_mode==WKF_SLOPPY) {
+	    newFocusedWindow = tmp;
+	} else if (wPreferences.focus_mode==WKF_SLOPPY
+		   || wPreferences.focus_mode==WKF_POINTER) {
             unsigned int mask;
             int foo;
             Window bar, win;
@@ -1223,20 +1205,24 @@ wUnmanageWindow(WWindow *wwin, int restore)
              * This way keybindings will not freeze.
              */
             tmp = NULL;
-            if (XQueryPointer(dpy, wwin->screen_ptr->root_win, &bar, &win,
+            if (XQueryPointer(dpy, scr->root_win, &bar, &win,
                               &foo, &foo, &foo, &foo, &mask))
                 tmp = wWindowFor(win);
             if (tmp == wwin)
                 tmp = NULL;
-            wSetFocusTo(wwin->screen_ptr, tmp);
-        }
+	    newFocusedWindow = tmp;
+        } else {
+	    newFocusedWindow = NULL;
+	}
     }
 #ifdef DEBUG
     printf("destroying window %x frame %x\n", (unsigned)wwin->client_win,
 	   (unsigned)frame->window);
-#endif 
+#endif
+    if (!wasNotFocused)
+	wSetFocusTo(scr, newFocusedWindow);
     wWindowDestroy(wwin);
-    XSync(dpy, 0);
+    XFlush(dpy);
 }
 
      
@@ -1246,18 +1232,14 @@ wWindowFocus(WWindow *wwin)
     wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
 
     wwin->flags.focused=1;
-    if (wPreferences.focus_mode==WKF_CLICK)
-      removeGrabs(wwin);
-    /* keep meta-click grabs */
-    if (wwin->window_flags.bind_mouse) {
-	XGrabButton(dpy, AnyButton, MOD_MASK,
-		    wwin->client_win, True, ButtonPressMask, GrabModeSync, 
-		    GrabModeAsync, None, wCursor[WCUR_ARROW]);
-    }
 
-    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_FOCUS);
+    wWindowResetMouseGrabs(wwin);
+
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
 }
 
+
+
 void
 wWindowUnfocus(WWindow *wwin)
 {
@@ -1265,7 +1247,7 @@ wWindowUnfocus(WWindow *wwin)
 
     wFrameWindowChangeState(wwin->frame, wwin->flags.semi_focused 
 			    ? WS_PFOCUSED : WS_UNFOCUSED);
-    
+
     if (wwin->transient_for!=None 
 	&& wwin->transient_for!=wwin->screen_ptr->root_win) {
 	WWindow *owner;
@@ -1279,12 +1261,9 @@ wWindowUnfocus(WWindow *wwin)
 	}
     }
     wwin->flags.focused=0;
-    if (wwin->window_flags.bind_mouse) {
-	XUngrabButton(dpy, Button1, MOD_MASK, wwin->client_win);
-    }
-    if (wPreferences.focus_mode==WKF_CLICK)
-	setupGrabs(wwin);
-    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_FOCUS);
+    wWindowResetMouseGrabs(wwin);
+
+    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
 }
 
 
@@ -1441,10 +1420,11 @@ wWindowSynthConfigureNotify(WWindow *wwin)
     sevent.xconfigure.height = wwin->client.height;
 
     sevent.xconfigure.border_width = wwin->old_border_width;
-    if (wwin->window_flags.titlebar)
-        sevent.xconfigure.above = wwin->frame->titlebar->window;
-    else
+    if (wwin->window_flags.no_titlebar)
 	sevent.xconfigure.above = None;
+    else
+        sevent.xconfigure.above = wwin->frame->titlebar->window;
+
     sevent.xconfigure.override_redirect = False;
     XSendEvent(dpy, wwin->client_win, False, StructureNotifyMask, &sevent);
 }
@@ -1477,13 +1457,6 @@ int req_width, req_height;	       /* new size of the client */
 {
     int synth_notify = False;
     int resize;
-
-    /* sanity test. Remove this */
-    if (!wwin->window_flags.closable && wwin->window_flags.close_button)
-	puts("ERROR IN CLOSABLE");
-    if (!wwin->window_flags.miniaturizable && wwin->window_flags.miniaturize_button)
-	puts("ERROR IN MINIATU");
-    /* end sanity test */
     
     resize = (req_width!=wwin->client.width
 	      || req_height!=wwin->client.height);
@@ -1585,15 +1558,14 @@ wWindowUpdateButtonImages(WWindow *wwin)
     Pixmap pixmap, mask;
     WFrameWindow *fwin = wwin->frame;
     
-    if (!wwin->window_flags.titlebar)
+    if (wwin->window_flags.no_titlebar)
       return;
 
     /* miniaturize button */
     
-    if (wwin->window_flags.miniaturize_button) {
+    if (!wwin->window_flags.no_miniaturize_button) {
 	if (wwin->wm_gnustep_attr
 	    && wwin->wm_gnustep_attr->flags & GSMiniaturizePixmapAttr) {
-	    
 	    pixmap = wwin->wm_gnustep_attr->miniaturize_pixmap;
 	      
 	    if (wwin->wm_gnustep_attr->flags&GSMiniaturizeMaskAttr) {
@@ -1624,10 +1596,9 @@ wWindowUpdateButtonImages(WWindow *wwin)
     
     /* close button */
 
-    if (wwin->window_flags.close_button) {
+    if (!wwin->window_flags.no_close_button) {
 	if (wwin->wm_gnustep_attr
 	    && wwin->wm_gnustep_attr->flags&GSClosePixmapAttr) {
-	    
 	    pixmap = wwin->wm_gnustep_attr->close_pixmap;
 	    
 	    if (wwin->wm_gnustep_attr->flags&GSCloseMaskAttr) {
@@ -1648,6 +1619,11 @@ wWindowUpdateButtonImages(WWindow *wwin)
 		fwin->rbutton_image->client_owned = 1;
 		fwin->rbutton_image->client_owned_mask = 1;
 	    }
+	} else if (wwin->window_flags.kill_close) {
+	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) {
+		wPixmapDestroy(fwin->rbutton_image);
+	    }
+	    fwin->rbutton_image = scr->b_pixmaps[WBUT_KILL];
 	} else if (wwin->window_flags.broken_close) {
 	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) {
 		wPixmapDestroy(fwin->rbutton_image);
@@ -1682,9 +1658,9 @@ wWindowConfigureBorders(WWindow *wwin)
 	int newy, oldh;
 	
 	flags = WFF_LEFT_BUTTON|WFF_RIGHT_BUTTON;
-	if (wwin->window_flags.titlebar)
+	if (!wwin->window_flags.no_titlebar)
 	    flags |= WFF_TITLEBAR;
-	if (wwin->window_flags.resizebar)
+	if (!wwin->window_flags.no_resizebar)
 	    flags |= WFF_RESIZEBAR;
 
 	oldh = wwin->frame->top_width;
@@ -1698,20 +1674,22 @@ wWindowConfigureBorders(WWindow *wwin)
 	}
 	
 	flags = 0;
-	if (wwin->window_flags.miniaturize_button
+	if (!wwin->window_flags.no_miniaturize_button
 	    && wwin->frame->flags.hide_left_button)
 	    flags |= WFF_LEFT_BUTTON;
-	if (wwin->window_flags.close_button
+	if (!wwin->window_flags.no_close_button
 	    && wwin->frame->flags.hide_right_button)
 	    flags |= WFF_RIGHT_BUTTON;
-	if (flags!=0)
+	if (flags!=0) {
+	    wWindowUpdateButtonImages(wwin);
 	    wFrameWindowShowButton(wwin->frame, flags);
+	}
 
 	flags = 0;
-	if (!wwin->window_flags.miniaturize_button
+	if (wwin->window_flags.no_miniaturize_button
 	    && !wwin->frame->flags.hide_left_button)
 	    flags |= WFF_LEFT_BUTTON;
-	if (!wwin->window_flags.close_button
+	if (wwin->window_flags.no_close_button
 	    && !wwin->frame->flags.hide_right_button)
 	    flags |= WFF_RIGHT_BUTTON;
 	if (flags!=0)
@@ -1770,14 +1748,14 @@ void wWindowSetShape(WWindow *wwin)
     XShapeCombineShape(dpy, wwin->frame->core->window, ShapeBounding,
 		       0, wwin->frame->top_width, wwin->client_win, 
 		       ShapeBounding, ShapeSet);
-    if (wwin->window_flags.titlebar) {
+    if (!wwin->window_flags.no_titlebar) {
 	rect[count].x = 0;
 	rect[count].y = 0;
 	rect[count].width = wwin->frame->core->width;
 	rect[count].height = wwin->frame->top_width;
 	count++;
     }
-    if (wwin->window_flags.resizebar) {
+    if (!wwin->window_flags.no_resizebar) {
 	rect[count].x = 0;
 	rect[count].y = wwin->frame->core->height - wwin->frame->bottom_width;
 	rect[count].width = wwin->frame->core->width;
@@ -1815,7 +1793,6 @@ getFocusMode(WWindow *wwin)
 }
 
 
-
 void
 wWindowSetKeyGrabs(WWindow *wwin)
 {
@@ -1828,34 +1805,57 @@ wWindowSetKeyGrabs(WWindow *wwin)
 	if (key->keycode==0) 
 	    continue;
 	if (key->modifier!=AnyModifier) {
-	    XGrabKey(dpy, key->keycode, key->modifier|IGNORE_MOD_MASK,
+	    XGrabKey(dpy, key->keycode, key->modifier|LockMask,
 		wwin->frame->core->window, True, GrabModeAsync, GrabModeAsync);
+#ifdef NUMLOCK_HACK
+	    /* Also grab all modifier combinations possible that include,
+	     * LockMask, ScrollLockMask and NumLockMask, so that keygrabs
+	     * work even if the NumLock/ScrollLock key is on.
+	     */
+	    wHackedGrabKey(key->keycode, key->modifier,
+			   wwin->frame->core->window, True, GrabModeAsync, 
+			   GrabModeAsync);
+#endif
 	}
 	XGrabKey(dpy, key->keycode, key->modifier,
 	     wwin->frame->core->window, True, GrabModeAsync, GrabModeAsync);
     }
-    
+
     wRootMenuBindShortcuts(wwin->frame->core->window);
 }
 
 
-static void
-setupGrabs(WWindow *wwin)
+
+void
+wWindowResetMouseGrabs(WWindow *wwin)
 {
-    /* setup passive grabs for the client window */
-    XGrabButton(dpy, AnyButton, AnyModifier, wwin->client_win,
-		True, ButtonPressMask, GrabModeSync, GrabModeAsync,
-		None, None);
+    XUngrabButton(dpy, AnyButton, AnyModifier, wwin->client_win);
+
+    if (!wwin->window_flags.no_bind_mouse) {
+	/* grabs for Meta+drag */
+	XGrabButton(dpy, AnyButton, MOD_MASK,
+		    wwin->client_win, True, ButtonPressMask, GrabModeSync,
+		    GrabModeAsync, None, wCursor[WCUR_ARROW]);
+	XGrabButton(dpy, AnyButton, MOD_MASK|LockMask,
+		    wwin->client_win, True, ButtonPressMask, GrabModeSync,
+		    GrabModeAsync, None, wCursor[WCUR_ARROW]);
+#ifdef NUMLOCK_HACK
+	wHackedGrabButton(AnyButton, MOD_MASK, wwin->client_win, 
+			  True, ButtonPressMask, GrabModeSync, 
+			  GrabModeAsync, None, wCursor[WCUR_ARROW]);
+#endif
+    }
+    
+    if (!wwin->flags.focused) {
+	/* the passive grabs to focus the window */
+	if (wPreferences.focus_mode == WKF_CLICK)
+	    XGrabButton(dpy, AnyButton, AnyModifier, wwin->client_win,
+			True, ButtonPressMask, GrabModeSync, GrabModeAsync,
+			None, None);
+    }
     XFlush(dpy);
 }
 
-static void
-removeGrabs(WWindow *wwin)
-{
-    /* remove passive grabs from the client window frame */
-    XUngrabButton(dpy, AnyButton, AnyModifier, wwin->client_win);
-    XFlush(dpy);
-}
 
 
 /* ====================================================================== */
@@ -1865,7 +1865,15 @@ resizebarMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 {
     WWindow *wwin = data;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+#ifndef NUMLOCK_HACK
+    if ((event->xbutton.state & VALID_MOD_MASK)
+	!= (event->xbutton.state & ~LockMask)) {
+	wwarning(_("the NumLock, ScrollLock or similar key seems to be turned on.\n"\
+		"Turn it off or some mouse actions and keyboard shortcuts will not work."));
+    }
+#endif
+    
+    event->xbutton.state &= VALID_MOD_MASK;
     
     CloseWindowMenu(wwin->screen_ptr);
 
@@ -1880,7 +1888,7 @@ resizebarMouseDown(WCoreWindow *sender, void *data, XEvent *event)
     if (event->xbutton.state & MOD_MASK) {
 	/* move the window */
 	if (XGrabPointer(dpy, wwin->frame->resizebar->window, True,
-			 ButtonMotionMask|ButtonReleaseMask,
+			 ButtonMotionMask|ButtonReleaseMask|ButtonPressMask,
 			 GrabModeAsync, GrabModeAsync, None, 
 			 None, CurrentTime)!=GrabSuccess) {
 #ifdef DEBUG0
@@ -1893,7 +1901,7 @@ resizebarMouseDown(WCoreWindow *sender, void *data, XEvent *event)
     } else {
         /* resize the window */
         if (XGrabPointer(dpy, wwin->frame->resizebar->window, False,
-                         ButtonMotionMask|ButtonReleaseMask,
+                         ButtonMotionMask|ButtonReleaseMask|ButtonPressMask,
                          GrabModeAsync, GrabModeAsync, None,
                          None, CurrentTime)!=GrabSuccess) {
 #ifdef DEBUG0
@@ -1913,11 +1921,11 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
 {
     WWindow *wwin = data;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+    event->xbutton.state &= VALID_MOD_MASK;
 
     if (event->xbutton.button==Button1) {
 	if (event->xbutton.state == 0) {
-	    if (wwin->window_flags.shadeable) {
+	    if (!wwin->window_flags.no_shadeable) {
 		/* shade window */
 		if (wwin->flags.shaded)
 		    wUnshadeWindow(wwin);
@@ -1937,7 +1945,7 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
             }
 
 	    /* maximize window */
-	    if (dir !=0 && wwin->window_flags.resizable) {
+	    if (dir !=0 && !wwin->window_flags.no_resizable) {
 		if (wwin->flags.maximized) 
 		    wUnmaximizeWindow(wwin);
 		 else
@@ -1959,7 +1967,7 @@ frameMouseDown(WObjDescriptor *desc, XEvent *event)
 {
     WWindow *wwin = desc->parent;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+    event->xbutton.state &= VALID_MOD_MASK;
     
     CloseWindowMenu(wwin->screen_ptr);
 
@@ -1974,7 +1982,7 @@ frameMouseDown(WObjDescriptor *desc, XEvent *event)
     if (event->xbutton.state & MOD_MASK) {
 	/* move the window */
 	if (XGrabPointer(dpy, wwin->client_win, False, 
-			 ButtonMotionMask|ButtonReleaseMask,
+			 ButtonMotionMask|ButtonReleaseMask|ButtonPressMask,
 			 GrabModeAsync, GrabModeAsync, None,
 			 None, CurrentTime)!=GrabSuccess) {
 #ifdef DEBUG0
@@ -1982,7 +1990,7 @@ frameMouseDown(WObjDescriptor *desc, XEvent *event)
 #endif
 	    return;
 	}
-	if (event->xbutton.button == Button3 && wwin->window_flags.resizable)
+	if (event->xbutton.button == Button3 && !wwin->window_flags.no_resizable)
 	    wMouseResizeWindow(wwin, event);
 	else 
 	    wMouseMoveWindow(wwin, event);
@@ -1996,7 +2004,16 @@ titlebarMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 {
     WWindow *wwin = (WWindow*)data;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+#ifndef NUMLOCK_HACK
+    if ((event->xbutton.state & VALID_MOD_MASK)
+	!= (event->xbutton.state & ~LockMask)) {
+	wwarning(_("the NumLock, ScrollLock or similar key seems to be turned on.\n"\
+		"Turn it off or some mouse actions and keyboard shortcuts will not work."));
+    }
+#endif
+    
+    event->xbutton.state &= VALID_MOD_MASK;
+    
 
     CloseWindowMenu(wwin->screen_ptr);
     
@@ -2021,7 +2038,7 @@ titlebarMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 	    return;
 	}
 	if (XGrabPointer(dpy, wwin->frame->titlebar->window, False, 
-			 ButtonMotionMask|ButtonReleaseMask,
+			 ButtonMotionMask|ButtonReleaseMask|ButtonPressMask,
 			 GrabModeAsync, GrabModeAsync, None, 
 			 None, CurrentTime)!=GrabSuccess) {
 #ifdef DEBUG0
@@ -2053,7 +2070,7 @@ windowCloseClick(WCoreWindow *sender, void *data, XEvent *event)
 {
     WWindow *wwin = data;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+    event->xbutton.state &= VALID_MOD_MASK;
 
     CloseWindowMenu(wwin->screen_ptr);
     
@@ -2088,7 +2105,7 @@ windowIconifyClick(WCoreWindow *sender, void *data, XEvent *event)
 {
     WWindow *wwin = data;
 
-    event->xbutton.state &= ~IGNORE_MOD_MASK;
+    event->xbutton.state &= VALID_MOD_MASK;
     
     CloseWindowMenu(wwin->screen_ptr);
 
@@ -2101,7 +2118,7 @@ windowIconifyClick(WCoreWindow *sender, void *data, XEvent *event)
 	    (event->xbutton.button == Button3)) {
 	    
 	    wapp = wApplicationOf(wwin->main_window);
-	    if (wapp && wwin->window_flags.application)
+	    if (wapp && !wwin->window_flags.no_appicon)
 		wHideApplication(wapp);
 	} else {
 	    wIconifyWindow(wwin);
