@@ -266,16 +266,27 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
         free(file);
 
         if (!path) {
-            wMessageDialog(panel->frame->screen_ptr, _("Error"),
-                           _("Could not find specified icon file"), WD_ERROR);
+	    char *buf;
+	    
+	    buf = wmalloc(strlen(file)+80);
+	    sprintf(buf, _("Could not find icon \"%s\" specified for this window"),
+		    file);
+            wMessageDialog(panel->frame->screen_ptr, _("Error"), buf, WD_ERROR);
+	    free(buf);
             return -1;
         }
         pixmap = WMCreatePixmapFromFile(scrPtr, path);
         free(path);
 
         if (!pixmap) {
-            wMessageDialog(panel->frame->screen_ptr, _("Error"),
-                           _("Could not open specified icon file"), WD_ERROR);
+	    char *buf;
+	    
+	    buf = wmalloc(strlen(file)+80);
+	    sprintf(buf, _("Could not open specified icon \"%s\":%s"),
+		    file, RErrorString);
+            wMessageDialog(panel->frame->screen_ptr, _("Error"), buf,
+			   WD_ERROR);
+	    free(buf);
             return -1;
         }
     }
@@ -495,12 +506,9 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     insertAttribute(dict, winDic, AKeepInsideScreen,  value, &different, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[4])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoAppIcon,         value, &different, flags);
-
-    value = (WMGetButtonSelected(panel->moreChk[5])!=0) ? Yes : No;
     insertAttribute(dict, winDic, AUnfocusable, value, &different, flags);
 
-    value = (WMGetButtonSelected(panel->moreChk[6])!=0) ? Yes : No;
+    value = (WMGetButtonSelected(panel->moreChk[5])!=0) ? Yes : No;
     insertAttribute(dict, winDic, ADontSaveSession,   value, &different, flags);
 
     /* application wide settings for when */
@@ -509,7 +517,9 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 	
 	value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
 	insertAttribute(dict, winDic, AStartHidden, value, &different, flags);
-	
+
+	value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
+	insertAttribute(dict, winDic, ANoAppIcon, value, &different, flags);
     } 
 
     PLRemoveDictionaryEntry(dict, key);
@@ -554,6 +564,9 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 	    value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
 	    insertAttribute(dict, appDic, AStartHidden,  value, &different, flags);
 
+	    value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
+	    insertAttribute(dict, appDic, ANoAppIcon,  value, &different, flags);
+
 	    PLRemoveDictionaryEntry(dict, key);
 	    if (different) {
 		PLInsertDictionaryEntry(dict, key, appDic);
@@ -578,7 +591,7 @@ makeAppIconFor(WApplication *wapp)
     if (wapp->app_icon)
         return;
 
-    if (wapp->main_window_desc->window_flags.application)
+    if (!wapp->main_window_desc->window_flags.no_appicon)
         wapp->app_icon = wAppIconCreate(wapp->main_window_desc);
     else
         wapp->app_icon = NULL;
@@ -610,7 +623,7 @@ makeAppIconFor(WApplication *wapp)
 	    XMapWindow(dpy, icon->core->window);
 
         if (wPreferences.auto_arrange_icons && !wapp->app_icon->attracted)
-            wArrangeIcons(wapp->main_window_desc->screen_ptr);
+            wArrangeIcons(wapp->main_window_desc->screen_ptr, True);
     }
 }
 
@@ -641,7 +654,7 @@ removeAppIconFor(WApplication *wapp)
     }
     wapp->app_icon = NULL;
     if (wPreferences.auto_arrange_icons)
-        wArrangeIcons(wapp->main_window_desc->screen_ptr);
+        wArrangeIcons(wapp->main_window_desc->screen_ptr, True);
 }
 
 
@@ -650,31 +663,31 @@ applySettings(WMButton *button, InspectorPanel *panel)
 {
     WWindow *wwin = panel->inspected;
     WWindowAttributes *wflags = &wwin->window_flags;
+    WWindowAttributes oldFlags = *wflags;
     WApplication *wapp = wApplicationOf(wwin->main_window);
     int floating, skip_window_list;
 
     showIconFor(WMWidgetScreen(button), panel, NULL, NULL, USE_TEXT_FIELD);
 
-    wflags->titlebar           = !WMGetButtonSelected(panel->attrChk[0]);
-    wflags->resizebar          = !WMGetButtonSelected(panel->attrChk[1]);
-    wflags->close_button       = !WMGetButtonSelected(panel->attrChk[2]);
-    wflags->miniaturize_button = !WMGetButtonSelected(panel->attrChk[3]);
+    wflags->no_titlebar        =  WMGetButtonSelected(panel->attrChk[0]);
+    wflags->no_resizebar       =  WMGetButtonSelected(panel->attrChk[1]);
+    wflags->no_close_button    =  WMGetButtonSelected(panel->attrChk[2]);
+    wflags->no_miniaturize_button = WMGetButtonSelected(panel->attrChk[3]);
     floating                   =  WMGetButtonSelected(panel->attrChk[4]);
     wflags->omnipresent        =  WMGetButtonSelected(panel->attrChk[5]);
     wflags->start_miniaturized =  WMGetButtonSelected(panel->attrChk[6]);
     skip_window_list           =  WMGetButtonSelected(panel->attrChk[7]);
-    wflags->hide_others        = !WMGetButtonSelected(panel->moreChk[0]);
-    wflags->bind_keys          = !WMGetButtonSelected(panel->moreChk[1]);
-    wflags->bind_mouse         = !WMGetButtonSelected(panel->moreChk[2]);
+    wflags->no_hide_others     =  WMGetButtonSelected(panel->moreChk[0]);
+    wflags->no_bind_keys       =  WMGetButtonSelected(panel->moreChk[1]);
+    wflags->no_bind_mouse      =  WMGetButtonSelected(panel->moreChk[2]);
     wflags->dont_move_off      =  WMGetButtonSelected(panel->moreChk[3]);
-    wflags->application        = !WMGetButtonSelected(panel->moreChk[4]);
-    wflags->focusable          = !WMGetButtonSelected(panel->moreChk[5]);
-    wflags->dont_save_session  =  WMGetButtonSelected(panel->moreChk[6]);
+    wflags->no_focusable       =  WMGetButtonSelected(panel->moreChk[4]);
+    wflags->dont_save_session  =  WMGetButtonSelected(panel->moreChk[5]);
     wflags->always_user_icon   =  WMGetButtonSelected(panel->alwChk);
     
-    if (!wflags->titlebar && wwin->flags.shaded)
+    if (wflags->no_titlebar && wwin->flags.shaded)
         wUnshadeWindow(wwin);
-    wflags->shadeable = wflags->titlebar;
+    wflags->no_shadeable = wflags->no_titlebar;
 
     if (wflags->floating != floating) {
         int wlevel = ((wflags->floating = floating))
@@ -687,17 +700,34 @@ applySettings(WMButton *button, InspectorPanel *panel)
             ? ACTION_REMOVE : ACTION_ADD;
         UpdateSwitchMenu(wwin->screen_ptr, wwin, action);
     }
+    
+    if (wflags->no_bind_keys != oldFlags.no_bind_keys) {
+	if (!wflags->no_bind_keys) {
+	    XUngrabKey(dpy, AnyKey, AnyModifier, wwin->frame->core->window);
+	} else {
+	    wWindowSetKeyGrabs(wwin);
+	}
+    }
+    
+    if (wflags->no_bind_mouse != oldFlags.no_bind_mouse) {
+	wWindowResetMouseGrabs(wwin);
+    }
 
     wwin->frame->flags.need_texture_change = 1;
     wWindowConfigureBorders(wwin);
     wFrameWindowPaint(wwin->frame);
     
     if (wapp) {
+	/* do application wide stuff */
+	wflags = &wapp->main_window_desc->window_flags;
 
-        if (wflags->application)
-            makeAppIconFor(wapp);
-        else
+	wflags->start_hidden =  WMGetButtonSelected(panel->appChk[0]);
+	wflags->no_appicon  =  WMGetButtonSelected(panel->appChk[1]);
+	
+        if (wflags->no_appicon)
             removeAppIconFor(wapp);
+	else
+            makeAppIconFor(wapp);
 
         if (wapp->app_icon && wapp->main_window == wwin->client_win) {
             char *file = WMGetTextFieldText(panel->fileText);
@@ -711,11 +741,6 @@ applySettings(WMButton *button, InspectorPanel *panel)
 		free(file);
 	    wAppIconPaint(wapp->app_icon);
         }
-
-	/* do application wide stuff */
-	wflags = &wapp->main_window_desc->window_flags;
-
-	wflags->start_hidden =  WMGetButtonSelected(panel->appChk[0]);
     }
 }
 
@@ -738,41 +763,39 @@ revertSettings(WMButton *button, InspectorPanel *panel)
         wm_instance = wwin->wm_instance;
         wm_class = wwin->wm_class;
     }
-
-    memset(&wwin->window_flags, 0xff, sizeof(WWindowAttributes));
+    memset(&wwin->window_flags, 0, sizeof(WWindowAttributes));
     wDefaultFillAttributes(wwin->screen_ptr, wm_instance, wm_class,
                            &wwin->window_flags, True);
-
-    wwin->window_flags.broken_close = (wwin->protocols.DELETE_WINDOW) ? 0 : 1;
+    wwin->window_flags.kill_close = (wwin->protocols.DELETE_WINDOW) ? 0 : 1;
     /* transients can't be iconified or maximized */
-    if (!wwin->window_flags.miniaturizable) {
-	wwin->window_flags.miniaturize_button = 0;
+    if (wwin->window_flags.no_miniaturizable) {
+	wwin->window_flags.no_miniaturize_button = 1;
     }
     /* if the window can't be resized, remove the resizebar */
-    if (!wwin->window_flags.resizable) {
-	wwin->window_flags.resizebar = 0;
+    if (wwin->window_flags.no_resizable) {
+	wwin->window_flags.no_resizebar = 1;
     }
 
 /*    wwin->window_flags.shadeable = old_shadeable;*/
-    if (!wwin->window_flags.titlebar && wwin->flags.shaded)
+    if (wwin->window_flags.no_titlebar && wwin->flags.shaded)
         wUnshadeWindow(wwin);
-    wwin->window_flags.shadeable = wwin->window_flags.titlebar;
+    wwin->window_flags.no_shadeable = wwin->window_flags.no_titlebar;
 
     for (i=0; i < 8; i++) {
 	int flag = 0;
 	
 	switch (i) {
 	 case 0:
-            flag = !wwin->window_flags.titlebar;
+            flag = wwin->window_flags.no_titlebar;
 	    break;
 	 case 1:
-	    flag = !wwin->window_flags.resizebar;
+	    flag = wwin->window_flags.no_resizebar;
 	    break;
 	 case 2:
-	    flag = !wwin->window_flags.close_button;
+	    flag = wwin->window_flags.no_close_button;
 	    break;
 	 case 3:
-	    flag = !wwin->window_flags.miniaturize_button;
+	    flag = wwin->window_flags.no_miniaturize_button;
 	    break;
 	 case 4:
             floating = WMGetButtonSelected(panel->attrChk[4]);
@@ -787,7 +810,7 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	    flag = wwin->window_flags.omnipresent;
 	    break;
 	 case 6:
-	    flag = !wwin->window_flags.focusable;
+	    flag = wwin->window_flags.no_focusable;
 	    break;
 	 case 7:
             skip_window_list = WMGetButtonSelected(panel->attrChk[7]);
@@ -801,41 +824,41 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	}
         WMSetButtonSelected(panel->attrChk[i], flag);
     }
-    for (i=0; i < 7; i++) {
+    for (i=0; i < 6; i++) {
 	int flag = 0;
 	
 	switch (i) {
 	 case 0:
-	    flag = !wwin->window_flags.hide_others;
+	    flag = wwin->window_flags.no_hide_others;
 	    break;
 	 case 1:
-	    flag = !wwin->window_flags.bind_keys;
+	    flag = wwin->window_flags.no_bind_keys;
 	    break;
 	 case 2:
-	    flag = !wwin->window_flags.bind_mouse;
+	    flag = wwin->window_flags.no_bind_mouse;
 	    break;
 	 case 3:
 	    flag = wwin->window_flags.dont_move_off;
 	    break;
 	 case 4:
-	    flag = !wwin->window_flags.application;
-	    break;
-	 case 5:
 	    flag = wwin->window_flags.start_miniaturized;
 	    break;
-	 case 6:
+	 case 5:
 	    flag = wwin->window_flags.dont_save_session;
 	    break;
 	}
 	WMSetButtonSelected(panel->moreChk[i], flag);
     }
-    if (panel->appFrm) {
-	for (i=0; i < 1; i++) {
+    if (panel->appFrm && wapp) {
+	for (i=0; i < 2; i++) {
 	    int flag = 0;
 	    
 	    switch (i) {
 	     case 0:
-		flag = wwin->window_flags.start_hidden;
+		flag = wapp->main_window_desc->window_flags.start_hidden;
+		break;
+	     case 1:
+		flag = wapp->main_window_desc->window_flags.no_appicon;
 		break;
 	    }
 	    WMSetButtonSelected(panel->appChk[i], flag);
@@ -860,10 +883,10 @@ revertSettings(WMButton *button, InspectorPanel *panel)
     }
 
     if (wapp) {
-        if (wwin->window_flags.application)
-            makeAppIconFor(wapp);
-        else
+        if (wapp->main_window_desc->window_flags.no_appicon)
             removeAppIconFor(wapp);
+	else
+            makeAppIconFor(wapp);
 
         /* same hack */
 	/* this hack does not work for apps with more than 1 window.
@@ -891,7 +914,7 @@ chooseIconCallback(WMWidget *self, void *clientData)
 
     panel->choosingIcon = 1;
     
-    WMSetButtonDisabled(panel->browseIconBtn, True);
+    WMSetButtonEnabled(panel->browseIconBtn, False);
     
     result = wIconChooserDialog(panel->frame->screen_ptr, &file);
     
@@ -904,7 +927,7 @@ chooseIconCallback(WMWidget *self, void *clientData)
 			USE_TEXT_FIELD);
 	    free(file);
 	}
-	WMSetButtonDisabled(panel->browseIconBtn, False);
+	WMSetButtonEnabled(panel->browseIconBtn, True);
     } else {
 	freeInspector(panel);
     }
@@ -921,11 +944,7 @@ createInspectorForWindow(WWindow *wwin)
     int i;
     int x, y;
 
-    panel = malloc(sizeof(InspectorPanel));
-    if (!panel) {
-	wwarning(_("not enough memory to open window inspector panel"));
-	return NULL;
-    }
+    panel = wmalloc(sizeof(InspectorPanel));
     
     panel->destroyed = 0;
 
@@ -1041,19 +1060,19 @@ createInspectorForWindow(WWindow *wwin)
 	switch (i) {
 	 case 0:
 	    caption = _("Disable titlebar");
-            flag = !wwin->window_flags.titlebar;
+            flag = wwin->window_flags.no_titlebar;
 	    break;
 	 case 1:
 	    caption = _("Disable resizebar");
-	    flag = !wwin->window_flags.resizebar;
+	    flag = wwin->window_flags.no_resizebar;
 	    break;
 	 case 2:
 	    caption = _("Disable close button");
-	    flag = !wwin->window_flags.close_button;
+	    flag = wwin->window_flags.no_close_button;
 	    break;
 	 case 3:
 	    caption = _("Disable miniaturize button");
-	    flag = !wwin->window_flags.miniaturize_button;
+	    flag = wwin->window_flags.no_miniaturize_button;
 	    break;
 	 case 4:
 	    caption = _("Keep on top");
@@ -1086,36 +1105,32 @@ createInspectorForWindow(WWindow *wwin)
     WMMoveWidget(panel->moreFrm, 20, 50);
     WMResizeWidget(panel->moreFrm, 220, 230);
 
-    for (i=0; i < 7; i++) {
+    for (i=0; i < 6; i++) {
 	char *caption = NULL;
 	int flag = 0;
 	
 	switch (i) {
 	 case 0:
 	    caption = _("Ignore HideOthers");
-	    flag = !wwin->window_flags.hide_others;
+	    flag = wwin->window_flags.no_hide_others;
 	    break;
 	 case 1:
 	    caption = _("Don't bind keyboard shortcuts");
-	    flag = !wwin->window_flags.bind_keys;
+	    flag = wwin->window_flags.no_bind_keys;
 	    break;
 	 case 2:
 	    caption = _("Don't bind mouse clicks");
-	    flag = !wwin->window_flags.bind_mouse;
+	    flag = wwin->window_flags.no_bind_mouse;
 	    break;
 	 case 3:
 	    caption = _("Keep inside screen");
 	    flag = wwin->window_flags.dont_move_off;
 	    break;
 	 case 4:
-	    caption = _("No application icon");
-	    flag = !wwin->window_flags.application;
+	    caption = _("Don't let it take focus");
+	    flag = wwin->window_flags.no_focusable;
 	    break;
 	 case 5:
-	    caption = _("Don't let it take focus");
-	    flag = !wwin->window_flags.focusable;
-	    break;
-	 case 6:
 	    caption = _("Don't Save Session");
 	    flag = wwin->window_flags.dont_save_session;
 	    break;
@@ -1218,7 +1233,7 @@ createInspectorForWindow(WWindow *wwin)
 	WMMoveWidget(panel->appFrm, 20, 50);
 	WMResizeWidget(panel->appFrm, 220, 230);
 	
-	for (i=0; i < 1; i++) {
+	for (i=0; i < 2; i++) {
 	    char *caption = NULL;
 	    int flag = 0;
 	    
@@ -1227,10 +1242,14 @@ createInspectorForWindow(WWindow *wwin)
 		caption = _("Start Hidden");
 		flag = wapp->main_window_desc->window_flags.start_hidden;
 		break;
+	     case 1:
+		caption = _("No application icon");
+		flag = wapp->main_window_desc->window_flags.no_appicon;
+		break;
 	    }
 	    panel->appChk[i] = WMCreateSwitchButton(panel->appFrm);
-	    WMMoveWidget(panel->appChk[i], 5, 20*(i+1));
-	    WMResizeWidget(panel->appChk[i], 205, 20);
+	    WMMoveWidget(panel->appChk[i], 10, 20*(i+1));
+	    WMResizeWidget(panel->appChk[i], 200, 20);
 	    WMSetButtonSelected(panel->appChk[i], flag);
 	    WMSetButtonText(panel->appChk[i], caption);
 	}
@@ -1243,20 +1262,24 @@ createInspectorForWindow(WWindow *wwin)
     
     parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, PWIDTH, PHEIGHT, 
 				 0, 0, 0);
+    XSelectInput(dpy, parent, KeyPressMask|KeyReleaseMask);
     panel->parent = parent;
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
-
+    
     XSetTransientForHint(dpy, parent, wwin->client_win);
 
     x = wwin->frame_x+wwin->frame->core->width/2;
     y = wwin->frame_y+wwin->frame->top_width*2;
     if (y + PHEIGHT > scr->scr_height)
-	y = scr->scr_height - PHEIGHT;
+	y = scr->scr_height - PHEIGHT - 30;
     panel->frame = wManageInternalWindow(scr, parent, wwin->client_win, 
 					 charbuf, x, y, PWIDTH, PHEIGHT);
     
-    panel->frame->window_flags.closable = 1;
-    panel->frame->window_flags.close_button = 1;
+    /* kluge to know who should get the key events */
+    panel->frame->client_leader = WMWidgetXID(panel->win);
+    
+    panel->frame->window_flags.no_closable = 0;
+    panel->frame->window_flags.no_close_button = 0;
     wWindowUpdateButtonImages(panel->frame);
     wFrameWindowShowButton(panel->frame->frame, WFF_RIGHT_BUTTON);
     panel->frame->frame->on_click_right = destroyInspector;
