@@ -36,7 +36,8 @@ void wWorkspaceDeleteLast(WScreen *scr);
 void wWorkspaceChange(WScreen *scr, int workspace);
 void wWorkspaceForceChange(WScreen *scr, int workspace);
 
-WMenu *wWorkspaceMenuMake(WScreen *scr);
+
+WMenu *wWorkspaceMenuMake(WScreen *scr, Bool titled);
 void wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu);
 
 void wWorkspaceMenuEdit(WScreen *scr);
